/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*******************************************************************************
***      File       :   DCMI_Conformance.cpp                                  **
***                                                                           **
***      Abstract   :   The main module that displays the user menu,          **
***                     reads the configuration file, establishes             **
***                     the connection via the transport chosen by            **
***                     User. It calls the appropriate tests to run           **
***                     the test procedures to validate the Platform          **
***                     for DCMI Compliance.                                  **
***                                                                           **
***      Contents   :                                                         **
***                                                                           **
***             Main of DCMI Conformance Module.                              **
***                                                                           **
***             Definition of following local functions                       **
***             DisplayHeader                                                 **
***             MenuCall                                                      **
***             VersionMenu                                                   **
***             checkParams                                                   **
***             printUsage                                                    **
***             CreateTestSuite                                               **
***                                                                           **
********************************************************************************/
/*
 * DCMI_Conformance.cpp
 *
 *  Created on: 28-03-2011
 */

#include <fstream>
#ifdef WIN32
#include <windows.h>
#endif

#include "parameters.h"

#include "version.h"
#include "CommonClasses.hpp"
#include "DCMI_Configuration.h"
#include "TransportManager.h"

#include "DCMI1_5ConformanceSuit.h"

void DisplayHeader(std::string &version);
void MenuCall(UserOption &);
void VersionMenu(std::string &);
bool checkParams(int argc, const char *argv[], DcmiConfiguration &config, UserOption &option, std::string &version);
void printUsage();

std::auto_ptr<TestUtil::TestSuite> CreateTestSuite(UserOption option, std::string &verson) /*throw(...)*/;


using TransportPlugin::TransportManager;

/***********************************************************************************/
/***********************************************************************************/
/**
 @fn         main
 @brief      Main control.
 @param      command line parameters :

 @return     error code
*/
int main(int argc, const char *argv[])
{
    std::ofstream logFile;
	DcmiConfiguration config;

	//early configuration loading and logger initialization ...
	// Load DCMI configuration from file.
	if (config.loadConfiguration())
	{
		std::cout<<"Failed to load Configuration"<<std::endl;
		//		WaitKeyPressed();
		return E_CONFIG_LOAD;
	}

	// Initialize logger.
	if (config.InitializeLogger(&logFile))
	{
		std::cout<<"Failed to open Logger File"<<std::endl;
		//		WaitKeyPressed();
		return E_INIT_LOGGER;
	}

	UserOption option = U_INVALID;
	//default version indicator ...
	std::string version;
	if (checkParams(argc, argv, config, option, version))
	{
		printUsage();
		return SUCCESS;
	}

	DisplayHeader(version);
	/*Call the main menu to allow validator to select test options
	 *Assumed they will have read the DCMI Test Users guide and have
	 *pre-configured the system and have the appropriate set-up.
	 */
	VersionMenu(version);
	if (!version.compare("-"))
	{
		LOG_SUMMARY<<"\nOption not supported. Exiting Conformance Test Suite\n";
		return E_INVALID_OPTION;
	}
	if(!version.size())
	{	//Exit Test option chosen - proceed to close interfaces and exit test
		LOG_SUMMARY<<"\nUser has opted to exit the test. Exiting Conformance Test Suite\n";
		//             DelayTime(2);
		return SUCCESS;
	}

	MenuCall(option);
	//switch for error...
	//if any other error options occur add only case phrase...
	switch (option)
	{
	case U_EXIT :
		//Exit Test option chosen - proceed to close interfaces and exit test
		LOG_SUMMARY<<"\nUser has opted to exit the test. Exiting Conformance Test Suite\n";
		//             DelayTime(2);
		return SUCCESS;

	case U_INVALID:
		//Invalid Option Chosen
		LOG_SUMMARY<<"\nOption not supported. Exiting Conformance Test Suite\n";
		//               DelayTime(2);
		return E_INVALID_OPTION;
	}//switch for exit ..


    //User input look OK so
 	 //configure and run.. test...
    int appret = SUCCESS;
	//create output buffer for combined summary, output summary stream
    std::stringstream oss(std::ostringstream::out);
    TestUtil::TestOutputText out(oss);
    try
	{	//Initialize the respective interface as per selection of user
		TransportManager::doConfigure(option, config);
		//Connection via configured transport ....
		TransportManager::connect();

		//initiate conformance test suit...
		//-->Run Test suit
		std::auto_ptr<TestUtil::TestSuite> suit (CreateTestSuite(option, version));

		suit->doConfigure(config);
		suit->runSuite(out);
		//<--
	}
    catch (std::exception &e)
	{
		LOG_ERR<<e.what()<<std::endl;
        appret = E_FAILURE;
	}

	//print summary
	LOG_SUMMARY<<"\n\n"<<oss.str()<<std::endl;

	TransportManager::disconnect();
	return appret;
}


/**************************************************************************/

/**************************************************************************
Function declaration

void DisplayHeader (void)
Function Name   :   DisplayHeader
Purpose         :   Displays the Header, Tool & Spec Version information to
                    Screen,File
Returns         :   void
***************************************************************************/
void DisplayHeader(std::string &version)
{
    LOG_SUMMARY << "DCMI Conformance Test Suite application started.\n\n";
    LOG_SUMMARY << "===============================================================\n";
    LOG_SUMMARY << "\t\t   DCMI Conformance Test Suite!\n\n";
    LOG_SUMMARY << "\t\t   **  Tool Revision "<<TOOL_REV<<" **\n\n";
//    LOG_SUMMARY << "\t\t **** Multi Version : "<<SPEC_REV/*version*/<<" ****\n";
    LOG_SUMMARY << "===============================================================\n\n\n";
}


void VersionMenu(std::string &version)
{
    LOG_SUMMARY<<"Enter a number and <Enter> to chose a specification:\n\n";
    LOG_SUMMARY<<" 1.1     - ............................................1\n";
    LOG_SUMMARY<<" 1.5     - ............................................2\n\n";
	LOG_SUMMARY<<"EXIT Test .............................................0\n\n";
	LOG_SUMMARY<<"-> ";
	if(version.size()>0)
	{
		if(version.compare("1.1")==0) {
			LOG_SUMMARY <<"1\n";
		}
		if(version.compare("1.5")==0) {
			LOG_SUMMARY <<"2\n";
		}
	} else {
		int sel = -1;
		std::cin >> sel;
		//added basic user input verification
		switch(sel) {
		case 0:
			version.clear();
			break;
		case 1:
			version="1.1";
			break;
		case 2:
			version="1.5";
			break;
		default:
			version="-";
			break;
		}
	}
}

/*************************************************************************
Function declaration
void MenuCall (void);
Function Name:		MenuCall
Purpose:			Displays the DCMI Conformance Test Options the user can
					 select from
Parameter:			[OUT] selection - The user selected option
Returns:			None
***************************************************************************/
void MenuCall(UserOption &option)
{
    LOG_SUMMARY<<"Enter a number and <Enter> to start the test required:\n\n";
    LOG_SUMMARY<<"RMCP+    - OOB Test....................................1\n";
    LOG_SUMMARY<<"KCS/HECI - InBand Test (Run on UUT only)...............2\n";
//#ifdef CIPHER_AUTOCHECK
    LOG_SUMMARY<<"Validate RMCP+ CipherSuites Supported on Platform OOB..3\n\n";
//#endif

    LOG_SUMMARY<<"EXIT Test .............................................0\n\n";
    LOG_SUMMARY<<"-> ";

    if (option == U_INVALID) {
    	int sel = U_EXIT;
    	std::cin >> sel;

		//added basic user input verification
		if ((sel < U_EXIT) || (sel > CIPHER_CHECK)) {
			sel = U_INVALID;
		}
		option = static_cast<UserOption>(sel);
    } else {
    	switch(option) {
    	case OOB_RMCPP:
    		LOG_SUMMARY<<"1\n";
    		break;
    	case IB_KCS_HECI:
    		LOG_SUMMARY<<"2\n";
    		break;
    	case CIPHER_CHECK:
    		LOG_SUMMARY<<"3\n";
    		break;
    	default:
    		LOG_SUMMARY<<"0\n";
    		break;
    	}
    }
}

/**
 * Factory method ... returns specific implementation instance
 * for requested test suit version or cipher test
 *
 * @param option -  user option cipher check or dcmi spec
 * @param version - version to test 1.1 or 1.5
 *
 * @return specific conformance test suit class
 */
std::auto_ptr<TestUtil::TestSuite> CreateTestSuite(UserOption option, std::string &version)
{
   if (option != CIPHER_CHECK) {
	   if(version.compare("1.1")==0) {
		   return std::auto_ptr<TestUtil::TestSuite>(new DCMIConformance::DCMI1_1ConformanceSuite());
	   }
	   if(version.compare("1.5")==0) {
		   return std::auto_ptr<TestUtil::TestSuite>(new DCMIConformance::DCMI1_5ConformanceSuite());
	   }
   }
   if (option == CIPHER_CHECK) {
	   int ver = ((0x1<<8) + (0x5));
	   if(version.compare("1.1")==0) {
		   ver = ((0x1<<8) + (0x1));
	   }
	   return std::auto_ptr<TestUtil::TestSuite>(new DCMIConformance::DCMICipherCheck(ver));
   }

   throw std::invalid_argument("Incorrect version parameter - Unknown version for test suit");
}


/*************************************************************************
Function declaration
void  sleep_printdots (int);
Function Name:		sleep_printdots
Purpose:			Delays the program for the time sent in seconds.

					prints some progress
Parameter:			[IN] time -> which is the time to delay in seconds.
Returns:			none
*/
void sleep_printdots(byte_t time)
{
	LOG_INFO<<"\n\tDelay for: "<<std::dec<<static_cast<int>(time)<<"s ";
	for(int i=0; i<time; i++ )
	{
/**
 * For compatibility win<->linux
 */
#ifdef WIN32
		Sleep(1000);
#else
		sleep(1);
#endif
		LOG_INFO<<".";
	}
}

void msleep( unsigned int msec)
{
#ifdef WIN32
		Sleep(msec);
#else
		usleep(msec*1000);
#endif
}

void strtotimeout(const char *src, int &dest)
{
	if(src==NULL)
		return ;

	//Convert the string to an integer
	int val = atoi(src);
	//Make sure delay time is < 50 seconds and > 0
	if (val > 0 && val < 50)
	{
		dest = val;
	}
}


bool checkParams(int argc, const char *argv[], DcmiConfiguration &config, UserOption &option, std::string &version)
{
	char *val = const_cast<char*>(config.getConfigurationValue("DCMI_Conformance", "version"));
	std::string ver;
	if(val != NULL)
	{
		ver = val;
	}

	for (int i = 1; i < argc; i++)
	{
		std::string tmp = argv[i];
		if (tmp.size()<2)  {
			LOG_ERR<<"\nUnrecognized parameter: '"<<tmp<<"'";
			continue ;
		}

		char c = tmp.at(0);
		if (c == '/' || c == '-')
			tmp.erase(tmp.begin());

		c = toupper(tmp.at(0));
		if (c =='H' || c =='?')
		{  //break ...
			return true;
		}
		if (c =='C')
		{
			option = CIPHER_CHECK;
			continue;
		}
		if (c =='R')
		{
			option = OOB_RMCPP;
			continue;
		}
		if (c == 'I')
		{
			option = IB_KCS_HECI;
			continue;
		}
		if (c == 'V')
		{
			if (++i < argc)
			{
				tmp = argv[i];
				if (tmp.size()>0 && tmp.size()<4)
				{
					ver = tmp;
				}
			}
			continue;
		}
		LOG_ERR<<"\nUnrecognized parameter: '"<<argv[i]<<"'";
	}//for parameters ....

	if (ver.size() != 0)
	{
		if (ver.compare("1.1") == 0 || ver.compare("1.5") == 0)
			version = ver;
		else {
			LOG_SUMMARY<<"\nUnsupported argument for version parameter: '"<<ver<<"'";
		}
	}

	return false;
}


void printUsage()
{
   LOG_SUMMARY<<"\n\n   DCTS Conformance suit\n"
		     <<"\tCommand Line Options:: ...\n"
			<< "\t\t[-?, -h]:\t\tPrints this Help\n"
			<< "\t\t[-c]:    \t\tChipher check test\n"
			<< "\t\t[-r]:    \t\tRMCP+ test\n"
			<< "\t\t[-i]:    \t\tInband test\n"
			<< "\t\t[-v] [1.1 or 1.5]:\tSpecification version to try\n"
			<< "\t\t\t\tdefault is 1.5\n"
			<< "\t\t\t\tcan be specified in userconfig.cfg file.\n";
}
