/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2011 INTEL CORPORATION                       **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/

/*
 * DCMIClasses.hpp
 *
 *  Created on: 01-04-2011
 */

#ifndef DCMICLASSES_HPP_
#define DCMICLASSES_HPP_

#include "CommonClasses.hpp"
#include "DCMI_Configuration.h"
#include "TestClasses.h"
#include "TransportCommon.h"

#include <iomanip>

//Define the Response length expected as in IPMI/DCMI spec
#define SET_PRIV_MIN_RSPLEN		1	//Min response length for Set Session Privilege level command
#define SET_PRIV_MAX_RSPLEN		1	//Max response length for Set Session Privilege level command

//Define the Response length expected as in IPMI/DCMI spec
#define GET_DEVID_MIN_RSPLEN	11	//Minimum response length for GetDeviceID command
#define GET_DEVID_MAX_RSPLEN	15	//Maximum response length for GetDeviceID command

using TestUtil::TestResult;
using TransportPlugin::LogLevel;

namespace DCMIClasses
{

const static byte_t DCMI_EXTID	= 0xDC;

#if !defined(ARRAYSIZE)
#define ARRAYSIZE(array) (sizeof(array)/sizeof(array[0]))
#endif


/*
 * data content
 * 0	Intel manufacturers ID - 0x000157, LS byte first
 * 3	Parameter Revision = 01h
 * 4:N  Parameter data ...
 */

/**
 * This class wraps data structure used to pass IPMI DCMI request to the transport
 * plug-in to the main application.
 * Basically request is stored in the raw format, exactly as defined in IPMI v2.0 specification.
 * Data request format:
 *
 * data content
 * 0	DCMI_Extension Id
 * 1:N  Parameter data ...
 */
class IpmiDCMIRequest : public CommonClasses::IpmiRequest
{

public:

	IpmiDCMIRequest(byte_t _cmd=0x00) :
		IpmiRequest(DCMI, _cmd, DCMI_EXTID)
	{
	}

	IpmiDCMIRequest(byte_t _cmd, byte_t _param) :
		IpmiRequest(DCMI, _cmd, DCMI_EXTID)
	{
		//add param to ipmi data request
		append(_param);
	}

	IpmiDCMIRequest(byte_t _cmd, const byte_t _param[], int _len) :
		IpmiRequest(DCMI, _cmd, DCMI_EXTID)
	{
		append(_param, _len);
	}

	virtual void clear()
	{
      IpmiRequest::clear();
      len += 1;
	}

};//IpmiDcmirequest

/**
 * This class wraps data structure used to get IPMI DCMI response from the transport
 * plug-in to the main application.
 * Basically response is stored in the raw format, exactly as defined in IPMI v2.0 specification.
 * Data response format:
 *
 * data content
 * 1 Completion Code. Refer to section 8, DCMI Completion Codes.
 * 2 Group Extension Identification = DCh
 * ....
 */
class IpmiDCMIResponse : public CommonClasses::IpmiResponse
{

public:

	IpmiDCMIResponse()
	{}

	IpmiDCMIResponse(size_t min_size, size_t max_size):
		IpmiResponse(min_size,max_size, 0x00)
	{}

	IpmiDCMIResponse(size_t min_size, size_t max_size, const byte_t ccode[], int cclen):
		IpmiResponse(min_size,max_size, ccode, cclen)
	{}


	/**************************************************************************
	method declaration
	bool isDCMIExtId (  );
	Function Name:		isDCMIExtId
	Purpose:			Checks to see that the DCMI Extension ID matches value

	 Returns:		true - response Extension ID matches DCMI_EXT_ID
						false- error, extension ID does not match DCMI_EXT_ID
	***************************************************************************/
	inline bool isDCMIExtId()
	{
		return (at(0) == DCMI_EXTID);
	}

	inline byte_t getDCMIExtId()
	{
		return at(0);
	}

	virtual bool isValidResponse()
	{
		return (IpmiResponse::isValidResponse() && isDCMIExtId());
	}

	//void print(std::ostream & stream);

}; //IpmiDCMIResponse


//std::_Smanip<IpmiDCMIResponse &> DCMIExtIdInfo(IpmiDCMIResponse &response);


/**
 * Base DCMI test package, provide some common method
 * bound with transport
 */
class DCMIBaseTest : public TestUtil::TestSuite
{
	TransportPlugin::TransportPlugIn &m_Transport;

public:

/*
	void doConfigure(DcmiConfiguration &options)
	{
		TestSuit::doConfigure(options);
	};
*/

	/* Tests Section */
	TestResult DeviceIdTest(any_value &)
	/*throw(...)*/;

	/* DCTS Utility Section */
	inline TransportPlugin::TransportPlugIn &getCurrentTransport() const { return m_Transport; };

	/**
	 * Send packet to the transport and make basic response validation,
	 * Critical errors are raised by exception - TestResult enum
	 *
	 * @param req - Request Data
	 * @param resp - Response storage class, and basic validation methods
	 * @param retryCount - retry count
	 */
	void sendIpmiCmd(CommonClasses::Request &req, CommonClasses::Response &resp,
      LogLevel level = TransportPlugin::logLevelDetails, byte_t retryCount = TransportPlugin::MAX_RETRY_COUNT)
	/*throw(...)*/;

	int setSessionPrivilegeLevel(PrivelegeLevel level)
	throw();

	bool isRemoteConnection() {
		return (m_Transport.connectionType() == TP_RMCPP);
	}

protected:

	/* Test suit section, do not allow to instance it straightly */
	DCMIBaseTest (std::string name, TransportPlugin::TransportPlugIn &transport);

};

}//namespace DCMIClasses

#endif /* DCMICLASSES_HPP_ */
