/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/

#ifndef _DCMI_CONFIGURATION_H
#define _DCMI_CONFIGURATION_H

#include <utility>
#include <iostream>
#include <string>
#include <set>
#include <map>

//Forward declarations
struct transport_plugin_interface;

/**
 * Manages DCMI configuration:
 * - reads configuration values from file,
 * - stores them in internal variables
 * - allows to fetch requested configuration value
 * - can apply configuration to Transport Plugin DLL
 */
class DcmiConfiguration
{
public:

    /**
     * Dtor.
     * Frees memory allocated for storing configuration.
     */
    ~DcmiConfiguration();

    /**
     * Loads configuration from file and stores configuration values in
     * internal variables.
     *
     * @retval 0 Successfull configuration load.
     * @retval 1 Failed to open configuration file or failed to process its contents.
     */
    int loadConfiguration();

    /**
     * Reads single configuration option value obtained by the
     * <code>loadConfiguration</code> method.
     * Example:
     * If the configuration file contains the following 2 lines:
     * [MySection]
     * MyOption=MyValue
     * then calling getConfigurationValue("MySection", "MyOption") will
     * return pointer to "MyValue" string.
     *
     * @param sectionName Section name in configuration file (i.e. transport plugin name)
     * @param optionName configuration option name
     *
     * @return Pointer to requested value or NULL if the option has not been found
     */
    const char* getConfigurationValue(const char* sectionName, const char* optionName) const;

    /**
     * Reads single configuration option value obtained by the
     * <code>loadConfiguration</code> method.
     * Example:
     * If the configuration file contains the following 2 lines:
     * [MySection]
     * MyOption=MyValue
     * then calling getConfigurationValue("MySection", "MyOption") will
     * return pointer to "MyValue" string.
     *
     * @param sectionName Section name in configuration file (i.e. transport plugin name)
     * @param optionName configuration option name
     *
     * @return Pointer to requested value or NULL if the option has not been found
     */
	 int setConfigurationValue(const char *sectionName, const char *optionName, const char *value);

	 /// Change the value of cipher, this function is used to
	/// run the test automatically for all ciphersuites
	int ChangeCipher(const char pCipher[]);

    /**
     * Applies configuration to single Transport Plugin DLL. The configuration
     * has to be previously read from configuration file by the
     * <code>loadConfiguration</code> method.
     * Example:
     * If the configuration file contains the following 3 lines:
     * [MyPluginName]
     * MyOption1=MyValue1
     * MyOption2=MyValue2
     * then calling applyPluginConfiguration(pTpif, "MyPluginName") will cause
     * the following configuration call attempts:
     * pTpif->configure("MyOption1", "MyValue1");
     * pTpif->configure("MyOption2", "MyValue2");
     *
     * @param pTpif Pointer to the Transport Plugin DLL interface
     * @param pluginName Name of the transport plugin
     *
     * @retval 0 Successfull plugin configuration.
     * @retval 1 Failed to find configuration for the plugin or failed to apply configuration.
     */
    int applyPluginConfiguration(transport_plugin_interface* pTpif, const char* pluginName) const;

	/*
	Verify that each option loaded in the Configuration is in valid range
	The valid range checks performed are
	*/
	int CheckRMCPPConfiguration();
   	/*
	Verify that each option loaded in the Configuration is in valid range
	The valid range checks performed are
	*/
	int CheckInBandConfiguration();

	int GetPassword();

	int InsertPassword(char[]);

	const char *Config_UserName;  //Store the username read from configuration file

	int InitializeLogger(std::ofstream* pLogFile);

	int ConvertVerbosity(TransportPlugin::LogLevel* pLogLevel, const char* logLevelString);

private:

	/// Matches the base string to the *strings list, and returns the location if the base string is found
	/// else returns -1.
    int match(const char* base, int MAX_STRINGS, const char *strings, ...);



	/// Checks the format of the TargetIP to be 4 numbers seperated by .
	/// valid range for these numbers is 0-255
	/// Also, the userconf.cfg file should have the IP in decimal
	int CheckTargetIP( const char* value);


	int CheckDriverName(const char *value);

	/// Internal type definition for storing single configuration value.
    typedef std::pair<char*,char*> DcmiConfigurationPair;
    /// Internal type definition for storing set of configuration values for single entity.
    typedef std::set<DcmiConfigurationPair*> DcmiEntityConfigurationSet;
    /// Internal type definition for storing entire configuration.
    typedef std::map<std::string, DcmiEntityConfigurationSet*> DcmiConfig;

	//Convert the entire string to lowercase
	void toLowerCase(char *);

	/// Stores entire configuration.
    DcmiConfig configuration;


};

#endif //_DCMI_CONFIGURATION_H
