/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2011 INTEL CORPORATION                       **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intel�s	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*
 * TestClasses.h
 *
 *  Created on: 30-03-2011
 */

#ifndef TESTCLASSES_H_
#define TESTCLASSES_H_

#include <string>
#include <list>
#include <ostream>
#include <algorithm>
#include <set>

#include "any_value.hpp"


using CommonClasses::any_value;

namespace TestUtil
{

enum TestType {
		TT_OPTIONAL,
		TT_MANDATORY,
		TT_INFO
};

enum TestResult {
		TR_NONE = -1,
		TR_SUCCESS = 0,
		TR_FAILURE,
		TR_SKIPPED,
		TR_NOT_SUPPORTED,
		TR_CC_FAILURE,
		TR_NETWORK_ERR,
		TR_INVALID_LEN
};

/**
 * Print formated Test info
 *
 * @param stream - stream to print out data
 * @param info - object to print out
 *
 * @return stream...
 */
std::ostream &operator<<(std::ostream &stream, TestResult &info);

/**
 * Print formated Test Type
 *
 * @param stream - stream to print out data
 * @param info - object to print out
 *
 * @return stream...
 */
std::ostream &operator<<(std::ostream &stream, TestType &info);


class TestInfo;

class TestOutput
{
public:

	TestOutput() {};
	virtual ~ TestOutput() {};

	virtual TestOutput& raw_print(const std::string &str) = 0;

	virtual void print_header(const std::string &name) = 0;
	virtual void suite_start(const std::string &name) = 0;
	virtual void suite_end() = 0;

	virtual void test_start(const std::string &name) = 0;
	virtual void test_info(const TestInfo &info, const std::string &error_msg) = 0;
	virtual void test_end() = 0;
};


class TestOutputText : public TestOutput
{
	std::ostream &m_stream;

	int m_suitcounter;
	int m_testcounter;

public:

	TestOutputText(std::ostream &stream) :
		m_stream (stream)
	{
		m_suitcounter = 0;
		m_testcounter = 0;
	}

	TestOutput &raw_print(const std::string &str);

	void print_header(const std::string &name);
	void suite_start(const std::string &name);
	void suite_end();

	/**
	 * Print formated Test info
	 *
	 * @param test info to print out
	 *
	 */
	void test_info(const TestInfo &info, const std::string &error_msg);

	void test_start(const std::string &name);
	void test_end();

};


/*
 * Tests base class for DCMI conformance suit
 *
 */
class TestSuite
{
	//test package name..
	std::string m_name;

	//all current tests ...
	std::list<TestInfo *> m_tests;
   //all testsuits - packages of tests
	std::list<TestSuite *> m_suits;

   void runTests(TestOutput &out);
   void runSuites(TestOutput &out);

public:

	//method to execute as one single test case ..
	typedef TestResult (TestSuite::*Function) (any_value &);
	//method to execute and print some extra info on output summary stream
	typedef TestResult (TestSuite::*FunctionInfo) (TestOutput &);

	virtual ~TestSuite();

	virtual void doConfigure(DcmiConfiguration &option);

	virtual void runSuite(TestOutput &out);

protected:

	/*
	 * Ctor, Protected, to avoid straight use
	 * */
  	TestSuite(std::string &name):
  		m_name (name)
  	{ };

  	TestSuite()
  	{ };

  	/**
  	 * Prevent from simple paste a pointer - avoid pointer owner problems
  	 */
	void addTestSuite(std::auto_ptr<TestSuite> &suit);

	/**
	 * Add to spooler a new suit
	 *
	 * @param fpt - pointer to Test Function
	 * @param fparam - any_value param
	 * @param desc - description
	 * @param type - type mandatory/optional
	 */
	void addTest(Function fpt, const any_value &fparam, const std::string &desc, const TestType type = TT_OPTIONAL);

	/**
	 * add to spooler list a pointer to info function ...
	 *
	 * @param fpt - pointer to Info Function
	 */
	void addInfo(FunctionInfo fpt, const std::string &desc);

	inline void setName(std::string name) { m_name = name; };
	inline std::string &getName() { return m_name; };

	/**
	 * Start up test suit ...
	 *
	 * can rise exception and terminate fulll test suits...
	 *
	 * @param  out stream to print info..
	 */
	virtual void start_up(TestOutput &out) /*throw(...)*/;

	virtual void tear_down(TestOutput &out);
};


#define ADD_TEST(func, msg, type) \
	addTest(reinterpret_cast<Function>(&func), any_value(), msg, type)

#define ADD_TEST_PARAM(func, param, msg, type) \
	addTest(reinterpret_cast<Function>(&func), any_value(param), msg, type)

#define ADD_INFO(func, msg) \
	addInfo(reinterpret_cast<FunctionInfo>(&func), msg)

#define ADD_PACKAGE(testClass) \
   { \
	   std::auto_ptr<TestUtil::TestSuite> suit(new testClass()); \
	   addTestSuite(suit);\
   }

#define ADD_PACKAGE_PARAM(testClass, ...) \
	{ \
		std::auto_ptr<TestUtil::TestSuite> suit(new testClass(__VA_ARGS__)); \
		addTestSuite(suit); \
	}

} //namespace

#endif /* TESTCLASSES_H_ */
