/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intel�s	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*******************************************************************************
***		 File		:	DCMI_Chassis_lib.cpp								  **
***																			  **
***		 Abstract	:	Implements low level procedures to send the           **
***				        commands under "Chassis  commands"                    **
***                     in Table6.1 "Platform Command Definition"            **
***                     of DCMI 1.0 spec. Also implements                     **
***						the test procedures to validate  the same commands    **
***                     in the DCMI_Conformance Module.                      **
***																		      **
********************************************************************************/

#include "parameters.h"
#include "declarations.h"
#include "DCMI_Chassis_lib.h"

using std::hex;
using std::showbase;

void ChassisBase::doConfigure(DcmiConfiguration &option)
{
	strtotimeout(option.getConfigurationValue("TimeOutValues","TimeOutPwrOn"), m_TimeOutPwrOn);
	strtotimeout(option.getConfigurationValue("TimeOutValues","TimeOutPwrOn"), m_TimeOutPwrOff);
	strtotimeout(option.getConfigurationValue("TimeOutValues","TimeOutLedBlink"), m_TimeOutLedBlink);
}

/*******************************************************************
int IssueChassisControl ( int iControlAction, byte_t *compCode)
Function Name:		IssueChassisControl
 Purpose:			Takes an input value of: iControl_action =>
 0 = Off
 1 = On
 2 = Reset
 And sends the IPMI command sequence to the designated interface to
 perform the selected action on the UUT.

 Prints information on the power of/on/reset issued to the UUT.
 Parameters: [IN]  chAction - The action to be performed on the UUT as below
					0 = Off
					1 = On
					2 = Reset
 *********************************************************************/
void
ChassisBase::issueChassisControl(POWER_CMD chAction)
{
    CommonClasses::IpmiRequest request(CHASSIS, 0x02);
    request.append<byte_t>(chAction);// ON/ OFF/ RESET

    //Build Command Object
    CommonClasses::IpmiResponse response(CHASSIS_CTRL_MIN_RSPLEN, CHASSIS_CTRL_MAX_RSPLEN);

    //Priv Level for Chassis Control is Operator
	setSessionPrivilegeLevel(PRIV_OPERATOR);

	//Send request data to the transport & Verify Response Completion Code etc...
	sendIpmiCmd(request, response);

	//ChassisControl command sent successfully to UUT
	//so Wait for System to stabilize after issuing Power state and returns OK
    switch (chAction)
	{
	case DC_OFF:
		LOG_INFO << "\n\tSystem Powered DOWN by Chassis Command.";
		//wait for a while and allow system time to "get over"
		sleep_printdots(m_TimeOutPwrOff);
		break;
	case DC_ON:
		LOG_INFO << "\n\tSystem Powered UP by Chassis Control Command.";
		//wait for a while and allow system time to "get over"
		sleep_printdots(m_TimeOutPwrOn);
		break;
	case SYSTEM_RESET:
		LOG_INFO << "\n\tSystem RESET issued by Chassis Control Command.";
		//wait for a while and allow system time to "get over"
		sleep_printdots(m_TimeOutPwrOn);
		break;
	}
}

/*************************************************************************
Function Name	: GetChassisStatus
Purpose			: Sends the IPMI command sequence to the designated interface to get
 chassis status of the UUT, primarily used to see if the unit is DC
 on or off.
**************************************************************************/
byte_t
ChassisBase::getChassisStatus()
{
  	CommonClasses::IpmiRequest request(CHASSIS, 0x01);
	//Build Command Object
	CommonClasses::IpmiResponse response(CHASSIS_STATUS_MIN_RSPLEN, CHASSIS_STATUS_MAX_RSPLEN);
	//Send request data to the transport & Verify Response Completion Code etc...
	sendIpmiCmd(request, response);
	return response[0];
}


/**************************************************************************************
Function Name:	ChassisIdentify
Purpose		 :	Sends the IPMI command sequence to the designated interface to get
                chassis Identify LED blink for few seconds specified in the
				IdentifyInterval, primarily used to see the chassisIdentify command
				is supported in UUT.
 *************************************************************************************/

void
ChassisBase::chassisIdentify(byte_t IdentifyInterval, byte_t ForceIdentify)
{
    //Build Command Object
	CommonClasses::IpmiRequest request(CHASSIS, 0x04);
	request.append<byte_t>(IdentifyInterval)
		   .append<byte_t>(ForceIdentify);

	//Build Command Object
	CommonClasses::IpmiResponse response(CHASSIS_IDENTIFY_MIN_RSPLEN, CHASSIS_IDENTIFY_MAX_RSPLEN);

	//Send request data to the transport
	sendIpmiCmd(request, response);
}

/**************************************************************************
Function Name:		GetAcpiPwrState
Purpose:			Issues the Get ACPI Power State Command
 **************************************************************************/
byte_t
ChassisBase::getAcpiPwrState ()
{
	//Build Command Object
	CommonClasses::IpmiRequest request(APPLICATION, 0x07);
	//Build Command Object
	CommonClasses::IpmiResponse response(GET_ACPI_PWR_MIN_RSPLEN,  GET_ACPI_PWR_MAX_RSPLEN);
	//Send request data to the transport
	sendIpmiCmd(request, response);
	//return Command Completion Status
	return response[0];
}

/***************************************************************************
Function Name:		CheckSystemOFFState
Purpose:			Do a "Get ACPI Power State" Command, and check the System
                    Power state value to be 0x21 (legacy off) or 0x05 (S5 state)
					0x07, 0x0A are also considered valid off state.
*****************************************************************************/
template<class Predicate>
bool
ChassisBase::checkSystemState( Predicate _pred )
{
	//Min Privilege Level for Get ACPI Power State is User
	setSessionPrivilegeLevel(PRIV_USER);

	//Send request data to the transport
	byte_t state = getAcpiPwrState();

	//Verify Completion Code & Process return Data
	int iSystemPwrState = (state & 0x7F); //bits 0-6 give the ACPI Power State of System
	if ( _pred(iSystemPwrState) )
	{	//System Power State in Get ACPI Power State cmd is indicated correctly
		LOG_INFO<<"\n\tACPI Power State is indicated correctly "<<hex<<showbase<<int(iSystemPwrState);
		return true;
	}

	LOG_ERR<<"\n\tACPI Power State received - "<<hex<<showbase<<(int)iSystemPwrState<<" is not valid  ";
	return false;
}

bool ChassisBase::isACPI_OFF_State(int iSystemPwrState)
{
	return ((iSystemPwrState == 0x05) || (iSystemPwrState == 0x07)
			|| (iSystemPwrState == 0x0A) || (iSystemPwrState == 0x21));
}

bool ChassisBase::isACPI_ON_State(int iSystemPwrState)
{
	return ((iSystemPwrState >= 0x00 && iSystemPwrState <= 0x04)
			|| (iSystemPwrState == 0x06) || (iSystemPwrState == 0x08)
			|| (iSystemPwrState == 0x09) || (iSystemPwrState == 0x20));
}


TestResult
ChassisTest::ChassisTurnOffTest(any_value &)
{
	LOG_DEBUG<<"\nIssue the Chassis Status to Power DOWN the System";
	issueChassisControl(DC_OFF);
	return TestUtil::TR_SUCCESS;
}

TestResult
ChassisTest::ChassisTurnOnTest(any_value &)
{
	LOG_DEBUG<<"\nIssue the Chassis Status to Power UP System";
	issueChassisControl(DC_ON);
	return TestUtil::TR_SUCCESS;
}

TestResult
ChassisTest::ChassisRebootTest(any_value &)
{
	LOG_DEBUG<<"\nIssue the Chassis Status to REBOOT System";
	issueChassisControl(SYSTEM_RESET);
	return TestUtil::TR_SUCCESS;
}

/***********************************************************************************
Function Name:		ChkStatusPwrON
 Makes sure that the system is in DC ON state. Calls the ChassisStatus() to get
 the status of the UUT. If UUT is in DC off state, then issues DC on with ChassisControl().
 Prints information on the state of UUT - DC off/on .
*************************************************************************************/

TestResult
ChassisTest::ChkStatusPwrONTest(any_value &)
		//const DcmiConfiguration* pConfiguration, int inBandFlag)
{
	LOG_DEBUG<<"\nIssue the Chassis Status to check if system is in DC ON state ";
	//Send request data to the transport
	byte_t response = getChassisStatus();
	//If valid completion code received then check UUT power status.
	//Checking lowest order bit to get UUT Power Status

	if ((response & 0x01) == 0x00 )
	{
		if (!isRemoteConnection())
		{
			LOG_ERR<<"Test is executing on Local Connection and Status Indicate Power OFF!!! Impossible!!";
			return TestUtil::TR_FAILURE;
		}

		LOG_INFO<<"\n\tPower is indicated as OFF. Turning ON power to start test.";
		//Turn on unit.  Value of one (1) will turn on unit.
		issueChassisControl(DC_ON);
	}

	LOG_INFO<<"\n\tSystem Power is indicated as ON";
	return TestUtil::TR_SUCCESS;
}


/*************************************************************************
Function Name:	ChassisIdentifyCheck
Purpose      :	Issues the ChassisIdentify command to turn LED to blink, and
				then turns it off. User has to check physically the
				chassis Identify LED on UUT blinking or not.
*************************************************************************/
TestResult
ChassisTest::ChassisIdentifyCheckTest(any_value &)
{
	//Turn the LED to Force Blink ON
	LOG_INFO<<"\nTurning UUT Identify LED ON";
	chassisIdentify(00, 01);
    //Valid completion code received, check UUT LED Status.
	LOG_INFO<<"\n\tPlease verify the Chassis Identify LED is blinking on UUT.";
	//wait for a while and allow system time to "get over"
	sleep_printdots(m_TimeOutLedBlink);

	//Turn LED off
	LOG_INFO<<"\n\tTurning UUT Identify LED OFF";
	chassisIdentify(00, 00);

	return TestUtil::TR_SUCCESS;
}

/*************************************************************************
Function Name:		TestAcpiPwrState
Purpose:			Implements the test case for verifying "Get ACPI Power State"
                    Command
**************************************************************************/
TestResult
ChassisTest::AcpiPwrStateTest(any_value &)
//(const DcmiConfiguration* pConfiguration, int iInbandFlag)
{
	/*
	Test Case Procedure :
	-Check the System Power State.
	- if System is ON
		- then check for valid ACPI Power State
		- only in OOB turn system off, and check for valid ACPI Power state as well
		- Restore system to original ON state
	- if System is OFF
        - the test is running in OOB, so first check for valid ACPI OFF Power State
		- turn system on and check for valid ACPI ON Power State
		- Leave System in ON state
	*/
	LOG_DEBUG<<"\nIssue ACPI Power State tests";
	//Priv Level for Chassis Status is USER
	setSessionPrivilegeLevel(PRIV_USER);

	//Check current state of system with GetChassisStatus
	//Send request data to the transport
	LOG_DEBUG<<"\n\tChecking for Chassis System Power State ";
	byte_t status = getChassisStatus();

    //Checking lowest order bit to get UUT Power Status
	if ((status & 0x01) == 0x01) //power is on
	{	//System Power is indicated as ON
		//so check for ACPI state to be S0 or 20
		LOG_DEBUG<<"\n\tChecking for valid ACPI System Power State, when system is ON";
		if (!checkSystemState(isACPI_ON_State))
		{
			return TestUtil::TR_FAILURE;
		}
		if (!isRemoteConnection())
		{
			LOG_WARN<<"\nTest is executing on Local Connection Can NOT make POWER Cycle!!";
			return TestUtil::TR_SKIPPED;
		}

		//Turn off System - to verify ACPI State
		// Wait for System to stablilise and Power State to be updated
		LOG_INFO<<"\n\tTurn off system, to check for valid ACPI Power State when system is OFF";
		issueChassisControl(DC_OFF);

		//Received valid completion code to Chassis Control Command from UUT
		//check system state
		if (!checkSystemState(isACPI_OFF_State))
		{
			return TestUtil::TR_FAILURE;
		}
		//Turn o System - to verify ACPI State
		//Wait for System to stablilise and Power State to be updated
		LOG_INFO<<"\n\tRestoring System State to ON";
		issueChassisControl(DC_ON);
		//System Restored to ON state??
	}//power is on
	else
	{	//System Power is indicated as OFF
		if (!isRemoteConnection())
		{ //how it is posible if we are on the host ....
			LOG_WARN<<"\nTest is executing on Local Connection and Status Indicate Power OFF!! Impossible!!!";
			return TestUtil::TR_FAILURE;
		}

		//System Power is indicated as OFF
		LOG_INFO<<"\n\tChecking for valid ACPI Power State, when system is OFF";
		//so check for ACPI state to be S5 or 21, then turn system on and wait for 10sec
		//then check ACPI state to be S0 or 20, turn system on
		if (!checkSystemState(isACPI_OFF_State))
		{
			return TestUtil::TR_FAILURE;
		}
		//Turn on System - to verify ACPI State
		// Wait for System to stablilise and Power State to be updated
		LOG_INFO<<"\n\tTurning system ON, to check for valid ACPI Power State when system is ON";
		issueChassisControl(DC_ON);

		// Received valid completion code to Chassis Control Command from UUT
		if (!checkSystemState(isACPI_ON_State))
		{
			return TestUtil::TR_FAILURE;
		}
	} //power id off

	//Leave system in ON state
	return TestUtil::TR_SUCCESS;
}

/*************************************************************************
Function Name	: ChassisCapabilitiesCheck
Purpose			: Sends the IPMI command sequence to the designated interface
to get chassis capabilities of the UUT, primarily used to get information
about main chassis management functions present on IPMB and what address
used to access these functions.
**************************************************************************/
TestResult
ChassisTest::ChassisCapabilitiesTest(any_value &)
{
	LOG_DEBUG<<"\nIssue Chassis Capabilities test.";
	CommonClasses::IpmiRequest request(CHASSIS, 0x00);
	//Build Command Object
	CommonClasses::IpmiResponse response(GET_CHASSIS_CAPAB_MIN_RSPLEN,GET_CHASSIS_CAPAB_MAX_RSPLEN);

	//Send request data to the transport
	sendIpmiCmd(request, response);

	LOG_INFO<<"\n\tCapabilities Flags\t\t\t : "<< hex<<showbase<<int(response[0]);
	LOG_INFO<<"\n\tChassis FRU Info Device Address\t\t : "<< hex<<showbase<<int(response[1]);
	LOG_INFO<<"\n\tChassis SDR Device Address\t\t : "<< hex<<showbase<<int(response[2]);
	LOG_INFO<<"\n\tChassis SEL Device Address\t\t : "<< hex<<showbase<<int(response[3]);
	LOG_INFO<<"\n\tChassis System Management Device Address : "<< hex<<showbase<<int(response[4]);
	LOG_INFO<<"\n\tChassis Bridge Device Address\t\t : "<< hex<<showbase<<int(response[5]);

	return TestUtil::TR_SUCCESS;
}

