/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008,2011 INTEL CORPORATION                  **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intel's	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*******************************************************************************
***		 File		:	DCMI_Configuration.cpp				 			      **
***																			  **
********************************************************************************/

#include <stdarg.h>
#include <fstream>

#include "parameters.h"
#include <string.h>

#ifdef WIN32
   #include <conio.h>
#else
   #include <stdio.h>
   #include <termios.h>
   #include <unistd.h>
#endif


#include "OsSpecificFunctions.h"
#include "TransportPluginDll.h"

#include "DCMI_Configuration.h"

using namespace std;
using TransportPlugin::OsSpecificFunctions;


#ifndef WIN32
/***************************************************************************
* int mygetch()
*
* Description : Prints the character entered by user
* Returns : the character
***************************************************************************/
int mygetch()
{
  struct termios oldt, newt;
  int ch;

  tcgetattr(STDIN_FILENO, &oldt);

  newt = oldt;
  newt.c_lflag &= ~(ICANON | ECHO);
  tcsetattr(STDIN_FILENO, TCSANOW, &newt);
  ch = getchar();
  tcsetattr(STDIN_FILENO, TCSANOW, &oldt);
  return ch;

}
#endif


/* This is a user defined swap function to swap the contents of first and second
   This has been implemented instead of built-in pair::swap which doesn't work
   as expected in Linux. */

template <class A>
void Dcmi_Swap(A *first, A *second)
{
	A temp = *first;
	*first = *second;
	*second = temp;
}

DcmiConfiguration::~DcmiConfiguration()
{
    for(DcmiConfig::iterator cfgIter = configuration.begin(); cfgIter != configuration.end(); cfgIter++)
    {
        DcmiEntityConfigurationSet* pCfgSet = cfgIter->second;
        for(DcmiEntityConfigurationSet::iterator setIter = pCfgSet->begin(); setIter != pCfgSet->end(); setIter++)
        {
            DcmiConfigurationPair* pCfgPair = *setIter;
            delete pCfgPair->first;
            delete pCfgPair->second;
            delete pCfgPair;
        }
        pCfgSet->clear();
        delete pCfgSet;
    }
    configuration.clear();
}

#ifdef CIPHER_AUTOCHECK
int DcmiConfiguration::ChangeCipher(const char pCipher[])
{
	return setConfigurationValue("RMCPP" , "ciphersuite" , pCipher);
}
#endif

int
DcmiConfiguration::CheckRMCPPConfiguration()
{
	int iRetVal = SUCCESS, optionNumber;

	/// The following are the options, that need to be present in the Configuration file
    const int totalMustOptions = 3;          //defines the number of elements in mustOptions array
	const char mustOptions[totalMustOptions][2][20] = { {"RMCPP","targetip"}, {"RMCPP","username"}, {"RMCPP","ciphersuite"}  };

	/// Check for support for must-be-present options in the configuration file
	for(int optionCtr=0; ((optionCtr < totalMustOptions) && (iRetVal==SUCCESS) ) ; optionCtr++)
	{
		const char *optionValue = this->getConfigurationValue(mustOptions[optionCtr][0],mustOptions[optionCtr][1]);
		if(  optionValue == NULL )
		{
			// The option is null or not defined in the configuration file
			cout << "\n Test Suite requires following option in configuration file userconf.cfg: \n";
			cout << "["<< mustOptions[optionCtr][0]<<"] - "<<mustOptions[optionCtr][1];
			iRetVal = E_CONFIG_LOAD;	//error, -3, invalid data in config file
		}
		else
		{
			int selection = match(mustOptions[optionCtr][0], 2, "RMCPP", "Dcmi_Conformance");
			switch( selection )
			{
			case 0 :
					//RMCPP
					optionNumber = match(mustOptions[optionCtr][1], 4,
                             "targetip", "username", "password" , "ciphersuite", NULL);

					switch( optionNumber )
					{
					case 0:
							//targetIP
							if ( CheckTargetIP(optionValue) )
							{
								//TargetIP is not in valid format
								cout<<"\n [RMCPP]-TargetIP initialized to invalid value in Userconf.cfg ";
								iRetVal = E_CONFIG_LOAD;	//error, -3, invalid data in config file
							}
							break;
					case 1:
							//Username
							//Check if username is less than 16 characters
							if ( strlen(optionValue) > 16 )
							{
								//Username is not in valid format
								cout<<"\n [RMCPP]-Username can be maximum of 16 characters only as per IPMI spec ";
								iRetVal = E_CONFIG_LOAD;	//error, -3, invalid data in config file
							}

							break;
					case 2:
							//Password
							//Check if password is less than 20 characters
							if ( strlen(optionValue) > 20 )
							{
								//Username is not in valid format
								cout<<"\n [RMCPP]-Password can be maximum of 20 characters only as per IPMI spec ";
								iRetVal = E_CONFIG_LOAD;	//error, -3, invalid data in config file
							}

							break;
					case 3:
							//CipherSuite
							//DCMI Table 4-1 Supported Cipher Suites
							optionNumber = match(optionValue, 10, "0","1","2","3","6",
												"7","8","11","12","17", NULL);
							if (optionNumber == -1 )
							{
								//CipherSuite not initialised with valid number as per DCMI spec
								cout<<"\n [RMCPP]-Ciphersuite initialized to invalid value in Userconf.cfg ";
								iRetVal = E_CONFIG_LOAD;	//error, -3, invalid data in config file
							}
							break;
					}
					break;
			}
		}
	}

/*
	if( iRetVal)
	{
		/// Print instruction to User - as error encountered in Configuration file
		cout << "\n Modify User Configuration file to fix above errors, and re-run test Suite";
	}
*/
return iRetVal;
}


int
DcmiConfiguration::CheckInBandConfiguration()
{
	int iRetVal = SUCCESS, optionNumber;

	/// The following are the options, that need to be present in the Configuration file
    const int totalMustOptions = 1;          //defines the number of elements in mustOptions array
	const char mustOptions[totalMustOptions][2][20] = { {"INBAND","DriverName"} };

	/// Check for support for must-be-present options in the configuration file
	for(int optionCtr=0; ((optionCtr < totalMustOptions) && (iRetVal==SUCCESS) ) ; optionCtr++)
	{
		const char *optionValue = this->getConfigurationValue(mustOptions[optionCtr][0],mustOptions[optionCtr][1]);
		if(  optionValue == NULL )
		{
			// The option is null or not defined in the configuration file
			cout << "\n Test Suite requires following option in configuration file userconf.cfg: \n";
			cout << "["<< mustOptions[optionCtr][0]<<"] - "<<mustOptions[optionCtr][1];
			iRetVal = E_CONFIG_LOAD;	//error, -3, invalid data in config file
		}
		else
		{
			int selection = match(mustOptions[optionCtr][0], 2, "INBAND", "Dcmi_Conformance");
			switch( selection )
			{
			case 0 :
					//inband
					optionNumber = match(mustOptions[optionCtr][1], 1, "DriverName", NULL);

					switch( optionNumber )
					{
					case 0:
							//drivername
							if ( CheckDriverName(optionValue) )
							{
								//TargetIP is not in valid format
								cout<<"\n [INBAND]-DriverName initialized to invalid value in Userconf.cfg ";
								iRetVal = E_CONFIG_LOAD;	//error, -3, invalid data in config file
							}
							break;
					}
					break;
			}
		}
	}

/*
	if( iRetVal)
	{
		/// Print instruction to User - as error encountered in Configuration file
		cout << "\n Modify User Configuration file to fix above errors, and re-run test Suite";
	}
*/
return iRetVal;
}

int DcmiConfiguration::CheckDriverName(const char * value)
{
	if (value == NULL)
		return E_CONFIG_LOAD;
	int selection = match(value, 3, "Microsoft_IPMI", "DCMI", "dcmi");
	return ((selection<0)?1:0);
}

int
DcmiConfiguration::loadConfiguration()
{
    char configFilename[] = "userconf.cfg";
	int  iRetVal = 0;

    ifstream configFile;

    configFile.open(configFilename, ios::in);

    if (!configFile)
    {
       cout<<"\n\tUnable to open or locate configuration file.\n";
       return E_CONFIG_LOAD;	//error, -3, invalid data in config file
    }

    char line[255];
    DcmiEntityConfigurationSet* pCurrConfigSet = NULL;
    while (configFile.getline(line, 255))
    {
        int length = (int) strlen(line);
        char* option = &line[0];
        char* value = NULL;
        char* bracketBegin = NULL;

        //read the .cfg file, assume no white space, = starts value, # starts comment
        //replace with '\0' for zero terminated string proccessing.
        for (int i=0; i<length; i++)
        {
            if (line[i] == '[')
            {
                bracketBegin = &line[i+1];
            }

            if (line[i] == ']' && bracketBegin != NULL)
            {
                line[i] = '\0';
                DcmiConfig::iterator it = this->configuration.find(bracketBegin);
                if (it == configuration.end())
                {
                    pCurrConfigSet = new DcmiEntityConfigurationSet();
                    configuration[bracketBegin] = pCurrConfigSet;
                }
                else
                {
                    pCurrConfigSet = it->second;
                }

                bracketBegin = NULL;
            }

            if (line[i] == '=')
            {
                line[i] = '\0';
                i++;
                //value string should after =, dont forget # or \n
                if (i<=length)
                {
                    value = &line[i];
                }
            }

            //dont read past newlines, space, null, comments, return?, tab ?...
            if ( (line[i] == '#')|| (line[i] == '\n')|| (line[i] == '\r')|| (line[i] == '\0')
                    || (line[i] == '\t')|| (line[i] == ' '))
            {
                line[i] = '\0';
                bracketBegin = NULL;
                break;
            }
        }

        if (value != NULL)
        {
            if (pCurrConfigSet != NULL)
            {
				//convert option to all lowercase, before inserting it
				toLowerCase(option);

                size_t optionStrLen = strlen(option) + 1;
                char* optionStr = new char[optionStrLen];
                TransportPlugin::OsSpecificFunctions::strcpys(optionStr, (unsigned int)optionStrLen, option);

				size_t valueStrLen = strlen(value) + 1;
				char *valueStr = new char[valueStrLen];
				TransportPlugin::OsSpecificFunctions::strcpys(valueStr, (unsigned int)valueStrLen, value);

                DcmiConfigurationPair* pConfigPair = new DcmiConfigurationPair(optionStr, valueStr);
                pCurrConfigSet->insert(pConfigPair);
            }
            else
            {
            	cout<<"\nError parsing configuration file. Cannot associate '"<<option<<"="<<value<<"' with any entity.\n";
            }
        }
    }

    if (!configFile.eof())
    {
        cout<<"\nFailed to process configuration file. Possible reason: Line in the file is longer than 255 characters.\n";
    }

    return iRetVal;
}

void
DcmiConfiguration::toLowerCase(char *option)
{
	size_t ctr;
	char ch;
	for(ctr=0;ctr<strlen(option);ctr++)
	{
		ch = option[ctr];
		option[ctr] = tolower(ch);
	}
}

#ifdef CIPHER_AUTOCHECK
int
DcmiConfiguration:: setConfigurationValue(const char * sectionName, const char* optionName, const char *pValue)
{
	int found=0;
	 // Try to find specified section.
    DcmiConfig::const_iterator cfgIter = this->configuration.find(sectionName);
    if (cfgIter == configuration.end())
    {
        // There was no such section in the configuration file.
        return NULL;
    }

    // Try to find specified option.
    DcmiEntityConfigurationSet* pCfgSet = cfgIter->second;
    for(DcmiEntityConfigurationSet::const_iterator setIter = pCfgSet->begin(); setIter != pCfgSet->end(); setIter++)
    {
        DcmiConfigurationPair* pCfgPair = *setIter;
        if (TransportPlugin::OsSpecificFunctions::stricmp(optionName, pCfgPair->first) == 0)
        {
			// The specified option found.
			size_t optionStrLen = strlen(optionName) + 1;
			size_t valueStrLen = strlen(pValue) + 1;

			std::auto_ptr<char> optionStr(new char[optionStrLen]);
			std::auto_ptr<char> valueStr(new char[valueStrLen]);

			TransportPlugin::OsSpecificFunctions::strcpys(optionStr.get(), (unsigned int)optionStrLen, optionName);
			TransportPlugin::OsSpecificFunctions::strcpys(valueStr.get(), (unsigned int)valueStrLen, pValue);
			DcmiConfigurationPair* pConfigPair = new DcmiConfigurationPair(optionStr.release(), valueStr.release());
			Dcmi_Swap(pCfgPair, pConfigPair);
			found = 1;

        }
    }

    // The specified option has not been found.
    return found;
}
#endif


const char*
DcmiConfiguration::getConfigurationValue(const char* sectionName, const char* optionName) const
{
    // Try to find specified section.
    DcmiConfig::const_iterator cfgIter = this->configuration.find(sectionName);
    if (cfgIter == configuration.end())
    {
        // There was no such section in the configuration file.
        return NULL;
    }

    // Try to find specified option.
    DcmiEntityConfigurationSet* pCfgSet = cfgIter->second;
    for(DcmiEntityConfigurationSet::const_iterator setIter = pCfgSet->begin(); setIter != pCfgSet->end(); setIter++)
    {
        DcmiConfigurationPair* pCfgPair = *setIter;
        if (TransportPlugin::OsSpecificFunctions::stricmp(optionName, pCfgPair->first) == 0)
        {
            // The specified option found.
            return pCfgPair->second;
        }
    }

    // The specified option has not been found.
    return NULL;
}

int
DcmiConfiguration::applyPluginConfiguration(transport_plugin_interface* pTpif, const char* pluginName) const
{
    // Find plugin with pluginName.
    DcmiConfig::const_iterator cfgIter = this->configuration.find(pluginName);
    if (cfgIter == configuration.end())
    {
        // No such plugin.
        return 1;
    }

    // Iterate through all options for the plugins and call configure for each option
    DcmiEntityConfigurationSet* pCfgSet = cfgIter->second;
    for(DcmiEntityConfigurationSet::const_iterator setIter = pCfgSet->begin(); setIter != pCfgSet->end(); setIter++)
    {
        DcmiConfigurationPair* pCfgPair = *setIter;
        int result = pTpif->configure(pCfgPair->first, pCfgPair->second);
        if (result != 0)
        {
            return 1;
        }
    }

    return 0;
}


int
DcmiConfiguration::match(const char* base, int MAX_STRINGS, const char* strings, ...)
{
    const char* compare;
    int stringNumber;
    va_list list;

    //The base string to compare with other strings (quit if NULL)
    if (base == NULL)
        return -1;

    //compare all the other strings (quit when NULL)
    va_start(list, strings);
    compare = (char *)strings;
    stringNumber = 0;
    while ((compare != NULL) && stringNumber < MAX_STRINGS)
    {
        if (strcmp(base, compare) == 0)
        {
            //printf("\n---\nMATCH = %s\n+++\n", compare);
            return stringNumber;
        }
        //no luck, try the next one
        compare = va_arg(list, char*);
        stringNumber++;
    }
    return -1;
}

int
DcmiConfiguration::CheckTargetIP(const char* value)
{
	int  len = (int) strlen(value);
    int  cntr, IP[5], IPelement, iRetVal, elementCnt;

	iRetVal = 0;
	//Check if each character in target IP is either '.' or a number 0-9
	for(cntr=0, IPelement=0, elementCnt=0; (cntr < len) && (elementCnt<=3) && (iRetVal == 0) ; cntr++)
	{

		if (value[cntr] == '.')
		{
			if( IPelement > 255 || IPelement < 0)
			{
				//Valid value for each elemnt of IP is 0-255
				cout<< "\nInvalid value initialised for [RMCPP]-TargetIP";
				iRetVal = -1;
			}
			IP[elementCnt] = IPelement;		//initialise the array with the IP element (ex) 222.222.222.14
											// here the 4 elements are 222, 222,222,14
			IPelement = 0;					// set to 0 to accumulate the next IP element
			elementCnt++;
		}
		else
		{
			//check for only numbers in the IP field
			if(  (value[cntr] < '0') || (value[cntr] > '9') )
			{
				cout<< "\nInvalid value initialized for [RMCPP]-TargetIP";
				iRetVal = -1;
			}
			else
			{
				IPelement = IPelement*10 + value[cntr]-'0';
			}
		}
	}
	IP[elementCnt] = IPelement;					//store the last element

	//Make sure there are only 4 digits in the IP
	if ( (cntr != len) || (elementCnt != 3) )
	{
		//Invalid Syntax : entire IP string has not been parsed, there are more than 4 digits
		iRetVal = -1;
	}

	//Make sure all the 4 digits are not 0
	if( IP[0]==0 && IP[1]==0 && IP[2]==0 && IP[3]== 0)
	{
		//0.0.0.0 is not valid IP
		iRetVal = -1;
	}

	return iRetVal;

}

//*************************************************************************
int DcmiConfiguration::InsertPassword(char pPassword[])
{
	DcmiEntityConfigurationSet* pCurrConfigSet = NULL;
	//char* bracketBegin = NULL;
	//Find the "RMCPP" section in the configuration file to insert a record under this.
	DcmiConfig::iterator it = this->configuration.find("RMCPP");

	if (it == configuration.end())
    {
		pCurrConfigSet = new DcmiEntityConfigurationSet();
        configuration["RMCPP"] = pCurrConfigSet;
	}
    else
    {
        pCurrConfigSet = it->second;
		if (pCurrConfigSet != NULL)
		{
			size_t optionStrLen = strlen("password") + 1;
			size_t valueStrLen = strlen(pPassword) + 1;

			//char* optionStr = new char[optionStrLen];
			std::auto_ptr<char> optionStr(new char[optionStrLen]);
			//char *valueStr = new char[valueStrLen];
			std::auto_ptr<char> valueStr(new char[valueStrLen]);
	
			TransportPlugin::OsSpecificFunctions::strcpys(optionStr.get(), (unsigned int)optionStrLen, "password");
			TransportPlugin::OsSpecificFunctions::strcpys(valueStr.get(), (unsigned int)valueStrLen, pPassword);

			DcmiConfigurationPair* pConfigPair = new DcmiConfigurationPair(optionStr.release(), valueStr.release());

			pCurrConfigSet->insert(pConfigPair);
		}
		else
        {
			printf("Error: Cannot associate 'Password = %s' with any entity.\n",pPassword);
			return -1;
        }

    }
	return 0;

}

int DcmiConfiguration::GetPassword()
{
	char Password[21];
	char  UserInput;
	int  iPwdLen=0;
	const char *Config_UserName, *Config_Cipher;  //Store the username read from configuration file

	/* Password not required for Cipher-0, so check for Cipher-0 selected in config file */
	Config_Cipher = getConfigurationValue("RMCPP","ciphersuite");
	LOG_SUMMARY<<"\nEstablishing RMCP+ session with CipherSuite: "<<Config_Cipher<<endl;
	if( strcmp(Config_Cipher, "0") == 0)
	{
		return 0;  // continue without prompting password from user
	}

	/*Read the UserName initialized in the configuration file
	  to display it to user to enter the corresponding password*/
	Config_UserName = getConfigurationValue("RMCPP","username");

	/*Checks whether the username is "NULL" in the configuration file.*/
	if( (Config_UserName == NULL) || (strlen(Config_UserName) == 0) )
	{
		cout<<"\nUserName in configuration file is 'NULL', proceeding with 'NULL' password to establish connection."<<endl;
		//DelayTime(2); // Time allowed for user to read the above statement.
	}
	else
	{
		/* Read the User Password initialized in the configuration file
		 * or ask for one, ask user to enter the corresponding password*/
		const char *Config_UserPass = getConfigurationValue("RMCPP","password");
			/*Checks whether the password is "NULL" in the configuration file.*/
		if( (Config_UserPass == NULL) || (strlen(Config_UserPass) == 0))
		{

			cout<<"\nEnter Password corresponding to UserName <" << Config_UserName <<">, press 'ENTER'"<<endl;
			cout<<"PASSWORD:  " <<endl;

#ifndef WIN32
			//Trim off the previous inputs in the buffer in Linux as fflush(stdin) is not supported in Linux
			do
			{
				UserInput = getchar();
			}
			while( UserInput != '\n' && UserInput != EOF);

#endif
			/*Maximum length allowed for password is 20 characters as per IPMI Spec 2.0
		  Read the user input until he hit "Enter" or 20 characters.*/
			while(iPwdLen < 20)
			{
				fflush(stdin);
#ifdef WIN32
				UserInput = _getch();
#else
				UserInput = mygetch();
#endif
				fflush(stdin);

#ifdef WIN32
				if(UserInput == 13)//Carriage Return
#else
					if(UserInput == 10) //Carriage Return in Linux
#endif
					{
						Password[iPwdLen] = '\0';
						break;
					}
					else if(UserInput == 8)//Backspace
					{
						/*Decrement the pointer if user hits backspace while entering his input.*/
						if (iPwdLen > 0)
						{
							iPwdLen--;
							cout<<"\b \b";
						}
					}
					else if (UserInput >= 0 ) //char is in range -127 to 127 && UserInput <= 255
					{
						Password[iPwdLen] = UserInput;
						iPwdLen++;
						cout<<"*";
					}


			}
			/*Insert the password into the RMCPP set to make the connection using
	      	  the user inputed password value.*/
			InsertPassword(Password);
		}
	}

	return 0; //Success

}


/*************************************************************************
Function Declaration
int ConvertVerbosity(LogLevel* pLogLevel, const char* logLevelString)
Function Name:		ConvertVerbosity
Purpose:			Converts verbosity level from string to enum value
Parameter:			Pointers to verbosity
					[IN] logLevelString
					[OUT] pLogLevel
Returns:			0 upon successful conversion, 1 otherwise
*************************************************************************/

int DcmiConfiguration::ConvertVerbosity(TransportPlugin::LogLevel* pLogLevel, const char* logLevelString)
{
    if (OsSpecificFunctions::stricmp(logLevelString, "no") == 0)
    {
       *pLogLevel = TransportPlugin::logLevelErrors;
    }
    else if (OsSpecificFunctions::stricmp(logLevelString, "summary") == 0)
    {
        *pLogLevel = TransportPlugin::logLevelSummary;
    }
    else if (OsSpecificFunctions::stricmp(logLevelString, "details") == 0)
    {
        *pLogLevel = TransportPlugin::logLevelDetails;
    }
    else if (OsSpecificFunctions::stricmp(logLevelString, "debug") == 0)
    {
        *pLogLevel = TransportPlugin::logLevelDebug;
    }
    else if (OsSpecificFunctions::stricmp(logLevelString, "info") == 0)
    {
        *pLogLevel = TransportPlugin::logLevelInfo;
    }
    else if (OsSpecificFunctions::stricmp(logLevelString, "warning") == 0)
    {
        *pLogLevel = TransportPlugin::logLevelWarnings;
    }
    else
    {
        return 1;
    }

    return 0;
}

/*********************************************************************************
Function Declaration:
int InitializeLogger(const DcmiConfiguration* pConfiguration, std::ofstream* pLogFile);
Function Name:		InitializeLogger
Purpose:			Initializes logging functionality
Parameters:			[IN]pConfiguration - Pointer to configuration object
                    [IN]pLogFile - pointer to log file object
 Returns:			0 upon successful initlialization
                    1 Error, unable to open logger file.

********************************************************************************/
int DcmiConfiguration::InitializeLogger(std::ofstream* pLogFile)
{
    // Obtain configuration of screen logger.
    const char* logScreenVerbosity = getConfigurationValue("DCMI_Conformance", "screen-output");
    if (logScreenVerbosity == NULL)
    {
        printf("Configuration file error: Cannot find 'screen-output' option in the [DCMI_Conformance] section.");
        return 1;
    }

    // Obtain configuration of file logger.
    const char* logFileName = getConfigurationValue("DCMI_Conformance", "log-file-name");
    const char* logFileVerbosity = getConfigurationValue("DCMI_Conformance", "log-to-file");
    if ((logFileVerbosity == NULL) && (logFileName != NULL))
    {
        if (strlen(logFileName) > 0)
        {
            printf("Configuration file error: Cannot find 'log-to-file' option in the [DCMI_Conformance] section.");
            return 1;
        }
    }

   // Each log message of the main application will be prefixed with "Main App".
   // Logger::setLogPrefix("Main App");//SM Commented to Display Asset tag without the prefix [Main App]

    // Initialize screen logger.
    TransportPlugin::LogLevel logLevel;
    if (ConvertVerbosity(&logLevel, logScreenVerbosity) != 0)
    {
        printf("Configuration file error: Incorrect value of 'screen-output' option: %s.", logScreenVerbosity);
        return 1;
    }
    // For screen optput verbosity=no is not supported.
    if (logLevel == TransportPlugin::logLevelErrors)
    {
        logLevel = TransportPlugin::logLevelSummary;
    }
    TransportPlugin::Logger::setOutputStream(std::cout, logLevel);

    // From this point we can stop using the printf function for displaying messages.

    // Initialize file logger.
    if (logFileName != NULL)
    {
        if (strlen(logFileName) > 0)
        {
            if (ConvertVerbosity(&logLevel, logFileVerbosity) != 0)
            {
                LOG_ERR << "\nConfiguration file error: Incorrect value of 'log-to-file' option: " << logFileVerbosity << ".";
                return 1;
            }

            if (logLevel != TransportPlugin::logLevelErrors)
            {
                pLogFile->open(logFileName, std::ios::app);
                if (!(*pLogFile))
                {
                    LOG_ERR << "\nFailed to open log file: " << logFileName << ".";
                    return 1;
                }
                TransportPlugin::Logger::setOutputStream(*pLogFile, logLevel);
            }
        }
    }

    return 0;
}

