/*******************************************************************************
********************************************************************************
***                                                                           **
***                         INTEL CONFIDENTIAL                                **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***                         All Rights Reserved                               **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***     The source code contained or described herein and all documents       **
***     related to the source code ("Material") are owned by Intel            **
***     Corporation or its suppliers or licensors. Title to the Material      **
***     remains with Intel Corporation or its suppliers and licensors.        **
***     The Material contains trade secrets and proprietary and confidential  **
***     information of Intel or its suppliers and licensors.                  **
***     The Material is protected by worldwide copyright and trade secret     **
***     laws and treaty provisions. No part of the Material may be used,      **
***     copied, reproduced, modified, published, uploaded, posted,            **
***     transmitted, distributed, or disclosed in any way without Intel�s   **
***     prior express written permission.                                     **
***                                                                           **
***     No license under any patent, copyright, trade secret or other         **
***     intellectual property right is granted to or conferred upon you by    **
***     disclosure or delivery of the Materials, either expressly, by         **
***     implication, inducement, estoppel or otherwise. Any license under     **
***     such intellectual property rights must be express and approved by     **
***     Intel in writing.                                                     **
********************************************************************************/
/********************************************************************************
***      File       :   DCMI_Discovery_lib.cpp                                **
***                                                                           **
***      Abstract   :   Implements the Discovery class                        **
***                              **
***                                                                           **
***
********************************************************************************/
#include "parameters.h"
#include "declarations.h"
#include "DCMI_Configuration.h"
#include "DCMI_Discovery_lib.h"

const static byte_t ASSETTAG_CC [] = {0x00, 0x80, 0x81, 0x82, 0x83};

//Prints the Capability summary of platform from the corresponding structures ..
/**
 * Print info
 *
 * @param stream - stream to print out data
 * @param ob - object to print out
 *
 * @return stream
 */
std::ostream &operator<<(std::ostream &stream, s_supported_DCMI_capab &s_DCMICapab)
{
	//Optional Platform capability
	if(s_DCMICapab.sopt_capab.bPowerMgmt == 1)
		stream<<"\n     Power Management\t\t\t-\tSupported.";
	else
		stream<<"\n     Power Management\t\t\t-\tNOT Supported.";

	//Manageability Access capability
	if(s_DCMICapab.smgmt_capab.bKCSpresent == 1)
		stream<<"\n     In-band System Interface\t\t-\tAvailable ";
	else
		stream<<"\n     In-band System Interface\t\t-\tNOT Available \t";

	if(s_DCMICapab.smgmt_capab.bOOBTmodePresent == 1)
		stream<<"\n     Out-Of-Band Serial TMODE\t\t-\tAvailable";
	else
		stream<<"\n     Out-Of-Band Serial TMODE\t\t-\tNOT Available";

	if(s_DCMICapab.smgmt_capab.bOOBSecLanPresent == 1)
		stream<<"\n     Out-Of-Band Secondary LAN\t\t-\tAvailable";
	else
		stream<<"\n     Out-Of-Band Secondary LAN\t\t-\tNOT Available";

	return stream;
}

/**
 * Print info
 *
 * @param stream - stream to print out data
 * @param ob - object to print out
 *
 * @return stream
 */
std::ostream &operator<<(std::ostream &stream, s_Min_Platform_Attrib &s_MinAttrib)
{
	//Minimum Platform Attributes
	//SEL
	if(s_MinAttrib.uSELAttrib[1] & 0x80)
		stream<<"\n     SEL automatic Rollover\t\t-\tEnabled";
	else
		stream<<"\n     SEL automatic Rollover\t\t-\tDisabled";

	return stream;
}

/**
 * Print info
 *
 * @param stream - stream to print out data
 * @param ob - object to print out
 *
 * @return stream
 */
std::ostream &operator<<(std::ostream &stream, s_Mgmt_Access_Attrib &s_MgmtAttrib)
{
	if(s_MgmtAttrib.PrimChannelNum != 0xFF)
		stream<<"\n     Primary LAN Out-Of-Band\t\t-\tSupported";
	else
		stream<<"\n     Primary LAN Out-Of-Band\t\t-\tNOT Supported";

	//Manageability Access Attributes
	if(s_MgmtAttrib.SecChannelNum != 0xFF)
		stream<<"\n     Secondary LAN Channel number\t-\t"<<std::hex<<int(s_MgmtAttrib.SecChannelNum);
	else
		stream<<"\n     Secondary LAN Channel number\t-\tNOT Supported.";
	//Serial Out-Of-Band TMODE Capability
	if(s_MgmtAttrib.TMODEChannelNum != 0xFF)
		stream<<"\n     Serial OOB TMODE Support Channel\t-\t"<<std::hex<<int(s_MgmtAttrib.TMODEChannelNum) ;
	else
		stream<<"\n     Serial OOB TMODE Support Channel\t-\tNOT Available";

	return stream;
}


DCMIDiscovery::DCMIDiscovery(int ver, TransportPlugin::TransportPlugIn &transport) :
		DCMIBaseTest("Basic Discovery", transport)
{
	m_Ver = ver;

	std::string errmsg("\n\nError!! DCMI Discovery phase failed - ");

	//make discovery ...
	TestResult cres = TestUtil::TR_SUCCESS;
	try
	{	//Check Basic Platform DCMI spec compliant by Get DCMI Capability Info Command
		//Supported DCMI Capabilities
		//info to dest location for the future use...
		if((cres = PlatformSuppCapab(s_DCMICapab)) == TestUtil::TR_SUCCESS)
		{		//Manageability Access Attributes
			cres = MgmtAccessAttrib(s_DCMICapab, s_MgmtAttrib);
		}//pass information ..
		LOG_SUMMARY<<"\n";
	}
	catch(TestUtil::TestResult &result) {
		cres = result;
		LOG_ERR<<errmsg<<result;
	}
	catch(std::exception &e) {
		cres = TestUtil::TR_FAILURE;
		LOG_ERR<<errmsg<<e.what();
	}
	catch ( ... ) {
		cres = TestUtil::TR_FAILURE;
		LOG_ERR<<errmsg<<"Unknown exception occurred.";
	}

	if (cres != TestUtil::TR_SUCCESS)
	{	//Get DCMI Capability info command not supported, abort Test
		throw std::runtime_error("\n\nAborting: Get DCMI Capability Info command not supported, cannot"\
								 "\nproceed to verify capabilities of DCMI Specification."\
								 "\n\nFAIL: Management Controller not in compliance with DCMI Specification.");
	}

}


void DCMIDiscovery::doConfigure(DcmiConfiguration &option)
{
	//configure all dependent packages
	TestSuite::doConfigure(option);
//Extra configuration action ...
//.....
}

/***************************************************************************
Function Name:  getDCMICapabInfoCmd
Purpose      :  calls the local function GetDCMICapabInfoCmd() and stores the
                response in the global structures
Parameters   :  None
Returns      :   command completion status
                    00 - DCMI Capab Info Cmd, and supported capabilities is
                         DCMI compliant - PASS
                    01 - DCMI CApab Info cmd and supported DCMI capabilities
                         not DCMI compliant - FAIL
                    CompCode - The response completion code, in case of command failure
                    TRANSPORT_ERROR -   Error due to network issues in transport layer
***************************************************************************/
void
DCMIDiscovery::getDCMICapabInfoCmd(byte_t param, DCMIClasses::IpmiDCMIResponse &response)
{
    //Build Command Object
    DCMIClasses::IpmiDCMIRequest request(0x01, param);

    LOG_DEBUG<<"\n\tGet Configuration Parameter ("<<std::hex<<std::showbase<<static_cast<int>(param)<<")\n";
	//Issue the command with Parameter selector: param
    sendIpmiCmd(request, response);
}


/*************************************************************************
Function declaration
int PlatformSuppCapab();
Function Name   :   PlatformSuppCapab
Purpose         :   Checks the Global Structure <gsSuppDCMICapab> for 'Supported
                    DCMI Capabilities' obtained from GetDCMICapabilityInfo command
                    in the platform to be DCMI Spec compliant
Parameters      :   None
Returns         :   command completion status. This is the value returned from the
                    transport layer, and reports success or failure while transferring
                    the command across the transport (rmcp+, Kcs etc). The possible return
                    values are
                    00          -   Supported DCMI Capabilities described in Table 6.3 of DCMI spec 1.0
                                    is DCMI compliant
                    01          -   Supported DCMI Capabilities is not DCMI compliant
***************************************************************************/

TestResult
DCMIDiscovery::PlatformSuppCapab(s_supported_DCMI_capab &s_DCMICapab)
{
	LOG_SUMMARY<<"\nChecking Supported DCMI Capabilities for Platform.";

	DCMIClasses::IpmiDCMIResponse response(DCMI_CAPAB_INFO_MIN_RSPLEN, DCMI_CAPAB_INFO_MAX_RSPLEN);
	getDCMICapabInfoCmd(0x01, response);

	//Get Supported DCMI Capabilities
	s_DCMICapab = response.getFromData<s_supported_DCMI_capab>(4);

	//Display to user - in case of DCMI Support Discovery
	LOG_INFO<<"\n\tDCMI Specification conformance Major version is:\t"<<std::hex<<std::showbase<<static_cast<int>(response[1]);
	LOG_INFO<<"\n\tDCMI Specification conformance Minor version is:\t"<<std::hex<<std::showbase<<static_cast<int>(response[2]);
	LOG_INFO<<"\n\tDCMI Specification Parameter Revision is       :\t"<<std::hex<<std::showbase<<static_cast<int>(response[3]);

	//Check if Major and Minor versions are valid.

    //c-p from old dcts, bellow does it make sense?? Will give this same byte on the upper and lower byte of int
	//if (((static_cast<int>(response[1]) << 8) + (static_cast<int>(response[1]))) != IPMI_DCMI_1_1_CONFORM) {
	int ver = ((static_cast<int>(response[1]) << 8) + (static_cast<int>(response[2])));
	if (ver != m_Ver) //if current version is bigger that expected if lower maybe it's ok...
	{ //do we really fail the test ...??
		LOG_ERR<<"\n\n\n\tFailed! Returned Version: "<<std::hex<<std::showbase<<ver<<" is Incorrect, expected "<<std::hex<<std::showbase<<m_Ver<<"!";
		return TestUtil::TR_FAILURE;
	}

	//Check if Platform Supported Capabilities is DCMI compliant
	TestResult  resComp = TestUtil::TR_SUCCESS;

	//Optional Platform Capability as PowerManagement is optional
	LOG_INFO<<"\n\n\tOptional Platform Capabilities in Power Management feature is"
			<< ((s_DCMICapab.sopt_capab.bPowerMgmt == 0) ? " not" : "") <<" supported.";

	//Manageability Access Capabilities
	LOG_INFO<<"\n"
			<<"\n\tInband Channel bit            - "<<static_cast<int>(s_DCMICapab.smgmt_capab.bKCSpresent);
	LOG_INFO<<"\n\tOOB TMode  bit                - "<<static_cast<int>(s_DCMICapab.smgmt_capab.bOOBTmodePresent);
	LOG_INFO<<"\n\tOOB Sec LAN Channel Available - "<<static_cast<int>(s_DCMICapab.smgmt_capab.bOOBSecLanPresent);

	if (s_DCMICapab.smgmt_capab.bKCSpresent==0)
	{
		LOG_WARN<<"\n\tPlatform Manageability Access Capabilities is NOT Compliant with DCMI Specification.";
		resComp = TestUtil::TR_FAILURE;
	}
	else
	{
		LOG_INFO<<"\n\tPlatform Manageability access Capabilities is Compliant with DCMI Specification.";
	}

	return resComp;
}


TestResult
DCMIDiscovery::PlatformSuppCapabSessionLessTest(any_value &)
{
	LOG_SUMMARY<<"\nSession Less Test.";

	if (!isRemoteConnection())
	{
		return TestUtil::TR_SKIPPED;
	}

	std::string errmsg("\n\nError!!Session Less DCMI Discovery phase failed - ");

	getCurrentTransport().closeSession();

	msleep(150);

	TestResult resComp = TestUtil::TR_SUCCESS;
	s_supported_DCMI_capab s_DCMICapab = {0};

	try {
		resComp = PlatformSuppCapab(s_DCMICapab);
	}
	catch(TestUtil::TestResult &result) {
		resComp = result;
		LOG_ERR<<errmsg<<result;
	}
	catch(std::exception &e) {
		resComp = TestUtil::TR_FAILURE;
		LOG_ERR<<errmsg<<e.what();
	}
	catch ( ... ) {
		resComp = TestUtil::TR_FAILURE;
		LOG_ERR<<errmsg<<"Unknown exception occurred.";
	}

	getCurrentTransport().openSession();
	return resComp;
}



TestResult
DCMIDiscovery::MinPlatformAttrib(s_Min_Platform_Attrib &s_MinAttrib)
{
	LOG_DEBUG<<"\nChecking Minimum Platform Attributes.";
   //Send request data to the transport critical error are reported by exception
	s_MinAttrib = getDCMICapabInfoCmd<s_Min_Platform_Attrib>(0x02);
	 //Minimum Platform Attributes

    //System Event Logging - populate SEL details to s_SELData structure
	//Calculate maximum number of SEL entries in system
	int SEL_Entry = s_MinAttrib.uSELAttrib[1];

	SEL_Entry = SEL_Entry & 0x0F;
	SEL_Entry = SEL_Entry << 8;
	SEL_Entry = SEL_Entry + s_MinAttrib.uSELAttrib[0];

	LOG_INFO<<"\n\tSEL Capabilities:";
	LOG_INFO<<"\n\tNumber of SEL entries is                       :\t"<<SEL_Entry;

	// Verify SEL Attributes to be spec compliant
	if((s_MinAttrib.uSELAttrib[1]&0x80) == 0)
	{
		LOG_WARN<<"\n\tPlatform SEL Roller Feature is not supported.";
	}
	else
	{
		if(SEL_Entry<64)
		{
			LOG_WARN<<"\n\tSupported Number of SEL Entries < 64. Not Compliant with DCMI Specification.";
			return TestUtil::TR_FAILURE;
		}
	}

	return TestUtil::TR_SUCCESS;
}

TestResult
DCMIDiscovery::OptPlatformAttrib(s_Opt_Platform_Attrib &s_OptAttrib)
{
	//Optional Platform Attributes
	LOG_DEBUG<<"\nChecking Optional Platform Attributes.";
	s_OptAttrib = getDCMICapabInfoCmd<s_Opt_Platform_Attrib>(0x03);
	//Since these are optional no check is required, command completed successfully
    //so log success
	return TestUtil::TR_SUCCESS;
}

TestResult
DCMIDiscovery::MgmtAccessAttrib(s_supported_DCMI_capab &s_DCMICapab, s_Mgmt_Access_Attrib &s_MgmtAttrib)
{
	//Check if Manageability attributes are DCMI compliant
	LOG_SUMMARY<<"\nChecking Manageability Access Attributes.";
	//Manageability Access Attributes
	s_MgmtAttrib = getDCMICapabInfoCmd<s_Mgmt_Access_Attrib>(0x04);

	TestResult  resComp = TestUtil::TR_SUCCESS;
	//Check for the primary channels supported, the channel number is valid
	if (s_MgmtAttrib.PrimChannelNum == 0xFF)
	{
		LOG_ERR<<"\n\tPrimary Channel is not supported. Not Compliant with DCMI Specification";
		resComp = TestUtil::TR_FAILURE;
	} else {
		LOG_INFO<<"\n\tPrimary LAN channel number is                  :\t0x"<<std::hex<<int(s_MgmtAttrib.PrimChannelNum);
	}

	//Check for valid Secondary Channel number if platform supports Secondary Channel
	if (s_DCMICapab.smgmt_capab.bOOBSecLanPresent == 1)
	{
		if (s_MgmtAttrib.SecChannelNum  == 0xFF)
		{
			LOG_ERR<<"\n\tSecondary Channel is supported, but Channel # is not valid";
			resComp = TestUtil::TR_FAILURE;
		} else {
			LOG_INFO<<"\n\tSecondary LAN channel number is                :\t0x"<<std::hex<<(int)s_MgmtAttrib.SecChannelNum ;
		}
	}
	return resComp;
}

TestResult
DCMIDiscovery::SystemPowerAttrib(s_supported_DCMI_capab &s_DCMICapab, s_System_Power_Attrib &s_SysPowerAttrib)
{
	//Check if Manageability attributes are DCMI compliant
	LOG_DEBUG<<"\nChecking Enhanced System Power Attributes.";

	//Enhanced System Power Attributes
	DCMIClasses::IpmiDCMIResponse response(DCMI_CAPAB_INFO_MIN_RSPLEN, DCMI_CAPAB_INFO_MAX_RSPLEN);
	getDCMICapabInfoCmd(0x05, response);

	//Get Supported DCMI Capabilities
	s_SysPowerAttrib.NumOfRollingPeriod = response[4];
	response.copy_to(5, s_SysPowerAttrib.RollingPeriods,
			std::min<size_t>(s_SysPowerAttrib.NumOfRollingPeriod, sizeof(s_SysPowerAttrib.RollingPeriods)));

	TestResult  resComp = TestUtil::TR_SUCCESS;
	//Check for the power management supported
	if (s_DCMICapab.sopt_capab.bPowerMgmt == 1)
	{
		if (s_SysPowerAttrib.NumOfRollingPeriod == 0)
		{
			LOG_ERR<<"\n\tPower management supported, but the Number Of Rolling Average Time Periods is 0";
			resComp = TestUtil::TR_FAILURE;
		} else {
			LOG_INFO<<"\n\tNumber Of Rolling Average Time Periods is                  :\t0x"<<std::hex<<int(s_SysPowerAttrib.NumOfRollingPeriod);
		}
	}
	return resComp;
}
/********************************************************************************************************************************/


TestResult
DCMIDiscovery::CapabilitySumaryInfo(TestUtil::TestOutput &out)
{
	//Display the Capability Summary in a Readable Format - Supported / NOT Supported.
	std::stringstream str ;
		str <<"\n" << getCapbilities()
		<< (s_MinAttrib)
		<< getMgmtAccessAttrib()
		<< "\n\n";

	out.raw_print(str.str());
	return TestUtil::TR_SUCCESS;
}

/**************************************************************************
Function Name	:	GUIDTest
Purpose			:   Checks platform compliance for Get Device GUID command, only
					if the platform supports the GUID feature. Also, checks if this
					is mandatory feature. Updates the test result in iConf_Result array
					at index SYSTEM_GUID_TEST
**************************************************************************/
TestResult
DCMIBasics::GUIDTest(any_value &)
{
	LOG_DEBUG<<"\nGet Device GUID from UUT.";
    //Build Command Object
	//Net Function	APPLICATION;
	//Command for Get Device GUID 0x37;
	CommonClasses::IpmiRequest request(APPLICATION, 0x37);
	//Build Command Object
	CommonClasses::IpmiResponse response(GET_DEV_GUID_MIN_RSPLEN, GET_DEV_GUID_MAX_RSPLEN);

	setSessionPrivilegeLevel(PRIV_USER);

    //Send request data to the transport
	sendIpmiCmd(request, response);
	//If valid completion code display giud data to the user.
	LOG_INFO<<"\n\tGet Device GUID returned: ";
   std::copy(&response.at(0), &response.at(response.getLength()), std::ostream_iterator<int>((LOG_INFO<<std::hex), " "));

	return TestUtil::TR_SUCCESS;
}

/**************************************************************************
Function Name	:	GetAssetTag
Purpose			:	Issues and processes the GetAssetTag command.
Parameters      :   Per the DCMI document Offset and Number of Bytes
					[IN] Offset  - Offset of AssetTag to read from
					[IN] NumBytes- Number of bytes to read from offset
					[OUT] resp_data - The response data with the
					assettag, number of bytes read etc. Data based upon
					requested discovery input
***************************************************************************/
void
DCMIBasics::getAssetTag(byte_t Offset, byte_t NumBytes, std::vector<byte_t> &resp_data)
{
	//Build Command Object
	//Issue the command with Parameter selector: param
	DCMIClasses::IpmiDCMIRequest request(0x06);

	request.append<byte_t>(Offset)		//Offset of Asset Tag to retrieve
		   .append<byte_t>(NumBytes);	//Number of bytes to retrieve from Asset Tag - KS. it look like it is not number but end offset

	DCMIClasses::IpmiDCMIResponse response(GET_ASSET_MIN_RSPLEN, GET_ASSET_MAX_RSPLEN, ASSETTAG_CC, ARRAYSIZE(ASSETTAG_CC));

	//Send request data to the transport critical error are reported by exception
	sendIpmiCmd(request, response);

	//skip only group ext id, asset tag length remains on the first position ...
	response.copy_to(1, resp_data);

	//Make sure length of bytes from assetTag read does not exceed 16 as per Spec
	//minus the length
	if (resp_data.size() > 17)
	{
		LOG_WARN<<"\nResponse data exceed specification size: "<< std::dec<<static_cast<int>(resp_data.size()-1)<<" truncated!";
		resp_data.resize(17);
	}
}

/***************************************************************************
Function Name:	IssueAssetTagCmd
Purpose      :	Reads the entire AssetTag with the GetAssetTag command.
				Checks only valid [-Ascii-] values are used in the AssetTag
Returns         :   command completion status

***************************************************************************/
TestResult
DCMIBasics::IssueAssetTagTest(any_value &)
{
	LOG_DEBUG<<"\nIssue Get AssetTag command";
	setSessionPrivilegeLevel(PRIV_USER);

	std::vector<byte_t> atchunk;
	//Get the Asset Tag length
	//Send request data to the transport critical error are reported by exception
	getAssetTag(0, 0, atchunk);
	if (atchunk.size() == 0)
	{	//Asset Tag not programmed, display warning and exit test
		LOG_WARN << "\n\tAsset Tag size not programmed in platform or incorrect size.";
		return TestUtil::TR_SUCCESS; //skipped
	}

	byte_t AssetLength = atchunk[0]; //Store Asset Tag length returned in response
	if (AssetLength == 0)
	{	//Asset Tag not programmed, display warning and exit test
		LOG_WARN<<"\n\tAsset Tag not programmed in platform. Program AssetTag and retry";
		return TestUtil::TR_SUCCESS; //skipped
	}

	if (AssetLength > ASSET_TAG_MAX_LEN )
	{//Maximum length of characters allowed as per spec for Asset Tag is ASSET_TAG_MAX_LEN
		LOG_ERR<<"\n\tAsset Tag Length 0x"<<std::hex<<int(AssetLength)<<" is greater than 64";
		LOG_ERR<<"\n\tChecking only first 64 bytes of AssetTag";
		AssetLength = ASSET_TAG_MAX_LEN;
	} else {
  	    LOG_DEBUG<<"\n\tAsset Tag Data Length = 0x"<<std::hex<<static_cast<int>(AssetLength);
	}

	//AssetTag array
	std::vector<byte_t> assettag;
	//Read the entire AssetTag and store it in stream
	while(assettag.size() < AssetLength)
	{	//Since data can be read in multiples of 16, unless the total length left is < 16
		//Issuing GetAssetTag command for getting the chunk of Data
		getAssetTag(static_cast<byte_t>(assettag.size()), static_cast<byte_t>(std::min<size_t>(AssetLength - assettag.size(), GET_ASSET_TAG_MAX_BYTES_TO_READ)), atchunk);
		//remove first element - length of returned data.
		atchunk.erase(atchunk.begin());
		//Store the characters read, to the end of the vector previously read
		assettag.insert(assettag.end(), atchunk.begin(), atchunk.end());
	}//end with asset array


	LOG_INFO<<"\n\tAsset Tag Data Read is: ";
	std::copy(assettag.begin(), assettag.end(), std::ostream_iterator<int>(LOG_INFO<<std::hex<<std::showbase, " "));

	return TestUtil::TR_SUCCESS;
}

