/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2011 INTEL CORPORATION                       **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/********************************************************************************
***		 File		:	DCMI_Misc_lib.cpp								      **
***																			  **
***		 Abstract	:	Implements Cipher tests class					      **
***																		      **
***		 Contents	:														  **
***				Definition of the CipherTest class							  **
***				 											  				  **
********************************************************************************/

#include "parameters.h"
#include "DCMI_Configuration.h"
#include "DCMI_Misc_lib.h"

using namespace TransportPlugin;

//Cipher list supported in table-6 of DCMI 1.1 spec
//PLEASE make sure in sort order !!!
const static byte_t m_Spec_1_1_CipherList[] = { 3, 8, 17 };


void CipherTest::doConfigure(DcmiConfiguration &option)
{
}

std::vector<byte_t> &CipherTest::specificationCiphers()
{
	if(m_SpecCiphers.empty())
		m_SpecCiphers = std::vector<byte_t>(m_Spec_1_1_CipherList, m_Spec_1_1_CipherList + sizeof(m_Spec_1_1_CipherList));
	return m_SpecCiphers;
}

TestResult CipherTest::PlatformCipherSuitesTest(any_value &)
{
	LOG_DEBUG<<"\nIssue Get Platform Cipher Suites and validate it per DCMI spec";
	std::vector<byte_t> pc; //platform ciphers
	getPlatformCipherSuites(pc);
	std::vector<byte_t> v = intersectCiphers(pc);
	return validateCipherSuites(v);
}

/*************************************************************************
Function declaration
int getChannelCipherSuites(int iPayloadType, int iListIndex,  std::vector<byte_t> &resp_data)

Function Name   :	getChannelCipherSuites
Purpose         :	Issues the 'Get Channel CipherSuite' command
                    to obtain the Ciphersuites supported by platform.
Parameters      :
                    [IN] iPayloadType
		    [IN] iListIndex
		    [OUT] resp_data - The response data consists of the response
                    as per IPMI spec
In case of problems, reports failure by rising exception while transferring
the command across the transport (rmcp+, Kcs, heci etc),
***************************************************************************/
void
CipherTest::getChannelCipherSuites(byte_t iPayloadType, byte_t iListIndex, std::vector<byte_t> &resp_data)
{
	CommonClasses::IpmiRequest request(APPLICATION, 0x54);
	//Build Command Object
	request.append<byte_t>(m_channelNum)
    			   .append<byte_t>(iPayloadType)
    			   .append<byte_t>(iListIndex);

	CommonClasses::IpmiResponse response(GET_CHANNEL_CIPHER_MIN_RSPLEN, GET_CHANNEL_CIPHER_MAX_RSPLEN);
	//Send request data to the transport
	sendIpmiCmd(request, response);

	response.copy_to(0, resp_data);
}


/*************************************************************************
 Function declaration
 void getPlatformCipherSuites (std::vector<byte_t> &ciphers);
 Parameters:
                    [OUT] ciphers - The response data consists of the pure cipher id list

 Purpose:           Issues the 'Get Channel CipherSuites' command
                    to get list of Ciphersuites supported in Platform.


 Returns:		Void
 *************************************************************************/
void
CipherTest::getPlatformCipherSuites(std::vector<byte_t> &ciphers)
{
	byte_t indexOffset=0;
	byte_t iListIndex, totalCiphers=0;
	int lastByte=0;//, numCiphersSupp=0, cipherFound=0;

	std::vector<byte_t> response;
	//Issue Get Channel CipherSuite Command
	//as long as response length is more than 2 bytes
	do
	{
		iListIndex =  0x80;  // initialise for List algorithms by CipherSuite
		iListIndex |= indexOffset;

		//Issue the command for IPMI payload, and obtain list algorithms by CipherSuite for primary LAN Channel
		getChannelCipherSuites(00, iListIndex, response);
		//Process returned Data
		//Search thru the result data for 0xC0 or 0xC1, and store the following Cipher number in the list
		//looks bad - copy-pasted from old DCTS version
		for(size_t i=1; i < response.size(); i++)
		{
			if(lastByte)
			{
				//The last byte from last command was C0, so store the first byte
				ciphers.push_back(response[i]); // store Cipher number in Array
				totalCiphers++;
				i++;
				lastByte=0;
			}
			if((response[i] == 0xC0) || (response[i] == 0xC1))
			{
				if(i == (response.size() - 1))
				{ //This is the last byte - so remember it
					lastByte = 1;
				}
				else
				{ //C0, C1 record header found, store the next byte - the cipher number
					//in platformCipherList array
					i++; // move to next byte i.e Cipher number
					ciphers.push_back(response[i]); // store Cipher number in Array
					totalCiphers++;
				}
			}
		}//for

		indexOffset++;  // Retrieve the next 16 bytes
	} while (response.size() == 17 && totalCiphers < MAX_CIPHER_SUPP);
}

std::vector<byte_t>
CipherTest::intersectCiphers(std::vector<byte_t> &ciphers)
{
	//Verify if ALL the Ciphers in the spec are in the ciphers array
	//For each CipherSuite in the list of DCMI supported CipherSuites,
	//check if corresponding entry exists in spec array
	std::sort(ciphers.begin(), ciphers.end()); //sort platform ciphers
	std::vector<byte_t> &spec = specificationCiphers();

	//intersection vector
	std::vector<byte_t> v(ciphers.size()); //reserve some space
	v.erase(std::set_intersection(ciphers.begin(), ciphers.end(), spec.begin(), spec.end(), v.begin()), v.end());

	return v;
}

TestResult
CipherTest::validateCipherSuites(std::vector<byte_t> &ciphers)
{
	//Print summary as to if all Ciphers or only a subset is supported
	std::vector<byte_t> &spec = specificationCiphers();
	if(ciphers.size() == spec.size())
	{
		LOG_INFO<<"\n\tThe Platform supports all the "<<static_cast<int>(spec.size())
					<<" Ciphersuites as per DCMI spec";
	}
	else
	{
		LOG_INFO<<"\n\tThe Platform supports only "<<static_cast<int>(ciphers.size())
					<< "of the \n\t"<< static_cast<int>(spec.size())
					<<"Ciphersuites supported in DCMI spec";
	}

	return TestUtil::TR_SUCCESS;
}

TestResult
CheckCiphersTest::validateCipherSuites(std::vector<byte_t> &ciphers)
{
	CipherTest::validateCipherSuites(ciphers);

	LOG_DEBUG<<"\n\tPrepare Tests for each Cipher";
	for(std::vector<byte_t>::iterator it = ciphers.begin(); it != ciphers.end(); it++ )
	{
		byte_t cipherId = *it;
		std::stringstream msg;
		msg<<"Cipher Test id: "<<"\""<<std::dec<<static_cast<int>(cipherId)<<"\"";

		ADD_TEST_PARAM(CheckCiphersTest::ChiperTest, cipherId, msg.str(), TestUtil::TT_OPTIONAL);
	}
    return TestUtil::TR_SUCCESS;
}


TestResult
CheckCiphersTest::ChiperTest(any_value &cipherid)
{
	if(cipherid.empty())
	{
		LOG_ERR<<"\nIncorrect parameter for Chipher test";
		return TestUtil::TR_NONE;
	}

	std::stringstream ss;
	ss <<std::dec<<static_cast<int>(cipherid.value<byte_t>());
	std::string cipher = ss.str();

	LOG_DEBUG<<"\nCipher: "<<cipher<<" Test";

	if (getCurrentTransport().isConnected())
		getCurrentTransport().disconnect();

	getCurrentTransport().configure("ciphersuite", cipher);

	if(getCurrentTransport().connect())
	{ //failed to connect
		LOG_ERR<<"\nFailed to connect to Cipher - "<<std::dec<<cipher<<"\n";
		return TestUtil::TR_FAILURE;
	}

   any_value anyv;
   return DeviceIdTest(anyv);
}

//*************************************************************************
/*
void setstdinEcho(bool enable = true)
{
#ifdef WIN32
    HANDLE handle = GetStdHandle(STD_INPUT_HANDLE);
    DWORD mode;
    GetConsoleMode(handle, &mode);
    if( !enable )
        mode &= ~ENABLE_ECHO_INPUT;
    else
        mode |= ENABLE_ECHO_INPUT;

    SetConsoleMode(handle, mode);

#else
    struct termios ttatr;
    tcgetattr(STDIN_FILENO, &ttatr);
    if( !enable )
        ttatr.c_lflag &= ~(ICANON | ECHO);
    else
        ttatr.c_lflag |= (ICANON | ECHO);

    tcsetattr(STDIN_FILENO, TCSANOW, &ttatr);
#endif
}
*/


