/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intel�s	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*******************************************************************************
***		 File		:	DCMI_SDR_lib.cpp									  **
***																			  **
***		 Abstract	:	Implements low level procedures to send the SDR       **
***				        commands in the "Sensor & Storage  commands"          **
***                     in Table6.1 "Platform Command Definition"            **
***                     of DCMI 1.0 spec. Also implements                     **
***						the test procedures to validate  the same commands    **
***                     in the DCMI_Conformance Module.                      **
***																		      **
***		 Contents	:														  **
***				Definition of following SDRBase class 	  					  **
********************************************************************************/
#include "parameters.h"
#include "DCMI_Configuration.h"
#include "DCMI_SDR_lib.h"

using std::hex;
using std::showbase;
using std::dec;

/*************************************************************************
Function Name	: GetSDRRepositoryInfo
Purpose			: Sends the IPMI command sequence to the designated interface
to get chassis capabilities of the UUT, primarily used to get information
about main chassis management functions present on IPMB and what address
used to access these functions.

Parameters      :	[OUT] resp_data - The response data consists of the
                    command response as per IPMI spec.
**************************************************************************/
void
SDRBase::getSdrRepositoryInfo(std::vector<byte_t> &resp_data)
{
    CommonClasses::IpmiRequest request(STORAGE, 0x20);
    //Build Command Object
    CommonClasses::IpmiResponse response(GET_SDR_REP_INFO_MIN_RSPLEN,GET_SDR_REP_INFO_MAX_RSPLEN);
    //Send request data to the transport
    //Process & Verify Completion Code errors are reported by exception
    sendIpmiCmd(request, response);
    response.copy_to(0, resp_data);
}

/**************************************************************************
Function declaration
void SDRRepositoryInfoCheck ( int *Conf_Result)
Function Name	:	SDRRepositoryInfoCheck

Purpose			:   Executes the Get SDR Repository Info command and display
					the response structure to User and result in the Conf_Result array
****************************************************************************/
TestResult
SDRTest::SDRRepositoryInfoCheckTest(any_value &)
{
	LOG_DEBUG<<"\nSDR Repository Info Test.";
	std::vector<byte_t> sRspData;
	//Read the SDR Repository Info
	//Issue the command for IPMI payload, and obtain
	getSdrRepositoryInfo(sRspData);

	////If valid completion code display SDR Repository Info to user.

	LOG_INFO<<"\n\tSDR Version               : "<< hex<<showbase<<int(sRspData[0]);
	//Calculating the FreeSpace and Record Count
	LOG_INFO<<"\n\tRecord Count              : "<< dec<<((int(sRspData[2])<<8) | int(sRspData[1]));
	LOG_INFO<<"\n\tFree Space                : "<< dec<<((int(sRspData[4])<<8) | int(sRspData[3]));
	LOG_INFO<<"\n\tRecent Addition TimeStamp : 0x"<<hex<<int(sRspData[8])<<int(sRspData[7])<<int(sRspData[6])<<int(sRspData[5]);
	LOG_INFO<<"\n\tRecent Erase TimeStamp    : 0x"<<hex<<int(sRspData[12])<<int(sRspData[11])<<int(sRspData[10])<<int(sRspData[9]);
	LOG_INFO<<"\n\tOperation Support         : "<<hex<<showbase<<int(sRspData[13])<<"\n";

	//return Command completion Status
	return TestUtil::TR_SUCCESS;
}
//*************************************************************************

/*********************************************************************************
Function Name:		getSdrValues
Purpose:			Takes an input value of record ID provided by Get DCMI Sensor Info
					command and returns the corresponding SDR record
Parameters:			[IN] recordIdLsb,recordIdMsb : Record ID of the SDR to be obtained
					[IN] reservationIdLsb, reservationIdMsb :
							The reservation ID for the GetSDRValues Command
					[IN] offset - Offset of the SDR to retrieve
					[IN] bytesToRead - number of bytes to read from the Record
					[OUT] sdrReturnValues - SDR parameters necessary to normalize
								Get Sensor Reading
***********************************************************************************/

void
SDRBase::getSdrValues(byte_t recordIdLsb, byte_t recordIdMsb, byte_t reservationIdLsb, byte_t reservationIdMsb,
             byte_t offset, byte_t bytesToRead, s_sdr_data &sdrReturnValues )
{
	CommonClasses::IpmiRequest request(STORAGE, 0x23);
    //Build Command Object
    request.append<byte_t>(reservationIdLsb)
		   .append<byte_t>(reservationIdMsb)
		   .append<byte_t>(recordIdLsb)
		   .append<byte_t>(recordIdMsb)
		   .append<byte_t>(offset)
		   .append<byte_t>(bytesToRead);

    //Build Command Object
    CommonClasses::IpmiResponse response(GET_SDR_MIN_RSPLEN, GET_SDR_MAX_RSPLEN);
    //Loop to get the applicable data fields from the SDR
    //For offset = 0x00 we need byte 8 which is the sensor number
    //For offset = 0x10 we need byte 22 which is the sensor units
    //For offset = 0x20 we need bytes 38, 39 41,42 which are the sensor limits
    for (int i=0; (i < 3); i++)
    {
    	request[4] = (request[4] + i) << 4;

    	//Send request data to the transport then Verify Completion Code
    	sendIpmiCmd(request, response);

        //Get the applicable data fields
        //Sensor number should be in the first 16 bytes of the SDR read, sensor units in
        //the second set of 16 bytes, and the sensor UC, UNC, LC, LNC bytes in the 3rd set
        //of sixteen bytes.  Each iteration of the loop will get a different set of bytes.
        if (i == 0)
        {
        	LOG_DEBUG<<"\n\tTesting Sensor Number                          :\t0x"<<hex<< int (response[9]);
        	sdrReturnValues.senseNumber = (response[9]);
        	sdrReturnValues.sensorType = response[14];    //Sensor Type
        }
        if (i == 1)
        {
        	sdrReturnValues.senseUnits = (response[7]);
        	sdrReturnValues.readMaskValue = (response[4]);
        }
        if (i == 2)
        {
        	sdrReturnValues.unrValue = (response[6]);
        	sdrReturnValues.ucValue  = (response[7]);
        	sdrReturnValues.uncValue = (response[8]);
        	sdrReturnValues.lnrValue = (response[9]);
        	sdrReturnValues.lcValue  = (response[10]);
        	sdrReturnValues.lncValue = (response[11]);
        }
    }//for
}

/*************************************************************************
Function Name	:	reserveSdr
Purpose			:	Reserves the SDR for read operations and returns
					the reservation ID

Parameters		:	[OUT] reservationIdLsb, reservationIdMsb -
					The reservation ID returned by ReserveSdr Command
***************************************************************************/
void
SDRBase::reserveSdr(byte_t &reservIdLsb, byte_t  &reservIdMsb)
{
	CommonClasses::IpmiRequest request(STORAGE, 0x22);
	//Build Command Object
	CommonClasses::IpmiResponse response(RESERVE_SDR_MIN_RSPLEN, RESERVE_SDR_MAX_RSPLEN);
	//Send request data to the transport
	sendIpmiCmd(request, response);

	//Get/Store Reservation ID
	reservIdLsb = (response[0]);
	reservIdMsb = (response[1]);
}

