/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*******************************************************************************
***		 File		:	DCMI_SOL_lib.cpp        							  **
***																			  **
***		 Abstract	:	Implements the SOL Test case                          **
***                      in the DCMI_Conformance Module.                      **
***																		      **
***		 Contents	:														  **
***				Definition of SOLTest class									  **
********************************************************************************/

#include "parameters.h"
#include "DCMI_Configuration.h"
#include "DCMI_SOL_lib.h"

using std::hex;
using std::showbase;

//According to table 26-5, correct no error CC can be: 0x00 and 0x80 (not supported)
const static byte_t SOLCONFIG_CC [] = {0x00, 0x80};

/*************************************************************************
Function Name:	getSolConfig
Purpose		 :	Gets the data for Serial Over Lan configuration for the input parameters provided.
Parameters	 :
                [IN] iSolChanNum - Serial Channel Number
				[IN] iSolParam - Serial Modem Parameter
				[IN] iSetSelect - Set Selector
				[IN] iBlockselect - Block Selector.
				[OUT] resp_data - Response of Get Sol Config command as per IPMI spec
***************************************************************************/
void
SOLTest::getSolConfig(int iSolParam, int iSetSelect, int iBlockSelect, std::vector<byte_t> &resp_data)
{
   	CommonClasses::IpmiRequest request(TRANSPORT, 0x22);
    //Build Command Object
    request.append<byte_t>(m_channelNum)
    	   .append<byte_t>(iSolParam)
    	   .append<byte_t>(iSetSelect)
    	   .append<byte_t>(iBlockSelect);

    CommonClasses::IpmiResponse response(GET_SOL_CONFIG_MIN_RSPLEN, GET_SOL_CONFIG_MAX_RSPLEN, SOLCONFIG_CC, ARRAYSIZE(SOLCONFIG_CC));

    //Send request data to the transport
    sendIpmiCmd(request, response);

    if (response.getCompCode() == 0x80)
    {
    	LOG_WARN <<"\n\tGetting SOL configuration parameter: "<< iSolParam <<" is not supported";
    	throw TestUtil::TR_NOT_SUPPORTED;
    }

    response.copy_to(0, resp_data);
}


/***********************************************************************************
Function Name:	getSolOrigValues
Purpose		 :	Gets the data for Serial Over Lan configuration for the input parameters
				provided.
Parameters	 :
				[IN] iSetSelect - Set Selector
				[IN] iBlockselect - Block Selector.
				[OUT] orig_sol_params - Original parameters
**************************************************************************************/
SOLTest::s_Sol_Orig_Params
SOLTest::getSolOrigValues(int iSetSelect, int iBlockSelect)
{
	std::vector<byte_t> resp_data;
	s_Sol_Orig_Params orig_sol_params = {0};

	LOG_DEBUG<<"\nSOL 1-1: Getting original Serial Over Lan settings for checking SOL support.";
	//Initialize iSolParam variable
	for (int iSolParam = 0; iSolParam < 7; iSolParam++)
	{
		//Send request data to the transport
		LOG_DEBUG<<"\n\tIssue <Get Serial Over Lan Configuration> Command to  get-\n\t\t- information for parameter #"<<hex<<showbase<<(int)iSolParam;
		getSolConfig(iSolParam, iSetSelect, iBlockSelect, resp_data);

		switch(iSolParam)
		{
		case 0:
			// Load struct with return values for Set in Progress
			orig_sol_params.iSetInProg = resp_data[1];
			break;

		case 1:
			// Load struct with return values for SOL Enable
			orig_sol_params.iSolEnable = resp_data[1];
			break;

		case 2:
			// Load struct with return values for SOL Authentication
			orig_sol_params.iSolAuth = resp_data[1];
			break;

		case 3:
			// Load struct with return values for SOL Character Accumulate
			orig_sol_params.iCharAccum[0] = resp_data[1];
			orig_sol_params.iCharAccum[1] = resp_data[2];
			break;

		case 4:
			// Load struct with return values for SOL Retry Intervals
			orig_sol_params.iSolRetry[0] = resp_data[1];
			orig_sol_params.iSolRetry[1] = resp_data[2];
			break;

		case 5:
			// Load struct with return values for SOL Non Volatile Bit Rate
			orig_sol_params.iSolNVBitRate = resp_data[1];
			break;

		case 6:
			// Load struct with return values for SOL Volatile Bit Rate
			orig_sol_params.iSolVolBitRate = resp_data[1];
			break;

		default:

			break;
		}
	}

	return orig_sol_params; //Test Passed
}



/*************************************************************************
Function Name:		SolConfig
Purpose:			Gets and sets the data for Serial Over Lan configuration for the input parameters provided
					and verifies that the values match those parameters defined by the specification.
Returns:			Response indicating pass/fail of the validation of SOL Configuration
*****************************************************************************************/
TestResult
SOLTest::SolConfigTest(any_value &)
{
	LOG_DEBUG<<"\nChecking Serial Over Lan Configuration";
	if(m_channelNum == 0xFF)
	{
		LOG_ERR<<"\nIncorrect Serial Over Lan channel";
		return TestUtil::TR_SKIPPED;
	}

	//Set Session Privilege level to Operator
	setSessionPrivilegeLevel(PRIV_OPERATOR);

	//Define the Serial Over Lan Config Response to load the return value from the response for comparison
	int iSetSelect = 0;
	int iBlockSelect = 0;

	//Get and store original SOL parameters
	//Call function to load the original params struct.
	//Instantiate the SOL parameter object.
	struct s_Sol_Orig_Params orig_sol_params = getSolOrigValues(iSetSelect, iBlockSelect);

	//Print the SOL Configuration obtained from Platform
	LOG_DEBUG<<"\n\tSOL set in progress is set to: " <<hex<<showbase<<(int)orig_sol_params.iSetInProg;
	switch (orig_sol_params.iSetInProg)
	{
	case 0:
		// Display Set Complete
		LOG_DEBUG<<"- Set Complete";
		break;

	case 1:
		// Display Set in progress
		LOG_DEBUG<<"- Set in Progress";
		break;
	default:

		break;
	}

	LOG_DEBUG<<"\n\tSOL enable is set to: " <<hex<<showbase<<(int)orig_sol_params.iSolEnable;

	switch (orig_sol_params.iSolEnable)
	{
	case 0:
		// Display Set Complete
		LOG_DEBUG<<"- SOL is disabled.";
		break;

	case 1:
		// Display Set in progress
		LOG_DEBUG<<"- SOL is enabled";
		break;
	default:

		break;
	}

	LOG_DEBUG<<"\n\tSOL Authentication is set to: "<<hex<<showbase<<(int)orig_sol_params.iSolAuth;
	switch (orig_sol_params.iSolAuth & 0x0F)
	{
	case 2:
		// Display Set Complete
		LOG_DEBUG<<"- User Level";
		break;

	case 3:
		// Display Set in progress
		LOG_DEBUG<<"- Operator Level";
		break;

	case 4:
		// Display Set in progress
		LOG_DEBUG<<"- Admin Level";
		break;

	default:

		break;
	}
	LOG_DEBUG<<"\n\tSOL Character Accumulate:  ";
	LOG_DEBUG<<"\n\t\tInterval is set to (milliseconds): "<<(int)orig_sol_params.iCharAccum[0];
	LOG_DEBUG<<"\n\t\tCharacter Send is set to: "<<(int)orig_sol_params.iCharAccum[1];
	LOG_DEBUG<<"\n\tSOL Retry: ";
	LOG_DEBUG<<"\n\t\tRetry Count is set to: "<<(int)orig_sol_params.iSolRetry[0];
	LOG_DEBUG<<"\n\t\tRetry Interval is set to (milliseconds):"<<(int) orig_sol_params.iSolRetry[1];
	LOG_DEBUG<<"\n\tSOL Non-Volatile bit rate is set to: "<<hex<<showbase<<(int)orig_sol_params.iSolNVBitRate;

	switch (orig_sol_params.iSolNVBitRate)
	{
	case 7:
		// Display Set Complete
		LOG_DEBUG<<"- 19.2Kbps.";
		break;

	case 8:
		// Display Set in progress
		LOG_DEBUG<<"\t 38.4 Kbps";
		break;

	case 9:
		// Display Set in progress
		LOG_DEBUG<<"\t 57.6 Kbps";
		break;

	case 10:
		// Display Set in progress
		LOG_DEBUG<<"\t 115.2 Kbps";
		break;
	default:

		break;
	}


	LOG_DEBUG<<"\n\tSOL Volatile bit rate is set to: "<<hex<<showbase<<(int)orig_sol_params.iSolVolBitRate;
	switch (orig_sol_params.iSolVolBitRate)
	{
	case 7:
		// Display Set Complete
		LOG_DEBUG<<"- 19.2Kbps\n";
		break;

	case 8:
		// Display Set in progress
		LOG_DEBUG<<"- 38.4 Kbps\n";
		break;

	case 9:
		// Display Set in progress
		LOG_DEBUG<<"- 57.6 Kbps\n";
		break;

	case 10:
		// Display Set in progress
		LOG_DEBUG<<" 115.2 Kbps\n";
		break;
	default:

		break;
	}

	return TestUtil::TR_SUCCESS;
};

//*************************************************************************

/*************************************************************************
Function Name:	getSolConfig
Purpose		 :	Gets the data for Serial Over Lan configuration for the input parameters provided.
Parameters	 :
                [IN] priv - privilege level
				[OUT] resp_data - Response of Get Sol Config command as per IPMI spec
***************************************************************************/
SOLTest::s_Channel_Auth_Cap
SOLTest::getSolChannelAuthCapabilities(PrivelegeLevel level)
{
   	CommonClasses::IpmiRequest request(APPLICATION, 0x38);
    //Build Command Object
    request.append<byte_t>(m_channelNum | 0x80) // Ask for IPMI v2 data as well
    	   .append<byte_t>(level);

    CommonClasses::IpmiResponse response(GET_CHANNEL_AUTH_MIN_RSPLEN, GET_CHANNEL_AUTH_MAX_RSPLEN);
    //Send request data to the transport
    try {
    	sendIpmiCmd(request, response);
    }
    catch(TestResult &res)
    {
    	/*
    	 * It's very possible that this failed because we asked for IPMI v2 data
    	 * Ask again, without requesting IPMI v2 data
    	 */
    	LOG_WARN <<"\n Request failed (" << res << "), Possible - because we asked for IPMI v2 data, ask again.";

        request[0] &= 0x7F;

    	sendIpmiCmd(request, response);
    }

    return response.getFromData<struct s_Channel_Auth_Cap>(0);
}



/*************************************************************************
Function Name:		SolChannelAuthCapabilities
Purpose:			Gets and sets the data for Serial Over Lan configuration for the input parameters provided
					and verifies that the values match those parameters defined by the specification.
Returns:			Response indicating pass/fail of the validation of SOL Configuration
*****************************************************************************************/
TestResult
SOLTest::SolChannelAuthCapabilitiesTest(any_value &param)
{
	LOG_DEBUG<<"\nChecking Serial Over Lan Channel Auth. Capabilities";
	if(!isRemoteConnection())
	{
		LOG_WARN<<"\nTest does not run in InBand" ;
		return TestUtil::TR_SKIPPED;
	}

	if(m_channelNum == 0xFF)
	{
		LOG_ERR<<"\nIncorrect Serial Over Lan channel";
		return TestUtil::TR_SKIPPED;
	}

	SOLTest::s_Channel_Auth_Cap auth_cap = getSolChannelAuthCapabilities(/*static_cast<PrivelegeLevel>(param.value<byte_t>())*/ param.value<PrivelegeLevel>() ); //Privilege level

	LOG_INFO << "\n\tChannel number             : " << (int)auth_cap.channel_number;
	LOG_INFO << "\n\tIPMI v1.5  auth. types     : ";
	if ( auth_cap.v15_auth_types & IPMI_1_5_AUTH_NONE )
		LOG_INFO << "None, ";
	if ( auth_cap.v15_auth_types & IPMI_1_5_AUTH_MD2 )
		LOG_INFO << "MD2, ";
	if ( auth_cap.v15_auth_types & IPMI_1_5_AUTH_MD5 )
		LOG_INFO << "MD5, ";
	if ( auth_cap.v15_auth_types & IPMI_1_5_AUTH_PASSWORD )
		LOG_INFO << "Password, ";
	if ( auth_cap.v15_auth_types & IPMI_1_5_AUTH_OEM )
		LOG_INFO << "OEM, ";

	LOG_INFO<<"\n\tPer message authentication : "<< ((auth_cap.per_message_auth) ? "disabled" : "enabled");
	LOG_INFO<<"\n\tUser level authentication  : "<< ((auth_cap.user_level_auth) ? "disabled" : "enabled");

	LOG_INFO<<"\n\tNon-null user names exist  : "<< ((auth_cap.non_null_usernames) ? "yes" : "no");
	LOG_INFO<<"\n\tNull user names exist      : "<< ((auth_cap.null_usernames) ? "yes" : "no");
	LOG_INFO<<"\n\tAnonymous login enabled    : "<< ((auth_cap.anon_login_enabled) ? "yes" : "no");

	if (auth_cap.v20_data_available)
	{
		LOG_INFO<<"\n\tKG status                  : "<< ((auth_cap.kg_status) ? "non-zero" : "default (all zeroes)");
		LOG_INFO<<"\n\tChannel supports IPMI v1.5 : "<< ((auth_cap.ipmiv15_support) ? "yes" : "no");
		LOG_INFO<<"\n\tChannel supports IPMI v2.0 : "<< ((auth_cap.ipmiv20_support) ? "yes" : "no");
	}

	/*
	 * If there is support for an OEM authentication type, there is some
	 * information.
	 */
	if (auth_cap.v15_auth_types & IPMI_1_5_AUTH_OEM)
	{
		LOG_INFO<<"\n\tIANA Number for OEM        : "<< std::hex << std::showbase << (int)(auth_cap.oem_id[0] | (auth_cap.oem_id[1] << 8) |(auth_cap.oem_id[2] << 16));
		LOG_INFO<<"\n\tOEM Auxiliary Data         : "<< (int)auth_cap.oem_aux_data;
	}

	return TestUtil::TR_SUCCESS;
}

SOLTest::s_Payload_Activation
SOLTest::getSolPayloadActivationStatus(byte_t num)
{
	CommonClasses::IpmiRequest request(APPLICATION, 0x4A);
	    //Build Command Object
	request.append<byte_t>(num);

	CommonClasses::IpmiResponse response(GET_PAYLOAD_MIN_RSPLEN, GET_PAYLOAD_MAX_RSPLEN);
	//Send request data to the transport
	sendIpmiCmd(request, response);

	return response.getFromData<struct s_Payload_Activation>(0);
}

TestResult
SOLTest::SolPayloadActivationStatusTest(any_value &param)
{
	LOG_DEBUG<<"\nChecking Serial Over Lan Payload Activation Status";
	if(!isRemoteConnection())
	{
		LOG_WARN<<"\nTest does not run in InBand." ;
		return TestUtil::TR_SKIPPED;
	}

	SOLTest::s_Payload_Activation pay_activ = getSolPayloadActivationStatus(param.value<byte_t>()); //type number

	LOG_INFO << "\n\tPossible Instance Number : " << (int)pay_activ.num_instances;
	short instances = *(reinterpret_cast<short *>(pay_activ.instances));
	for ( int i=1; i<=16; i++)  {
		LOG_INFO<<  "\n\tInstance "<< i << " is "<<((instances&0x01) ? "activated" : "deactivated");
		instances >>= 1;
	}

	return TestUtil::TR_SUCCESS;
}

SOLTest::s_Payload_Info
SOLTest::getSolPayloadInstanceInfo(byte_t typ_num, byte_t inst_num)
{
	CommonClasses::IpmiRequest request(APPLICATION, 0x4B);
	    //Build Command Object
	request.append<byte_t>(typ_num)
		   .append(inst_num);

	CommonClasses::IpmiResponse response(GET_PAYLOAD_INF_MIN_RSPLEN, GET_PAYLOAD_INF_MAX_RSPLEN);
	//Send request data to the transport
	sendIpmiCmd(request, response);

	return response.getFromData<struct s_Payload_Info>(0);
}

TestResult
SOLTest::SolPayloadInstanceInfoTest(any_value &param)
{
	LOG_DEBUG<<"\nChecking Serial Over Lan Payload Instance Info";
	if(!isRemoteConnection())
	{
		LOG_WARN<<"\nTest does not run in InBand." ;
		return TestUtil::TR_SKIPPED;
	}

	short int nums = param.value<short>();
	SOLTest::s_Payload_Info pay_inf = getSolPayloadInstanceInfo(nums>>8, nums&0x0F); //type number

	LOG_INFO << "\n\tSession ID  : " << std::hex << std::showbase << (int)pay_inf.sess_id[0] << ":" << (int)pay_inf.sess_id[1] << ":"
									 << (int)pay_inf.sess_id[2] << ":" << (int)pay_inf.sess_id[3];
	LOG_INFO << "\n\tPort number : " << (int)pay_inf.port_num ;

	return TestUtil::TR_SUCCESS;
}


SOLTest::s_Payload_Support
SOLTest::getSolPayloadSupport(byte_t chann_num)
{
	CommonClasses::IpmiRequest request(APPLICATION, 0x4E);
		    //Build Command Object
	request.append<byte_t>(chann_num & (0x0F));

	CommonClasses::IpmiResponse response(GET_PAYLOAD_SUP_MIN_RSPLEN, GET_PAYLOAD_SUP_MAX_RSPLEN);
		//Send request data to the transport
	sendIpmiCmd(request, response);

	return response.getFromData<struct s_Payload_Support>(0);
}

TestResult
SOLTest::SolPayloadSupportTest(any_value &param)
{
	LOG_DEBUG<<"\nChecking Serial Over Lan Payload Support";
	if(!isRemoteConnection())
	{
		LOG_WARN<<"\nTest does not run in InBand." ;
		return TestUtil::TR_SKIPPED;
	}

	SOLTest::s_Payload_Support pay_supp = getSolPayloadSupport(m_channelNum); //channel number

	short payload = *(reinterpret_cast<short *>(pay_supp.standard_payload));
	for(int i=0; i<16; i++)  {
		LOG_INFO<<  "\n\t\tStandard payload type #"<< i << " is "<<((payload&0x01) ? "" : "not ") << "supported";
		payload >>= 1;
	}

	payload = *(reinterpret_cast<short *>(pay_supp.session_setup));
	for(int i=0; i<16; i++)  {
		LOG_INFO<<  "\n\t\tSession setup payload type #"<< i << " is "<<((payload&0x01) ? "" : "not ") << "supported";
		payload >>= 1;
	}

	payload = *(reinterpret_cast<short *>(pay_supp.oem_payload));
	for(int i=20; i<(20+16); i++)  {
		LOG_INFO<<  "\n\t\tPayload (OEM) type "<< std::hex << std::showbase << i << ((payload&0x01) ? "" : " not") << " used";
		payload >>= 1;
	}

	return TestUtil::TR_SUCCESS;
}

