/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intel�s	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*******************************************************************************
***		 File		:	DCMI_Sensor_lib.cpp									  **
***																			  **
***		 Abstract	:	Implements low level procedures to send the Sensor    **
***				        commands in the "Sensor & Storage  commands"          **
***                      in Table6.1 "Platform Command Definition"            **
***                     of DCMI 1.0 spec. Also implements                     **
***						the test procedures to validated the same commands    **
***                      in the DCMI_Conformance Module.                      **
***																		      **
***		 Contents	:														  **
***				Definition of following class SensorTest                      **
***																			  **
********************************************************************************/

#include "parameters.h"
#include "DCMI_SDR_lib.h"
#include "DCMI_Sensor_lib.h"

using std::hex;
using std::showbase;
using std::dec;

/*************************************************************************
Function Name	:	sensorDiscovery
Purpose		:	Issues and processes the DCMI Sensor Discovery command.
 The intent here is to issue the command and get the supported parameters
 and related entities for the various DCMI sensors.
Parameters:	[IN] discCmdData - Per the DCMI document Extension, Discovery ID, Request Data
			[OUT] instanceRet - the total number of available instances
            [OUT] vector of the Record ID of the Sensor
***************************************************************************/
void
SensorTest::sensorDiscovery(s_sensor discCmdData, byte_t &instanceRet, std::vector<record_id> &rec_ids)
{
	//Build Command Object
	DCMIClasses::IpmiDCMIRequest request(0x07);

	request.append<byte_t>(discCmdData.sensorType)
		   .append<byte_t>(discCmdData.entityId)
		   .append<byte_t>(discCmdData.entityInstance)
		   .append<byte_t>(discCmdData.entityStart);

	DCMIClasses::IpmiDCMIResponse response(DCMI_SENSOR_INFO_MIN_RSPLEN, DCMI_SENSOR_INFO_MAX_RSPLEN);

	//Send request data to the transport
	sendIpmiCmd(request, response);

	//Check and store values received back if valid based upon number of entities
	instanceRet = response[1];
	int max_idx = 2 * (response[2] + 1);
	if (response.getLength() < static_cast<size_t>(max_idx))
	{
		LOG_ERR << "\n\tInconsistent response, number of expected records: "
				<< response[2] << ", does not meet size of returned data";
		throw TestUtil::TR_FAILURE;
	}

	rec_ids.clear(); //clear the vector
	record_id rec(0,0);
	for (int idx=3; idx<max_idx; idx++)
	{	//Get the SDR Record ID from the command response
		rec.rLsb = response[idx];
		rec.rMsb = response[++idx];
		rec_ids.push_back(rec);	
	}
}


/***************************************************************************
Function Name:		getSensorReading
Purpose:			Issues and processes the Get Sensor Reading command.
					The intent here is to process the sensor numbers returned
					from the Get DCMI Sensor Info.
 Parameter:			Sensor Number (IN) - The sensor number for which GetSensorReading
										is issued
					Sensor Read Value (OUT) - Sensor reading
*************************************************************************************/

int
SensorTest::getSensorReading(int sensorNum )
{
    CommonClasses::IpmiRequest request(SENSOR, 0x2D, sensorNum);
    CommonClasses::IpmiResponse response(GET_SENSOR_RDNG_MIN_RSPLEN, GET_SENSOR_RDNG_MAX_RSPLEN);
    sendIpmiCmd(request, response);
    return response[0];
}

/***************************************************************************
Function Name:		SensorThresholdTest(int sensorNum);

Purpose:			Issues and displays the Get Sensor Threshold for each sensors.

 Parameter:			Sensor Number (IN) - The sensor number for which GetSensorReading
										is issued
*************************************************************************************/
void
SensorTest::sensorThresholdTest(int sensorNum)
{
	LOG_DEBUG<<"\nDCMI Sensor Threshold command.";
	//Call the sensor with supplied sensor number return the value.
	CommonClasses::IpmiRequest request(SENSOR, 0x27, sensorNum);
	//    request.setRsAddr(BMC_ADD);
	//    request.setRsLun(LUN);

    CommonClasses::IpmiResponse response(GET_SENSOR_RDNG_MIN_RSPLEN, GET_SENSOR_RDNG_MAX_RSPLEN);
    sendIpmiCmd(request, response);

	//Verify Completion Code & Process return Data
    LOG_DEBUG<<"\n\t\tThresholds for sensor number - "<<hex<<showbase<<int(sensorNum);
    LOG_DEBUG<<"\n\t\tReadable Thresholds          - "<<hex<<showbase<<int(response[0]);
    LOG_DEBUG<<"\n\t\tLower Non-Critical Threshold - "<<hex<<showbase<<int(response[1]);
    LOG_DEBUG<<"\n\t\tLower Critical Threshold     - "<<hex<<showbase<<int(response[2]);
    LOG_DEBUG<<"\n\t\tLower Non-Recoverable        - "<<hex<<showbase<<int(response[3]);
    LOG_DEBUG<<"\n\t\tUpper Non-Critical Threshold - "<<hex<<showbase<<int(response[4]);
    LOG_DEBUG<<"\n\t\tUpper Critical Threshold     - "<<hex<<showbase<<int(response[5]);
    LOG_DEBUG<<"\n\t\tUpper Non-Recoverable        - "<<hex<<showbase<<int(response[6]);
}


TestResult
SensorTest::SingleSensorTest(any_value &an_sensor)
{
	if(an_sensor.empty())
	{
		LOG_ERR<<"Incorrect parameter for entity sensor test";
		return TestUtil::TR_NONE;
	}

	s_sensor sensor = an_sensor.value<s_sensor>();

	LOG_DEBUG<<"\nVerifying Sensor "<<std::hex<<std::showbase<<static_cast<int>(sensor.entityId)<<", "<<std::hex<<static_cast<int>(sensor.sensorType)
			  <<" Instance: "<<std::dec<<static_cast<int>(sensor.entityInstance);

	s_sdr_data sdrReturnValues;
	std::vector<record_id> recids ;

	byte_t instanceRet = 0;

	sensorDiscovery(sensor, instanceRet, recids);
	if (!recids.size())
	{
	    LOG_ERR << "\n\tNo sensor data found!";
	    return TestUtil::TR_FAILURE;
	}

	//Here will need to call the get SDR routine to get the specific record ID
	//Reserve SDR
	byte_t reservationLsb = 0x00;
	byte_t reservationMsb = 0x00;
	reserveSdr(reservationLsb, reservationMsb);

	//Get SDR with Reservation ID and Record ID
	//With the record ID obtain the specific parameters from the SDR for the sensor call
	//sensor number and the parameters to do the compare
	byte_t reservationIdLsb = reservationLsb;
	byte_t reservationIdMsb = reservationMsb;

	//at least one record
	record_id recid = recids[0];

	getSdrValues(recid.rLsb, recid.rMsb, reservationIdLsb, reservationIdMsb, 0x00, 0x10, sdrReturnValues);

	//Check the SensorType returned in the SDR record for DCMI spec compliance for the entity
	if (sdrReturnValues.sensorType != sensor.sensorType)
	{
		//Test Failed as the SensorType defined in DCMI 1.1 ver spec does not match the sensorType in SDR
		LOG_DEBUG<<"\n\tEntity ID : "<<hex<<showbase<<(int)sensor.entityId;
		LOG_DEBUG<<"\n\tRecord Type as per DCMI spec : "<<hex<<showbase<<(int)sensor.sensorType;
		LOG_DEBUG<<"\n\tObtained Record Type as in SDR Record : "<<hex<<showbase<<(int)sdrReturnValues.sensorType;
		LOG_WARN<<"\n\tSensor Record Type of Entity ID : "<<hex<<showbase<<(int)sensor.entityId<<" is not DCMI spec Compliant";
		return TestUtil::TR_FAILURE;
	}

	//Call the sensor with supplied sensor number return the value.
	int sensorReadValue = getSensorReading(sdrReturnValues.senseNumber);
	LOG_DEBUG<<"\n\t\tUUT sensor number - 0x"<<hex<<int(sdrReturnValues.senseNumber);
	LOG_DEBUG<<"\n\tValue from Sensor is                    :\t0x"<<hex<<int(sensorReadValue);

	sensorThresholdTest(sdrReturnValues.senseNumber);

	return TestUtil::TR_SUCCESS;
}


void
SensorTest::verifySensor( byte_t entity_id, byte_t entity_nr )
{
   /*
	*  Load variables needed for getting the DCMI specific sensors using the Discovery Command
    *  If command does not return will not execute sensor routine.
    *
    *  Check the return value of the sensor discovery and determine whether to start the loop.
    *  Start for loop for number of entity ID's.  Pass the ID to the function to be processed.
	*/

	LOG_DEBUG<<"\nVerifying Entity Sensor ID " <<hex<<showbase<<int(entity_id)<<" and associated Sensors, SDRs";

	s_sensor sensdata = {0};

	sensdata.sensorType = entity_nr;
	sensdata.entityId = entity_id;
	sensdata.entityInstance = 0x00;
	sensdata.entityStart = 0x00;

	byte_t instanceRet = 2; //Value set to start a value.  True value to be returned via function
					        // execution.
	std::vector<record_id> recids ;
	//Find the number of instances for this EntityID
	sensorDiscovery(sensdata, instanceRet, recids);

	if (instanceRet<1) {
		LOG_ERR<<"\n\tNo valid sensors found!";
		throw TestUtil::TR_SKIPPED;
	}

	for(int instance = 1; instance <= instanceRet; instance++)
	{
		sensdata.entityInstance = instance;
		sensdata.entityStart = 1;

		std::stringstream msg;
		msg<<"Sensor Entity ID: "<<hex<<showbase<<static_cast<int>(entity_id)
							<<", Type: "<<hex<<showbase<<static_cast<int>(entity_nr)
							<<", Instance: "<<dec<<static_cast<int>(instance);

		ADD_TEST_PARAM(SensorTest::SingleSensorTest, sensdata, msg.str(), TestUtil::TT_OPTIONAL);
	}//End of for Instance
}
