/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intel�s	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*******************************************************************************
***		 File		:	DCMI_TMODE_lib.cpp        							  **
***																			  **
***		 Abstract	:	Implements the TMODE Test class                       **
***                     in the DCMI_Conformance Module.                       **
***																		      **
********************************************************************************/

#include "parameters.h"
//#include "declarations.h"
#include "DCMI_Configuration.h"
#include "DCMI_Tmode_lib.h"


/***********************************************************************************************
Function Name:	getSerialModemConfig
Purpose:		Gets the data for Serial/Modem configuration for the input parameter provided.
Parameters:		[IN] iSerialChanNum - Serial channel Number
				[IN] iSerModParam   - Serial Modem Parameter
				[IN] iSetSelector	- Set Selector
				[IN] iInbandFlag	- Inband Flag
				[OUT]resp_data- Response for the parameter requested for Get Serial Modem Configuration

********************************************************************************************/
void
TmodeTest::getSerialModemConfig(int iSerModParam, int iSetSelector, int iBlockSelector, std::vector<byte_t> &resp_data)
{
	CommonClasses::IpmiRequest request(TRANSPORT, 0x11);
	//Build Command Object
	request.append<byte_t>(m_channelNum)
	 	   .append<byte_t>(iSerModParam)
	 	   .append<byte_t>(iSetSelector)
	 	   .append<byte_t>(iBlockSelector);

 	CommonClasses::IpmiResponse response(GET_TMODE_CONFIG_MIN_RSPLEN, GET_TMODE_CONFIG_MAX_RSPLEN);

	//Send request data to the transport
	sendIpmiCmd(request, response);

	response.copy_to(0, resp_data);
}

/****************************************************************************************
Function Name:	ValidateTModeConfig
Purpose:		Gets the data for Serial/Modem configuration for the input parameter provided
				and verifies that the values match those parameters defined by the specification.

Returns:		Response indicating pass/fail of the validation of TMode
****************************************************************************************/
TestResult
TmodeTest::ValidateTModeConfigTest(any_value &)
{
	LOG_DEBUG<<"\nChecking Get Serial/Modem Configuration for TMode.";
	//Verify TMODE Test support in Platform
	if (m_TmodePresent == 0)
	{
	//  TMode is not supported
	//	if(gsOM_Ret.OOB_TM == 'O')
		{
			//Feature is optional as per DCMI spec
			LOG_WARN<<"\n\tTMODE is NOT Supported and it is Optional.";
			return TestUtil::TR_SKIPPED;

		}
	/*	else
		{
			//Feature is mandatory and has to be supported - so fail the test
			LOG_DEBUG<<"\n\tWarning: TMODE is Mandatory feature but NOT Supported in Platform.";
			return TestUtil::TR_FAILURE;
		}
	*/
	};

	int	iSerModParam = 3;
	int	iSetSelector = 0;
	int	iBlockSelector = 0;

	//Define the Serial Config Response to load the return value from the response for comparison
	std::vector<byte_t> SerialConfigRsp;

	//Issue Get Serial/Modem Configuration command to start comparison of parameter 3 which is connection mode
	//Min Prev Level for SetSOLConfig command is  : Operator
	setSessionPrivilegeLevel(PRIV_OPERATOR);

	//Issue the command for IPMI payload, and obtain
	getSerialModemConfig(iSerModParam, iSetSelector, iBlockSelector, SerialConfigRsp);

	//If compCode is 0 then check the values to see if basic mode and Tmode are enabled.
	//Check the bit map for byte 1 returned to see if serial basic bit is enabled
	int iTmodeSupport = 0;
	if ((SerialConfigRsp[1] & 0x80) == 0x80)
	{
		LOG_INFO<<"\n\tSerial Basic for TMode is supported on the platform.";
		iTmodeSupport++;

	} else
	{
		LOG_WARN<<"\n\tSerial Basic for TMode is not supported on the platform.";
	}

	//Check the bit map for byte 1 returned to see if Tmode bit is enabled
	if ((SerialConfigRsp[1] & 0x04) == 0x04)
	{
		LOG_INFO<<"\n\tTMode is enabled on the platform.";
		iTmodeSupport++;

	} else
	{
		LOG_WARN<<"\n\tTMode is not enabled on the platform.";
	}

	//Issue Get Serial/Modem Configuration command to start comparison of parameter 7 which is communication settings
	//Display to user the selected baud rate.
	iSerModParam = 7;
	//Min Prev Level for SetSOLConfig command is  : Operator
	//setSessionPrivilegeLevel(PRIV_OPERATOR);

	//Issue the command for IPMI payload, and obtain
	getSerialModemConfig(iSerModParam, iSetSelector, iBlockSelector, SerialConfigRsp);

	//Switch on the return data to display to the user the baud rate the UUT has enabled
	//No checks other than if the value is not one of the supported baud rates
	switch (SerialConfigRsp[2])
	{
	case 6:
		LOG_INFO << "\n\tBaud Rate is set to 9.6k BPS.";
		break;
	case 7:
		LOG_INFO << "\n\tBaud Rate is set to 19.2k BPS.";
		break;
	case 8:
		LOG_INFO << "\n\tBaud Rate is set to 38.4k BPS.";
		break;
	case 9:
		LOG_INFO << "\n\tBaud Rate is set to 57.6k BPS.";
		break;
	case 10:
		LOG_INFO << "\n\tBaud Rate is set to 115.2k BPS.";
		break;
	default:
		LOG_WARN << "\n\tBaud Rate value is unrecognizable.";
	}

	if (iTmodeSupport == 2)
	{
		//Issue Get Serial/Modem Configuration command to start comparison of parameter 29 which is TMode Configuration settings
		//Display to user the selected baud rate.
		iSerModParam = 29;
		//Min Prev Level for SetSOLConfig command is  : Operator
		//setSessionPrivilegeLevel(PRIV_OPERATOR);
		//Issue the command for IPMI payload, and obtain
		getSerialModemConfig(iSerModParam, iSetSelector, iBlockSelector, SerialConfigRsp);

		//Dump the raw data received to the query using parameter = 29 to the display
		//If the user wishes to decipher they must get the bit parameters from the IPMI spec
		int iTmodeOp = SerialConfigRsp[1] & 0x2F;
		int iParamOp = SerialConfigRsp[1] & 0xC0;
		iParamOp >>= 6;
		int iInSeq = SerialConfigRsp[2] & 0x0F;
		int iOutSeq = SerialConfigRsp[2] & 0xF0;
		iOutSeq >>=4;
		LOG_INFO<<"\n\tParameter Operation bits are:\t0x"<<std::hex<<iParamOp<<"\n";
		LOG_INFO<<"\n\tTerminal Mode Options bits are:\t0x"<<std::hex<<iTmodeOp<<"\n";
		LOG_INFO<<"\n\tInput Line Sequence is        :\t0x"<<std::hex<<iInSeq<<"\n";
		LOG_INFO<<"\n\tOutput Line Sequence is       :\t0x"<<std::hex<<iOutSeq<<"\n";
	}

	return TestUtil::TR_SUCCESS;	//TMODE PASS
}
//*************************************************************************
