/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/********************************************************************************
***		 File		:	DCMI_VLAN_lib.cpp									  **
***																			  **
***		 Abstract	:	Implements all the procedures for VLAN testcase       **
***						clases LanBaseTest, VLanTest				          **
***																		      **
********************************************************************************/
#include "parameters.h"
#include "DCMI_VLAN_lib.h"
#include "DCMI_Configuration.h"


using std::hex;
using std::showbase;
using std::setw;
using std::setfill;


/*
 * Local tool class used by as a functor,
 * 	used by IssueGetMcidStringTest, RMCPPEntiresTest ...
 *
 */
class local
{
	public:
		static int isnull(byte_t value)
		{
			return (value == 0);
		}

		static void show_value(byte_t value)
		{
				LOG_INFO<<"\n\t\tCipher - "<<hex<<showbase<<static_cast<int>(value);
		}

};


/**************************************************************************
Function declaration
int GetLANConfig ( int iChannel, int iParam, dcmi_rsp_t *pRspData, int inbandFlag);
Function Name	:	GetLANConfig
Purpose			:   Issues the 'Get LAN Configuration Parameters' command
                    and gets the parameter iParam. The value of the iParam corresponds
					to the LAN Configuration Parameter Table in the IPMI spec.
Parameters		:
                    IN  iParam   - Value of Parameter to get as in IPMI spec
			        OUT pRspData - The response data, command response as per IPMI spec
****************************************************************************/
void
LanBaseTest::getLANConfig (byte_t param, std::vector<byte_t> &resp_data)
{
	CommonClasses::IpmiRequest request(TRANSPORT, 0x02);
	request.append<byte_t>(m_channelNum & 0x0F)//Channel Number bits 0-3, so mask 0x03 is more suitable
		   .append<byte_t>(param)
		   .append<byte_t>(0x00)
		   .append<byte_t>(0x00);

	//Build Command Object
	CommonClasses::IpmiResponse response(GET_LAN_CONFIG_MIN_RSPLEN, GET_LAN_CONFIG_MAX_RSPLEN);

	//Min Prev Level for GetLANConfig command is  : Operator
	setSessionPrivilegeLevel(PRIV_OPERATOR);

    //Send request data to the transport
    sendIpmiCmd(request, response);

	//Return Command Completion Status
    response.copy_to(0, resp_data);
}
//**************************************************************************

/**************************************************************************
Function Name	:	VLANID
Purpose			:   Validates the platform support for setting and getting the
                    VLANID. The parameter 20 of the 'Set LAN Configuration Parameters'
					and 'Get LAN Configuration Parameters' commands correspond to
					VLANID. Verify that these commands support parameter 20.
Returns         :   command completion status
****************************************************************************/
TestResult
VLanTest::VLANIDTest(any_value &)
{
	LOG_DEBUG<<"\nGet VLAN ID and confirm command supported.";
	if(m_channelNum == 0xFF)
	{
		LOG_ERR<<"\nIncorrect LAN channel";
		return TestUtil::TR_SKIPPED;
	}

	std::vector<byte_t> response;
	/*
	 * GetVLANID and confirm command supported, if not exit test
	 */

	// Get VLANID for LAN primary Channel
	getLANConfig (VLANID_PARAM, response);
	/* VLANID obtained successfully
	 * Save the VLANID, and the VLAN enabled or disabled state to be restored at end of test
	 */
	// Obtain current VLAN enable/disable state and store in iCurVlanState for cleanup at end of test
	int iCurVlanState = response[2]&0x80;
	//store platform VLANID in iCurVlanID for test cleanup.
	int iCurVlanID = (response[2] & 0x0F);
	iCurVlanID = iCurVlanID << 8;
	iCurVlanID += response[1];

	//LOG successful completion for GetVLANID
	if ( iCurVlanState )
	{
		LOG_INFO<<"\n\tVLAN is enabled in Platform";
		LOG_INFO<<"\n\tVLAN ID: 0x" << hex << iCurVlanID;
	}
	else
	{
		LOG_INFO<<"\n\tVLAN is disabled in Platform";
	}

	return TestUtil::TR_SUCCESS;
}
//**************************************************************************


/**************************************************************************
Function Name	:	VLANPriority
Purpose			:   Make sure the platform supports setting and getting the
                    VLAN Priority.

Returns         :   command completion status
****************************************************************************/
TestResult
VLanTest::VLANPriorityTest (any_value &)
{
	LOG_DEBUG<<"\nGet VLAN Priority for primary LAN Channel and confirm command supported.";
	if(m_channelNum == 0xFF)
	{
		LOG_ERR<<"\nIncorrect LAN channel";
		return TestUtil::TR_SKIPPED;
	}
	/* GetVLANPriority and confirm command supported, if not exit test
	 * for LAN primary Channel
	*/
	// Get VLANID for LAN primary Channel
	std::vector<byte_t> response;
	getLANConfig (VLAN_PRIORITY_PARAM, response);

	//Issue Get LAN Configuration Parameter command, for parameter VLAN Priority
	// Save the VLANPriority
    int iCurVlanPriority = response[1];
    LOG_INFO<<"\n\tCurrent VLAN Priority is "<<hex<<iCurVlanPriority;

	return TestUtil::TR_SUCCESS;
}
//**************************************************************************


/**************************************************************************
Function Name	:	VLANTag
Purpose			:   Make sure the platform supports setting and getting the
                    VLAN Tag.

Returns         :   Test completion status
****************************************************************************/
TestResult
VLanTest::VLANTagTest (any_value &)
{
	LOG_DEBUG<<"\nGet VLAN Tag and confirm if command supported.";
	if (m_channelNum == 0xFF)
	{
		LOG_ERR<<"\nIncorrect LAN channel";
		return TestUtil::TR_SKIPPED;
	}

	// Get the Number of Destination Parameter (17) to verify if VLAN Tag is programmed in Platform
	std::vector<byte_t> response;
	getLANConfig (NUM_DESTINATION, response);

    int iLanAlertSupp = response[0] & 0x0F;
	LOG_DEBUG<<"\n\tCurrent VLAN Alert is "<<hex<<iLanAlertSupp;
	//Obtain LANAlertSupported from response data
	if (iLanAlertSupp)
	{  /* LAN Alert is supported, so Param25- VLANTag should be supported
   		* GetVLANTag and confirm command supported, if not exit test
		* for LAN primary Channel
		*/
		LOG_INFO<<"\n\tLAN Alert supported, testing VLANTag support ";
		getLANConfig (VLAN_TAG_PARAM, response);
		// Save the VLANTag
		int iCurVlanTag = response[0];
		LOG_INFO<<"\n\tCurrent VLAN Tag is "<<hex<<iCurVlanTag;
	}
	else
	{	//Since LAN Alert is not supported, VLANTAG cannot be validated, test PASS
		//LAN Alert is not supported, so just try to GetVLANTag and if it fails report warning
		try {
			LOG_DEBUG<<"\n\tGet Lan Configuration for parameter: VLAN Tag";
			getLANConfig (VLAN_TAG_PARAM, response);
		}
		catch (TestResult &e)
		{
			if (TestUtil::TR_CC_FAILURE == e)
			{
				LOG_WARN << "\n\tPlatform does not support LAN Alert, VLAN Tag parameter cannot be validated";
			}
			else
				return e;
		}
		catch(...)
		{
			LOG_ERR<<"\n\tWarning: Unknown Error occurred ";
			return TestUtil::TR_FAILURE;
		}
	}
	return TestUtil::TR_SUCCESS;
}
//*************************************************************************


/**************************************************************************
Function Name	:	RMCPPSupport
Purpose			:   Provides a count of the number of Cipher Suites available to be enabled for use
					with IPMI Messaging on the given channel.
Returns         :   command completion status
****************************************************************************/
TestResult
VLanTest::RMCPPSupportTest(any_value &)
{
	LOG_DEBUG<<"\nIssue VLAN RMCPP+ Messaging Cipher Suite Entry Support";
	if (m_channelNum == 0xFF)
	{
		LOG_ERR<<"\nIncorrect LAN channel";
		return TestUtil::TR_SKIPPED;
	}
	//Issue Get LAN Configuration Parameter command, for parameter VLAN_RMCPP_ENTRY_SUPP
	std::vector<byte_t> response;
	getLANConfig (VLAN_RMCPP_ENTRY_SUPP, response);

	//Test Result - 0 -pass, other values-fail
	//Check Command CompletionCode
	int	iSupportetCiphersCount = response[1];
	LOG_INFO<<"\n\tCurrent number of RMCPP Cipher Suites Entry Support is "<<iSupportetCiphersCount;

	return TestUtil::TR_SUCCESS;
}
//*************************************************************************


/**************************************************************************
Function Name	:	RMCPPEntires
Purpose			:   List Cipher Suite Ids for Supher Suites that can be used for
					establishing an IPMI messaging session with BMC.
Returns         :   command completion status
****************************************************************************/
TestResult
VLanTest::RMCPPEntiresTest (any_value &)
{
	LOG_DEBUG<<"\nIssue VLAN RMCPP+ Messaging Cipher Suites Entry";
	if (m_channelNum == 0xFF)
	{
		LOG_ERR<<"\nIncorrect LAN channel";
		return TestUtil::TR_SKIPPED;
	}
	//Issue Get LAN Configuration Parameter command, for parameter VLAN_RMCPP_ENTIRES
	std::vector<byte_t> response;
	getLANConfig (VLAN_RMCPP_ENTIRES, response);

	response.erase(response.begin(), response.begin() + 2);

	LOG_INFO<<"\n\tSupportet RMCPP Cipher Suites Entries are :";

	//could use mem_fun(&VLanTest::show_value) ... in this way we have local
    //method and everything is "on view"
	std::for_each(response.begin(), response.end(), local::show_value);

	return TestUtil::TR_SUCCESS;
}
//*************************************************************************


/**************************************************************************
Function Name	:	TestRmcppLevel
Purpose 		:	Check Cipher Suite privilege levels.
					1.	Get LAN Config.

Returns         :   Test completion status
****************************************************************************/
TestResult
VLanTest::RMCPPPrivLevelTest(any_value &)
{
	LOG_DEBUG<<"\nIssue RMCPP+ Messaging Cipher Suite Privilege Level";
	if (m_channelNum == 0xFF)
	{
		LOG_ERR<<"\nIncorrect LAN channel";
		return TestUtil::TR_SKIPPED;
	}
	//Issue Get LAN Configuration Parameter command, for parameter VLAN_RMCPP_ENTIRES
	std::vector<byte_t> response;
	getLANConfig (VLAN_RMCPP_PRIV_LEVEL, response);

	//Print supported Cipher Suits with priv. lvl.
	//GetCipherSuitsPrivLvl(pConfiguration, sRspData, &iSaveCipherLevel, &iSetCipherLevel, iSupEntiresTab);
	// //PASS

	return TestUtil::TR_SUCCESS;
}
//*************************************************************************

/**************************************************************************
Function Name	:	GetSessionInfo
Purpose			:   Issues the 'Get Session Info' command
Parameters		:   OUT Resp_Data - The response data consists of the
                    command response as per IPMI spec
****************************************************************************/
void
LanTest::getSessionInfo(std::vector<byte_t> &resp_data)
{
	CommonClasses::IpmiRequest request(APPLICATION, 0x3D);
	request.append<byte_t>(0x00);//get info for active/current session

	//Build Command Object
	CommonClasses::IpmiResponse response(GET_SESSION_INFO_MIN_RSPLEN, GET_SESSION_INFO_MAX_RSPLEN);

	//Min Prev Level for Session info, command is: Operator
	setSessionPrivilegeLevel(PRIV_USER);

	//Send request data to the transport
	sendIpmiCmd(request, response);

	//Return Command Completion Status
	response.copy_to(0, resp_data);
}
//**************************************************************************


/**************************************************************************
Function Name	:	GetChannelAccess
Purpose			:   Issues the 'Get Channel Access' command
Parameters		:   IN  iChannel - LAN Channel number for which iParam is obtained
 			        OUT pRspData - The response data consists of the
                    completion code, and command response as per IPMI spec
****************************************************************************/
void
LanTest::getChannelAccess(std::vector<byte_t> &resp_data)
{
	CommonClasses::IpmiRequest request(APPLICATION, 0x41);
	request.append<byte_t>(m_channelNum & 0x0F)//Channel Number bits 0-3, so mask 0x03 is more suitable
		   .append<byte_t>(0x80);

	//Build Command Object
	CommonClasses::IpmiResponse response(GET_LAN_CHANNEL_MIN_RSPLEN, GET_LAN_CHANNEL_MAX_RSPLEN);

	//Min Prev Level for GetLANConfig command is  : Operator
	setSessionPrivilegeLevel(PRIV_USER);

	//Send request data to the transport
	sendIpmiCmd(request, response);

	//Return Command Completion Status
	response.copy_to(0, resp_data);
}
//**************************************************************************


/**************************************************************************
Function Name	:	MultiSession
Purpose			:   Issue GetSessionInfo command and log maximum sessions allowed
					by platform, currently used number of sessions.
					Establish multiple session and verify platform support for establishing
					multiple sessions.
Parameters      :
Returns         :   Test completion status
****************************************************************************/
TestResult
LanTest::MultiSessionTest (any_value &)
{
	LOG_DEBUG<<"\nIssue Multi Session Info command ";
	if(!isRemoteConnection())
	{
		LOG_WARN<<"\nTest does not run in InBand" ;
		return TestUtil::TR_SKIPPED;
	}
	//Issue GetSessioninfo command and log Max session, currently used up session count
	// channel is given as 0x01 to get information on current session, which gives
	// the max sessions in byte 2 and currently used sessions in byte 3
	std::vector<byte_t> response;
	getSessionInfo(response); //As RMCP session can be established only OOB third
									 //parameter channel is set to 0
	/* Session Info obtained successfully
	 * Log info to user	 */
	LOG_INFO<<"\n\tPlatform allows maximum sessions of "<< static_cast<int>(response[1]);
	LOG_INFO<<"\n\tCurrently "<< static_cast<int>(response[2])<<" sessions established \n ";
	return TestUtil::TR_SUCCESS;
}


/*************************************************************************
Function Name   :	MACaddr
Purpose         :	Obtains the MAC Address of the Platform.
Returns         :   Test completion status
***************************************************************************/
TestResult
LanTest::MACaddrTest(any_value &)
{
	/*
	 * Get MAC Address and display MAC Address
	 */
	LOG_DEBUG<<"\nIssue Get MAC Address command";
	if(m_channelNum==0xff)
	{
		LOG_ERR<<"\nIncorrect LAN channel";
		return TestUtil::TR_SKIPPED;
	};

	// Get MAC Address
	std::vector<byte_t> response;
	getLANConfig (MAC_ADDR_PARAM, response);
	/* MAC Address obtained successfully
	 */
	LOG_INFO<<"\n\tMAC Address of Platform  (Hex) :\t"<<showbase<<hex<<setw(2)<<setfill('0')<<int(response[1]);
	LOG_INFO<<hex<<setw(2)<<setfill('0')<<int(response[2]);
	LOG_INFO<<hex<<setw(2)<<setfill('0')<<int(response[3])<<hex<<setw(2)<<setfill('0')<<int(response[4]);
	LOG_INFO<<hex<<setw(2)<<setfill('0')<<int(response[5])<<hex<<setw(2)<<setfill('0')<<int(response[6]);

	return TestUtil::TR_SUCCESS;
}

/*************************************************************************
Function Name   :	GratArp
Purpose         :	Checks if Gratuitous ARP is turned OFF as per DCMI spec in
                    platform.
Returns         :   Test completion status
***************************************************************************/
TestResult
LanTest::GratArpTest(any_value &)
{
	LOG_DEBUG<<"\nGet Gratuitous Arp";
	if (m_channelNum == 0xFF)
	{
		LOG_ERR<<"\nIncorrect LAN channel";
		return TestUtil::TR_SKIPPED;
	}
	/*
	 * Get Gratutious Arp and verify if it is set correctly
	 */
	std::vector<byte_t> response;
	getLANConfig (ARP_CTRL_PARAM, response);

	/* ARP control param obtained successfully */
	//Check if bit 0 of second byte in response indicates Gratuitous ARP is disabled
	if ((response[1] & 0x01) == 0x01)
	{	//Gratuitous ARP is enabled
		LOG_INFO<<"\n\tGratuitous ARP is enabled in Platform"	;
		//iRetVal = 1; should not be a failure per 1.1 spec.
	}
	else
	{
		LOG_INFO<<"\n\tGratuitous ARP is disabled, Platform compliant with DCMI spec "	;
	}
	return TestUtil::TR_SUCCESS;
}


/*************************************************************************
Function Name   :	TestArpCtrl

Purpose         :	Checks if Gratuitous ARP is turned OFF as per DCMI spec in
                    platform.
Returns         :   Test completion status
***************************************************************************/
TestResult
LanTest::TestArpCtrl(any_value &)
{
	LOG_DEBUG<<"\nGet Arp Control Check";
	if (m_channelNum == 0xFF)
	{
		LOG_ERR<<"\nIncorrect LAN channel";
		return TestUtil::TR_SKIPPED;
	}
	/*
	 * Get Arp Control and verify if it is set correctly
	 */

	std::vector<byte_t> response;
	getLANConfig (ARP_CTRL_PARAM, response);

	/* ARP control param obtained successfully
	 */
	//Check if bit 1 of second byte in response indicates BMC-generated Arp Responses is enabled
	if ( response[1] & 2 )
	{
		//Gratuitous ARP is enabled - compliant with DCMI spec
		LOG_INFO<<"\n\tBMC generated ARP response is enabled in Platform"	;
	}
	else
	{
		LOG_INFO<<"\n\tBMC Generated ARP response is disabled"	;
		//iRetVal = 1; //not true per 1.1 spec
	}

	return TestUtil::TR_SUCCESS;
}

/*************************************************************************
Function declaration
Function Name   :	IPSrcTest
Purpose         :	Checks if IP Source is static/dynamic per DCMI spec in
                    platform.
Returns         :   Test completion status
***************************************************************************/
TestResult
LanTest::IPsrcTest(any_value &)
{
	/*
	 * Get IP source and verify if it is set correctly
	 */
	// Get IP Source param
	LOG_DEBUG<<"\nGet IP Source param";
	if (m_channelNum == 0xFF)
	{
		LOG_ERR<<"\nIncorrect LAN channel";
		return TestUtil::TR_SKIPPED;
	}

	std::vector<byte_t> response;
	getLANConfig (IP_SRC_PARAM, response);
	/* IP Source param obtained successfully */
	//Check if bit 1 of second byte in response indicates BMC-generated Arp Responses is enabled
	switch ((response[1] & 0x0F))
	{
	case 1 :
		//IP source is set to static - compliant with DCMI spec
		LOG_INFO<<"\n\tIP source is set to Static, compliant with DCMI spec"	;
		break;
	case 2 :
		//IP source is set to dynamic - compliant with DCMI spec
		LOG_INFO<<"\n\tIP source is set to dynamic, compliant with DCMI spec"	;
		break;

	default:
		LOG_INFO<<"\n\tBMC IP Source is not compliant with DCMI spec"	;
		return TestUtil::TR_FAILURE;
	}

	return TestUtil::TR_SUCCESS;
}

/**************************************************************************
Metchod declaration
Name   			:	AccessModeCheckTest
Purpose         :	Checks if LAN Configuration Access Mode is always available
                    per DCMI spec in platform.
Parameters      :  [IN] any_value - not used

Returns         :   Test completion status
****************************************************************************/
TestResult
LanTest::AccessModeCheckTest(any_value &)
{
	//Access mode is not set to Always Available
	LOG_DEBUG<<"\nGet Primary Channel Access mode";
	if (m_channelNum == 0xFF)
	{
		LOG_ERR<<"\nIncorrect LAN channel";
		return TestUtil::TR_SKIPPED;
	}

	//Issue GetChannelAccess command and check AccessMode to be 'Always Available'
	std::vector<byte_t> response;
	getChannelAccess ( response );
	// Check Access Mode in response
	if( (response[0] & 0x03) != 2)
	{
		//Access mode is not set to Always Available
		LOG_WARN<<"\n\tPrimary Channel Access mode is NOT set to Always available"
				<<"\n\t\tNot Compliant with DCMI Spec";
		return TestUtil::TR_FAILURE;
	}

	LOG_INFO<<"\n\tPrimary Channel Access mode is set to Always available";

	return TestUtil::TR_SUCCESS;
}


/**************************************************************************
Name			:	getMcidString
Purpose			:	Issues and processes the GetMcidString command.
Parameters      :   Per the DCMI document Offset and Number of Bytes
					[IN] Offset  - Offset of AssetTag to read from
					[IN] NumBytes- Number of bytes to read from offset
					[OUT] vector - The response data with the
					assettag, number of bytes read etc. Data based upon
					requested discovery input
Returns         :   void,  error are reported by exception
***************************************************************************/
void
LanTest::getMcidString(byte_t Offset, byte_t NumBytes, std::vector<byte_t> &resp_data)
{
	DCMIClasses::IpmiDCMIRequest request(0x09);
	request.append<byte_t>(Offset)//
		   .append<byte_t>(NumBytes);

	//Build Command Object
	DCMIClasses::IpmiDCMIResponse response(GET_MCID_MIN_RSPLEN, GET_MCID_MAX_RSPLEN);

	//Send request data to the transport
    LOG_DEBUG<<"\n\tIssue <Get MCID String> Command to read chunk of data.";

	//Send request data to the transport
	sendIpmiCmd(request, response);

	response.copy_to(1, resp_data); //copy from MC Tag length and data
	//Make sure length of bytes from assetTag read does not exceed 16 as per spec  + 1byte for length
	if (resp_data.size() > 17)
	{
		LOG_WARN<<"\nResponse data exceed specification size: "<< std::dec<<static_cast<int>(resp_data.size()-1);
		resp_data.resize(17);
	}
}

/***************************************************************************
Method declaration

Name		 :	IssueGetMcidStringTest
Purpose      :	Reads the entire MC ID String with the Get MC ID String command.
				Checks only valid Ascii values are used in the string
Parameters   :	None
Returns      :   command completion status
***************************************************************************/
TestResult
LanTest::IssueGetMcidStringTest(any_value &)
{
	//Get the MC ID String
	LOG_DEBUG<<"\nIssue Get MC ID String command";

	std::vector<byte_t> mcidchunk;
	getMcidString(0, 1, mcidchunk);

	byte_t McidLength = mcidchunk[0]; //Store length returned in response
	if(McidLength==0)
	{
		LOG_ERR<<"\n\tFAIL: The MC ID String is empty \n\t\tNOT Compliant with DCMI Specification.";
		return TestUtil::TR_FAILURE;
	}

	if (McidLength > MCID_MAX_LEN )
	{	//Maximum length of characters allowed as per spec for String ID Tag is MCID_MAX_LEN
		LOG_ERR<<"\n\tMC ID String Length 0x"<<hex<<int(McidLength)<<" is greater than 64";
		LOG_ERR<<"\n\tChecking only first 64 bytes of MC ID String";
		McidLength = MCID_MAX_LEN;
	} else {
		LOG_INFO<<"\n\tMC ID String Data Length = 0x"<<std::hex<<static_cast<int>(McidLength);
	}

	//AssetTag string stream
	std::vector<byte_t> mciddata;
	//Read the entire String ID by 16 chunks
	while(mciddata.size() < McidLength)
	{ //Since mc id can be read in multiples of 16, unless the total length left is < 16
      //Issuing GetAssetTag command for getting the Data
		getMcidString(static_cast<byte_t>(mciddata.size()), static_cast<byte_t>(std::min<size_t>(McidLength - mciddata.size(), GET_MCID_MAX_BYTES_TO_READ)), mcidchunk);
		//remove first element - length of returned data.
		mcidchunk.erase(mcidchunk.begin());
		//Store the characters read, to the end of the string previously read
		mciddata.insert(mciddata.end(), mcidchunk.begin(), mcidchunk.end());
	}
	//end with string terminator

	LOG_DEBUG<<"\n\tMC ID Data Read is: ";
	std::copy(mciddata.begin(), mciddata.end(), std::ostream_iterator<int>(LOG_DEBUG<<std::hex<<std::showbase, " "));

	//is null value or not, anyway we have null or end of array, this will cover both 1.1 and 1.5. 1.1 has null sign whereas 1.5 has not.
	std::vector<byte_t>::iterator end_it = std::find_if(mciddata.begin(), mciddata.end(), local::isnull);

	//Make sure data has only Ascii Characters
	std::vector<byte_t>::iterator it = std::find_if(mciddata.begin(), end_it, std::not1(std::ptr_fun<int, int>(::isprint))); //isascii
	if(it != end_it)
	{
	   	LOG_ERR<<"\n\tTest Failed: MC ID String has invalid ASCII value.";
	   	return TestUtil::TR_FAILURE;
	}

	//copy data as long as possible
	std::stringstream mcidstr;
	std::copy(mciddata.begin(), end_it, std::ostream_iterator<char>(mcidstr));

	m_MCIDString = mcidstr.str();
	LOG_INFO<<"\n\tMC ID String is: '"<<m_MCIDString<<"' and has only ASCII characters.";
	return TestUtil::TR_SUCCESS;  // Test Pass or Fail
}
//*************************************************************************

TestResult
LanTest::McidNameTest(any_value &)
{
	LOG_DEBUG<<"\nMC ID String test";
	if (m_MCIDString.length() == 0)
	{
		LOG_WARN<<"\n\tMC ID String is empty, skipping test";
		return TestUtil::TR_SKIPPED;
	}

	if( !isRemoteConnection()) {
		LOG_WARN<<"\n\tLocal connection";
		return TestUtil::TR_SKIPPED;
	}

	int /*iRetVal = 0,*/ MaxRetry = 3;
	std::string InputHostName ;
	//Providing 3 retries for user to enter Host Name String.
	for(int Iter = 0; Iter < MaxRetry; Iter++)
	{
		LOG_SUMMARY<<"\nMC ID Packet Capture is: '"<<m_MCIDString<<"'";
		LOG_SUMMARY<<"\nPlease enter the DHCP Host Name as you see in Packet Capture: ";

		std::cin >> InputHostName;

		std::transform(InputHostName.begin(), InputHostName.end(), InputHostName.begin(), ::toupper);
		//To sync C & C++ library
		std::ios::sync_with_stdio();
		if (InputHostName.compare(m_MCIDString) != 0)
		{
			LOG_DEBUG << "Invalid DHCP Host Name.";
			continue;
		}

		LOG_DEBUG << "Successful Input.";
		return TestUtil::TR_SUCCESS;
	} //for try loop

	return TestUtil::TR_FAILURE;;
}


/**************************************************************************
Function Name	:	GetChannelInfo
Purpose			:   Issues the 'Get Channel Access' command
Parameters		:   IN  iChannel - LAN Channel number for which iParam is obtained
 			        OUT pRspData - The response data consists of the
                    completion code, and command response as per IPMI spec
****************************************************************************/
void
LanTest::getChannelInfo(std::vector<byte_t> &resp_data)
{
	CommonClasses::IpmiRequest request(APPLICATION, 0x42);
	request.append<byte_t>(m_channelNum & 0x0F);//Channel Number bits 0-3, so mask 0x03 is more suitable

	//Build Command Object
	CommonClasses::IpmiResponse response(GET_LAN_CHANNEL_INFO_MIN_RSPLEN, GET_LAN_CHANNEL_INFO_MAX_RSPLEN);

	//Min Prev Level for GetLANConfig command is  : Operator
	setSessionPrivilegeLevel(PRIV_OPERATOR);

	//Send request data to the transport
	sendIpmiCmd(request, response);

	//Return Command Completion Status
	response.copy_to(0, resp_data);
}
//**************************************************************************



/**************************************************************************
Function Name	:	GetChannelInfo
Purpose			:   Issues the 'Get Channel Access' command
Parameters		:   IN  iChannel - LAN Channel number for which iParam is obtained
 			        OUT pRspData - The response data consists of the
                    completion code, and command response as per IPMI spec
****************************************************************************/
std::string
LanTest::getUserName(byte_t user_id)
{
	CommonClasses::IpmiRequest request(APPLICATION, 0x46);
	request.append<byte_t>(user_id & 0x1F);//user id

	//Build Command Object
	CommonClasses::IpmiResponse response(GET_USER_NAME_MIN_RSPLEN, GET_USER_NAME_MAX_RSPLEN);

	//Min Prev Level for GetLANConfig command is  : Operator
	setSessionPrivilegeLevel(PRIV_OPERATOR);

	//Send request data to the transport
	sendIpmiCmd(request, response);

	std::vector<byte_t> user_data;

	response.copy_to(0, user_data);

	//is null value or not, anyway we have null or end of array
	std::vector<byte_t>::iterator end_it = std::find_if(user_data.begin(), user_data.end(), local::isnull);

	//Make sure data has only Ascii Characters
	std::vector<byte_t>::iterator it = std::find_if(user_data.begin(), end_it, std::not1(std::ptr_fun<int, int>(::isprint))); //isascii
	if(it != end_it)
	{
		LOG_WARN<<"\n\tUser name string has invalid ASCII value.";
	}

	//copy data as long as possible
	std::stringstream user_str;
	std::copy(user_data.begin(), it, std::ostream_iterator<char>(user_str));

	std::string username = user_str.str();
	return username;
}
//**************************************************************************


LanTest::s_User_Access
LanTest::getUserAccess(byte_t user_id)
{
	CommonClasses::IpmiRequest request(APPLICATION, 0x44);
	request.append<byte_t>(m_channelNum)
		   .append<byte_t>(user_id & 0x1F);//user id

	//Build Command Object
	CommonClasses::IpmiResponse response(GET_USER_ACCESS_MIN_RSPLEN, GET_USER_ACCESS_MAX_RSPLEN);

	//Min Prev Level for GetLANConfig command is  : Operator
	setSessionPrivilegeLevel(PRIV_OPERATOR);

	//Send request data to the transport
	sendIpmiCmd(request, response);

	return response.getFromData<struct s_User_Access>(0);
}



LanTest::s_User_Payload_Access
LanTest::getUserPayloadAccess(byte_t user_id)
{
	CommonClasses::IpmiRequest request(APPLICATION, 0x4D);
	request.append<byte_t>(m_channelNum)
		   .append<byte_t>(user_id & 0x1F);//user id

	//Build Command Object
	CommonClasses::IpmiResponse response(GET_USER_P_ACCESS_MIN_RSPLEN, GET_USER_P_ACCESS_MAX_RSPLEN);

	//Min Prev Level for Get User payload access command is  : Admin
	setSessionPrivilegeLevel(PRIV_ADMIN);

	//Send request data to the transport
	sendIpmiCmd(request, response);

	return response.getFromData<struct s_User_Payload_Access>(0);
}



TestResult
LanTest::UserPayloadAccessTest(any_value &)
{
	LOG_DEBUG<<"\nUser payload access info command";
	if(!isRemoteConnection())
	{
		LOG_WARN<<"\nTest does not run in InBand, skipped" ;
		return TestUtil::TR_SKIPPED;
	}

	//Issue GetSessioninfo command and log user id is for session,
	std::vector<byte_t> response;
	getSessionInfo(response);

	if (response.size()<4)
	{
		LOG_ERR<<"\nSession info error";
		return TestUtil::TR_FAILURE;
	}

	/* Session Info obtained successfully take info about user	 */
	byte_t user_id = response[3];
	LOG_INFO << "\n\tUser Id: "<< (short)user_id;

	std::string username = getUserName(user_id);
	LOG_INFO << "\n\tUser name: '"<< username <<"'";

	s_User_Payload_Access user_p_access = getUserPayloadAccess(user_id);
	LOG_INFO <<"\n\tUser payloads access: ";
	byte_t payl = (user_p_access.st_payload1.st_payloads);
	payl >>= 1;
	LOG_INFO<<  "\n\t\tStandard payload 1 (SOL) is "<<((payl&0x01) ? "enabled" : "disabled");

	for(int i=2; i<8; i++) {
		payl >>= 1;
		LOG_INFO<<  "\n\t\tStandard payload "<< i << " is "<<((payl&0x01) ? "enabled" : "disabled");
	}

	payl = user_p_access.oem_payload1;
	for(int i=0; i<8; i++) {
		LOG_INFO<<  "\n\t\tOEM payload instance "<< i << " is "<<((payl&0x01) ? "enabled" : "disabled");
		payl >>= 1;
	}

	return TestUtil::TR_SUCCESS;
}

TestResult
LanTest::UserAccessTest(any_value &)
{
	LOG_DEBUG<<"\nUser info command";
	if(!isRemoteConnection())
	{
		LOG_WARN<<"\nTest does not run in InBand, skipped" ;
		return TestUtil::TR_SKIPPED;
	}

	//Issue GetSessioninfo command and log user id is for session,
	std::vector<byte_t> response;
	getSessionInfo(response);

	if (response.size()<4)
	{
		LOG_ERR<<"\nSession info error";
		return TestUtil::TR_FAILURE;
	}

	/* Session Info obtained successfully take info about user	 */
	byte_t user_id = response[3];
	LOG_INFO << "\n\tUser Id: "<< (short)user_id;

	std::string username = getUserName(user_id);
	LOG_INFO << "\n\tUser name: '"<< username <<"'";

	s_User_Access user_access = getUserAccess(user_id);
	LOG_INFO <<"\n\tUser channel access: ";
	LOG_INFO <<"\n\t\t" << (user_access.no_callin_access ? "callback" : "call-in or callback") <<" connection";
	LOG_INFO <<"\n\t\t" << (user_access.link_auth_access ? "enabled" : "disabled") <<" for link authentication";
	LOG_INFO <<"\n\t\t" << (user_access.ipmi_messaging_access ? "enabled" : "disabled") <<" for IPMI Messaging";

	return TestUtil::TR_SUCCESS;
}
