/*
/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2011 INTEL CORPORATION                       **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intel�s	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************
********************************************************************************/
/* TestClasses.cpp
 *
 *  Created on: 30-03-2011
 */

#include "parameters.h"
#include "DCMI_Configuration.h"
#include "TestClasses.h"
#include "any_value.hpp"

#include <iomanip>

using std::setw;
using std::setfill;
using std::left;

namespace TestUtil
{

template <typename T>
void deleteWrapper(T *ptr)
{
   	if(ptr == NULL)
   		return;
   	delete ptr;
}

std::string getTTypeName(const TestType &ttype)
{
	switch (ttype)
	{
	case TT_OPTIONAL:
		return "O";
	case TT_MANDATORY:
		return "M";
	}
	return "UNKNOWN";
}

std::string getTResultName(const TestResult &result)
{
	switch (result)
	{
	case TR_NONE:
		return "NONE";
	case TR_SUCCESS:
		return "PASS";//"Success";
	case TR_FAILURE:
		return "FAIL";//"Failure";
	case TR_CC_FAILURE:
		return "FAIL";//": CCode";
	case TR_SKIPPED:
		return "SKIPPED";
	case TR_NOT_SUPPORTED:
		return "SKIPPED";//Not Supported
	case TR_INVALID_LEN:
		return "FAIL";//": Length";
	case TR_NETWORK_ERR:
		return "FAIL";//": Network";
	}
	return "unknown";
}

/**
 * Print formated Test info
 *
 * @param stream - stream to print out data
 * @param info - object to print out
 *
 * @return stream...
 */
std::ostream &operator<<(std::ostream &stream, TestResult &info)
{
	return stream << getTResultName(info);
}

/**
 * Print formated Test Type
 *
 * @param stream - stream to print out data
 * @param info - object to print out
 *
 * @return stream...
 */
std::ostream &operator<<(std::ostream &stream, TestType &info)
{
	return stream << getTTypeName(info);
}



//single test case info
class TestInfo
{
	friend class TestExecutor;
	friend class TestSuite;
	friend class TestOutputText;
    friend class TestRunner;

	//methods to execute
	//test type
	TestType m_ttype;
	union
	{	//test method to execute
		TestSuite::Function m_ftpt;
		//info print method to execute
		TestSuite::FunctionInfo m_fipt;
	} function;

	//function argument ...
	CommonClasses::any_value m_fparam;
	//test description
	std::string m_desc;
	//test result
	TestResult m_result;


	TestInfo(TestSuite::Function fpt, const any_value &fparam, const std::string &desc, const TestType ttype)
	{
		if (ttype == TT_INFO)
		{
			throw std::invalid_argument("Invalid type of Test function");
		}

		m_ttype = ttype;

		function.m_ftpt = (fpt);
		m_fparam = (fparam);
		m_desc = (desc);
		m_result = (TR_NONE);
	}

	TestInfo(TestSuite::FunctionInfo fpt, const std::string &desc)
	{
		m_ttype = (TT_INFO);
		m_desc = (desc);
		function.m_fipt = (fpt);
	}
};//TestInfo

/*
 * TestSuit definition ..
 */
TestSuite::~TestSuite()
{
	//remove test info's
	std::for_each(m_tests.begin(), m_tests.end(), deleteWrapper<TestInfo>);
	//remove dependent suits ..
	std::for_each(m_suits.begin(), m_suits.end(), deleteWrapper<TestSuite>);
}

void TestSuite::addTestSuite(std::auto_ptr<TestSuite> &suit)
{
	if (suit.get()==NULL)
		return;
	m_suits.push_back(suit.release());
}


void TestSuite::addTest(Function fpt, const any_value &param, const std::string &desc, const TestType type)
{
	if (fpt == NULL)
		return;
	m_tests.push_back(new TestInfo(fpt, param, desc, type));
}


void TestSuite::addInfo(FunctionInfo fpt, const std::string &desc)
{
	if (fpt == NULL)
		return;
	m_tests.push_back(new TestInfo(fpt, desc));
}


class Configurator
{
	DcmiConfiguration &m_conf;
public:
	Configurator(DcmiConfiguration &conf) :
		m_conf(conf)
	{}

	void operator() (TestSuite *suit)
	{
		suit->doConfigure(m_conf);
	}
};

void TestSuite::doConfigure(DcmiConfiguration &option)
{
	Configurator cfg(option);
	std::for_each(m_suits.begin(), m_suits.end(), cfg);
}


void TestSuite::runSuite(TestOutput &out)
{
	runTests(out);
	runSuites(out);
}


//single test executor functor
class TestRunner
{
	TestSuite *m_owner;
	TestOutput &m_out;

public:

	TestRunner(TestSuite *owner, TestOutput &out) :
		m_owner(owner), m_out(out)
	{ }

	void ExecuteInfo(TestInfo &info)
	{
		m_out.test_info(info, "");
		try
		{
			TestSuite::FunctionInfo fpt = info.function.m_fipt;
			info.m_result = (*m_owner.*fpt)(m_out);
		} catch (TestResult &resCode)
		{
			info.m_result = resCode;
		} catch (...)
		{
			info.m_result = TR_FAILURE;
		}
	}

	void ExecuteTest(TestInfo &info)
	{
		std::string msg;
		try
		{
			TestSuite::Function fpt = info.function.m_ftpt;
			info.m_result = (*m_owner.*fpt)(info.m_fparam);
		} catch (TestResult &resCode)
		{
			info.m_result = resCode;
		} catch (std::exception &e)
		{
			msg = e.what();
			info.m_result = TR_FAILURE;
		} catch (...)
		{
			msg = "Unknown exception occurred";
			info.m_result = TR_FAILURE;
		}

		m_out.test_info(info, msg);
	}

	void operator() (TestInfo *info)
	{
		if (info == NULL)
			return;

		m_out.test_start(info->m_desc);

		if (info->m_ttype == TT_INFO)
			ExecuteInfo(*info);
		else
			ExecuteTest(*info);

		m_out.test_end();
	}

};//TestRunner


void TestSuite::runTests(TestOutput &out)
{
	if ( m_tests.empty() ) {
		return ;
	}

	start_up(out);

	TestRunner run(this, out);
	std::for_each(m_tests.begin(), m_tests.end(), run);
	tear_down(out);
}

//single suit functor executor
class SuitRunner
{
   TestOutput &m_out;

public:

   SuitRunner (TestOutput &out) :
	   m_out(out)
   { }

   void operator() (TestSuite *suit)
   {
	    suit->runSuite(m_out);
   }
}; //SuitRunner

void TestSuite::runSuites(TestOutput &out)
{
   if ( m_suits.empty() ) {
   		return ;
   	}

   SuitRunner run(out);
   std::for_each(m_suits.begin(), m_suits.end(), run);
}

void TestSuite::start_up(TestOutput &out)
{
	out.suite_start(m_name);
}

void TestSuite::tear_down(TestOutput &out)
{
	out.suite_end() ;
}

/*------------------------------------------------------------*/

void TestOutputText::print_header(const std::string &name)
{
	m_stream << name << std::endl;
}

void TestOutputText::suite_start(const std::string &name)
{
	std::stringstream ss;
	ss << "Tests " << std::left << ++m_suitcounter << ", " << name << std::endl;

	LOG_SUMMARY	<< ss.str();
	m_stream <<  ss.str();;

	m_testcounter = 0;
};

TestOutput &TestOutputText::raw_print(const std::string &str)
{
	m_stream << str;

	return *this;
}

void TestOutputText::suite_end()
{
	m_stream << std::endl;
	LOG_SUMMARY	<< std::endl;
}

void TestOutputText::test_start(const std::string &name)
{
	std::stringstream ss;
	std::stringstream ssn;
	ssn << "Test Case " << std::left << m_suitcounter << "." << ++m_testcounter ;
	ss << setw(17) << setfill(' ') << std::left << ssn.str() << ": ";

	std::string str = name.substr(0, 50);
	if (name.length() > 50)
	{
		ss << str << "\n    ";
		str = name.substr(50);
	}
	ss << setw(50) << setfill(' ') << left << str;

	LOG_SUMMARY	<< ss.str();;
	m_stream << ss.str();;
}

void TestOutputText::test_info(const TestInfo &info, const std::string &error_msg)
{
	if (info.m_ttype != TT_INFO)
	{
		std::stringstream ss;
		ss << /*setw(3) << setfill(' ') << left << getTTypeName(info.m_ttype)
				 <<*/ "   " << getTResultName(info.m_result);

		LOG_SUMMARY << "\n" <<  ss.str();
		m_stream <<  ss.str();
	}

	//add custom error message
	if ((info.m_result != TR_SUCCESS))
	{
		if (error_msg.length())
			LOG_SUMMARY<< "\n\t-" << error_msg ;
		else {
			switch (info.m_result)
			{
			case TR_CC_FAILURE:
				LOG_SUMMARY<< "\t-Invalid completion code";//": CCode";
				break;
			case TR_INVALID_LEN:
				LOG_SUMMARY<< "\t-Invalid length";//": Length";
				break;
			case TR_NETWORK_ERR:
				LOG_SUMMARY<< "\t-Network error";//": Network";
				break;
			default: ;
			}
		}
	}
}

void TestOutputText::test_end()
{
	LOG_SUMMARY	<< std::endl;
	m_stream << std::endl;
}


}//namespace
