/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2011 INTEL CORPORATION                       **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intel�s	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*
 * TransportManager.cpp
 *
 *  Created on: 31-03-2011
 */
#include <stdexcept>
#include "parameters.h"

#include "Logger.h"
#include "DCMI_Configuration.h"

#include "TransportCommon.h"
#include "TransportPluginDll.h"
#include "TransportManager.h"


namespace TransportPlugin
{

class TransportPlugInImpl: public TransportPlugIn
{   //bellow usage of composition instead of inheritance
	std::auto_ptr<transport_plugin_interface> m_pTpi;
	std::auto_ptr<TransportPluginDll> m_pDll;

	std::string m_pluginName;

	bool m_isConnected;

	friend class TransportManager;

protected:

	TransportPlugInImpl(const std::string &pluginName)
	{
		m_pluginName = pluginName;
		if (!m_pluginName.length())
			throw std::invalid_argument("Transport plug-in name invalid");

		//only name without extension .. (.dll)
		m_pDll.reset(new TransportPluginDll(m_pluginName.c_str()));

		m_pTpi.reset(new transport_plugin_interface);
		m_pDll->GetInterface(m_pTpi.get());

		struct dcmi_init_info_t initInfo /* = { .loggerFun = TransportPlugin::Logger::log } */;
		initInfo.loggerFun = TransportPlugin::Logger::log;
		m_pTpi->initialize(&initInfo);
		m_isConnected = false;
	}

public:

	virtual ~TransportPlugInImpl()
	{
		disconnect();
		m_pTpi->shutdown();
	}

	inline bool isConnected()
	{
		return m_isConnected;
	}

	int connect()
	{
		if (m_isConnected)
			return E_OK;

		int ret = m_pTpi->connect();
		if (ret == E_OK)
			m_isConnected = true;
		return ret;
	}

	int openSession()
	{
		if (!isConnected())
         return E_NOT_CONNECTED;

      return m_pTpi->open_session();
	}

	int closeSession()
	{
      return m_pTpi->close_session();
	}

	void disconnect() throw()
	{
		if (!m_isConnected)
			return;

		m_pTpi->disconnect();
		m_isConnected = false;
	}

	int configure(const std::string &name, const std::string &value)
	{
		if (!(name.length() && value.length()))
			return E_INVALID_VALUE;

		return m_pTpi->configure(name.c_str(), value.c_str());
	}

	int sendIpmiCmd(CommonClasses::Request &request, CommonClasses::Response &response, byte_t maxRetryCount)
	{
		//Fix for Keeping Session Alive - this entire fn changed
		bool bIsTimeOut = true;
		int iCompStatus = 0, iretryCount = 1;
		if (isConnected())
		{
			do
			{ //do'nt retry unless this is a network problem?
				iCompStatus = m_pTpi->sendIpmiCmd(&request, &response);
				if (iCompStatus == E_NETWORK_PROBLEM)
				{
					bIsTimeOut = true;
				}
				else
				{ //Command completed without errors
					bIsTimeOut = false;
				}

				if (bIsTimeOut)
				{
					if (iretryCount > 1)
					{
						LOG_WARN << "\nNetwork Problem, retry "
								<<static_cast<int>(iretryCount) << " of " << static_cast<int>(maxRetryCount)
								<< std::endl;
					}
					else
					{
						LOG_WARN << "\nNetwork Problem" << std::endl;
					}
				}
				else
				{//Command completed without errors
					break;
				}
				//could bee here a small delay ..
			} while (++iretryCount <= maxRetryCount);
		} //if connected

		//todo: determine if FW has auto-disconnected (>60 seconds)
		if (bIsTimeOut || (!isConnected()))
		{
			LOG_WARN << "\nSession Timed out... Attempting to reconnect and retransmit " << std::endl;
			LOG_WARN << "\n\tDisconnecting" << std::endl;
			//make sure the current session is Disconnected
			disconnect();
			LOG_WARN << "\n\tConnecting..." << std::endl;
			//re-establish a new session
			iCompStatus = connect();
			if (iCompStatus == E_OK)
			{ //Session has been established successfully, Re-send the command
				iCompStatus = m_pTpi->sendIpmiCmd(&request, &response);
			}
		}

		if (iCompStatus == E_NETWORK_PROBLEM)
		{//recovery failed - unable to re-establish session with UUT, return Error
			LOG_WARN << "\nLost connection with the ME, Aborting...\n"	;
			disconnect();
			return E_NETWORK_PROBLEM;
		}

		return iCompStatus;
	}

	transport_plugin_id connectionType()
	{
		return TransportManager::connectionType();
	}

};


/*
 * ********************************
 * Connection manager -> Singleton pattern for create and configure .... transport layer
 */
static std::auto_ptr<TransportPlugIn> m_Transport;
static std::map<transport_plugin_id, std::string> m_Plugins;
static transport_plugin_id m_PluginId;

void Init()
{
		m_Plugins[TP_RMCPP] = "TransportRMCPP";
		m_Plugins[TP_INB]   = "TransportInBand";
}
//early static initializer ...
int init = (Init(), 1);


void TransportManager::doConfigure(UserOption useropt, DcmiConfiguration &config)
{
	if (useropt == IB_KCS_HECI)
	{   /*Check if all the Configuration values read from userconf.cfg are in valid range
	 	 *before proceeding with test*/
		LOG_DEBUG<<"\nChecking InBand transport configuration";
		if (config.CheckInBandConfiguration())
			throw std::runtime_error(
         "\nError: Please check configuration for incorrect values, transport layer: InBand ");
	}
	else
	{	/*Check if all the Configuration values read from userconf.cfg are in valid range
		 *before proceeding with test*/
		LOG_DEBUG<<"\nChecking RMCPP transport configuration";
		if (config.CheckRMCPPConfiguration())
			throw std::runtime_error(
         "\nError: Please check configuration for incorrect values, transport layer: RMCPP");
		/*
		 * Get the password from the user input to establish a connection. Also verify the password length.
		 * If it is valid password, insert the password entity. */
		config.GetPassword();
	}

	m_PluginId = UserOption2PluginId(useropt);
	std::string name = m_Plugins.find(m_PluginId)->second;

	LOG_DEBUG<<"\nTransport plugin name: " << name <<std::endl;
	m_Transport.reset(new TransportPlugInImpl(name));

	name = UserOption2ConfigName(useropt);
	if (config.applyPluginConfiguration(
			static_cast<TransportPlugInImpl*> (m_Transport.get())->m_pTpi.get(),
			name.c_str()))
	{
		throw std::runtime_error("Failed to configure transport layer: \""
				+ name + "\"");
	}
}

TransportPlugIn &TransportManager::getCurrentTransport()
{
	if (m_Transport.get() == NULL)
		throw std::logic_error("Configure transport layer before!");
	return *m_Transport;
}

void TransportManager::connect()
{
   int stat = getCurrentTransport().connect();
   if (stat)
   {
	   std::stringstream msg;
	   msg<<"\n\tABORT: Transport layer returned an error: "<<std::hex<<std::showbase<<stat;

	   if (getCurrentTransport().connectionType() == TP_INB)
	   {
		   msg << "\n\tUnable to establish connection to DCMI-Hi driver";
		   msg << "\n\tCheck following and re-run the Test Suite";
		   msg << "\n\t -Verify Management Engine Interface driver";
		   msg << "\n\t -Verify DCMI driver";
	   }
	   else
	   {
		   msg << "\n\tUnable to establish RMCP+ connection to UUT";
		   msg << "\n\tCheck following and re-run the Test Suite";
		   msg << "\n\t -Verify Account with Username exists in Manageability Controller in UUT ";
		   msg << "\n\t -Verify Password ";
		   msg << "\n\t -Check  targetIP/username in UserConf.cfg ";
		   msg << "\n\t -Verify CipherSuite supported on UUT \n";
	   }

	   throw std::runtime_error(msg.str());
   }
}



void TransportManager::disconnect() throw()
{
	try	{
		getCurrentTransport().disconnect();
    } catch (std::exception &e)	{
		LOG_ERR << e.what() << std::endl;
	}
}

transport_plugin_id TransportManager::connectionType()
{
	return m_PluginId;
}

}//namespace
