/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/

#include "Logger.h"
#include "OsSpecificSockets.h"
#include "IpmiRequest.h"
#include "IpmiResponse.h"

#include "IpmiWMIImpl.h"

#define CHECK_HRESULT(hr, funcName) \
   if (FAILED(hr)) \
      throw CError(__FUNCTION__": "##funcName, hr)

using namespace std;

//Early COM library initalization ....
//
static struct CoInit
{
    CoInit() { CoInitialize(NULL); }
    ~CoInit() { CoUninitialize(); }
} coInit;


IpmiWMIImpl::IpmiWMIImpl(void)
{
    //LOG_DEBUG << "Initializing COM";
    //HRESULT hr = CoInitialize(NULL);
    //CHECK_HRESULT(hr, "CoInitialize");

    // Microsoft_IPMI WMI interface
    LOG_DEBUG << "Initializing COM Security";
    HRESULT hr = CoInitializeSecurity(NULL, -1, NULL, NULL, RPC_C_AUTHN_LEVEL_PKT,
                              RPC_C_IMP_LEVEL_IMPERSONATE, NULL, EOAC_NONE, NULL);
    CHECK_HRESULT(hr, "CoInitializeSecurity");

    CComPtr<IWbemLocator> wbemLocator;
    hr = wbemLocator.CoCreateInstance(CLSID_WbemLocator, NULL, CLSCTX_INPROC_SERVER);
    CHECK_HRESULT(hr, "CoCreateInstance(CLSID_WbemLocator)");

    // obtain a pointer to the services
    hr = wbemLocator->ConnectServer(L"root\\wmi",
                                    NULL /*Username*/,
                                    NULL /*Password*/,
                                    NULL /*Locale*/,
                                    0 /*Flags*/,
                                    NULL /*Authority*/,
                                    NULL /*Context*/,
                                    &wbemServices);
    CHECK_HRESULT(hr, "IWbemLocator::ConnectServer(root\\wmi)");

    wbemLocator.Release();

    // configure the proxy
    hr = CoSetProxyBlanket(wbemServices, RPC_C_AUTHN_WINNT, RPC_C_AUTHZ_NONE, NULL,
                           RPC_C_AUTHN_LEVEL_CALL, RPC_C_IMP_LEVEL_IMPERSONATE, NULL,
                           EOAC_NONE);
    CHECK_HRESULT(hr, "CoSetProxyBlanket");

    bstr_t ipmiClassName("Microsoft_IPMI");

    // enumerate the Microsoft_IPMI instances available locally
    // (only one is expected)
    CComPtr<IEnumWbemClassObject> ipmiInstanceEnum;
    hr = wbemServices->CreateInstanceEnum(ipmiClassName, WBEM_FLAG_SHALLOW | WBEM_FLAG_FORWARD_ONLY,
                                          NULL, &ipmiInstanceEnum);
    CHECK_HRESULT(hr, "IWbemServices::CreateInstanceEnum(Microsoft_IPMI)");

    ULONG retObjectCount = 0;
    CComPtr<IWbemClassObject> ipmiInstance;
    hr = ipmiInstanceEnum->Next(WBEM_NO_WAIT, 1, &ipmiInstance, &retObjectCount);
    CHECK_HRESULT(hr, "IEnumWbemClassObject::Next");
    if (1 != retObjectCount)
        throw runtime_error("No Microsoft_IPMI object found");

    // get the path of the first object
    CComVariant comVar;
    hr = ipmiInstance->Get(L"__PATH", 0, &comVar, NULL, NULL);
    CHECK_HRESULT(hr, "IWbemClassObject::Get(__PATH)");

    ipmiInstancePath = comVar.bstrVal;
    LOG_ERR << "WMI object instance: " << ipmiInstancePath;

    comVar.Clear();
    hr = ipmiInstance->Get(L"BMCAddress", 0, &comVar, NULL, NULL);
    CHECK_HRESULT(hr, "IWbemClassObject::Get(BMCAddress");
    bmcAddress = comVar.bVal;
    LOG_ERR << "BMC Address: 0x" << hex << (int)bmcAddress;

    // cache in-params signature
    CComPtr<IWbemClassObject> ipmiClass;
    hr = wbemServices->GetObjectA(ipmiClassName, WBEM_FLAG_RETURN_WBEM_COMPLETE, NULL, &ipmiClass, NULL);
    CHECK_HRESULT(hr, "IWbemServices::GetObject(Microsoft_IPMI)");
    hr = ipmiClass->GetMethod(L"RequestResponse", 0, &inParamsDef, NULL);
    CHECK_HRESULT(hr, "IWbemClassObject::GetMethod(RequestResponse)");
}

IpmiWMIImpl::~IpmiWMIImpl()
{
//    CoUninitialize();
}

IpmiResponse
IpmiWMIImpl::requestResponse(const IpmiRequest& req)
{
    CComPtr<IWbemClassObject> inParams;
    CComPtr<IWbemClassObject> outParams;

    LOG_DEBUG << "IpmiWMIImpl::requestResponse() invoked";

    //NOTE:
    //
    // All MS coding examples use SpawnInstance() but this
    // method looses all qualifiers except CIMTYPE and ID.
    // Clone() copies them all including WmiSizeIs
    // of RequestData, so we do not receive INVALID_PARAMETER.
    //
    // WinRM seems to be affected by this ?bug? as well.
    //
    //HRESULT hr = inParamsDef->SpawnInstance(0, &inParams);
    HRESULT hr = inParamsDef->Clone(&inParams);

    CHECK_HRESULT(hr, "IWbemClassObject::SpawnInstance (inParams)");

    // 1. Translate in-params

    CComVariant arg(static_cast<BYTE>(req.getNetFn()));
    hr = inParams->Put(L"NetworkFunction", 0, &arg, 0);
    CHECK_HRESULT(hr, "Setting NetworkFunction");
    arg.Clear();
    LOG_DEBUG << "NetworkFunction set to " << hex << (int)req.getNetFn();

    arg = static_cast<BYTE>(req.getLun());
    hr = inParams->Put(L"Lun", 0, &arg, 0);
    CHECK_HRESULT(hr, "Setting Lun");
    arg.Clear();
    LOG_DEBUG << "Lun set to " << hex << (int)req.getLun();

    arg = static_cast<BYTE>(req.getCmd());
    hr = inParams->Put(L"Command", 0, &arg, 0);
    CHECK_HRESULT(hr, "Setting Command");
    arg.Clear();
    LOG_DEBUG << "Command set to " << hex << (int)req.getCmd();

    arg = bmcAddress;
    hr = inParams->Put(L"ResponderAddress", 0, &arg, 0);
    CHECK_HRESULT(hr, "Setting ResponderAddress");
    arg.Clear();
    LOG_DEBUG << "ResponderAddress set to " << hex << 0x20;

    CComSafeArray<BYTE> argArray;
    arg = static_cast<int>(req.getData().size());
    hr = inParams->Put(L"RequestDataSize", 0, &arg, 0);
    CHECK_HRESULT(hr, "Setting RequestDataSize");
    arg.Clear();
    LOG_DEBUG << "RequestDataSize set to " << hex << req.getData().size();

    if (req.getData().size() > 0)
    {
        hr = argArray.Add(static_cast<ULONG>(req.getData().size()), &req.getData().front());
        CHECK_HRESULT(hr, "Creating RequestData array");
    }
    else
    {
        hr = argArray.Create();
        CHECK_HRESULT(hr, "Creating empty RequestData array");
    }
    arg = argArray;
    argArray.Destroy();

    hr = inParams->Put(L"RequestData", 0, &arg, 0);
    CHECK_HRESULT(hr, "Setting RequestData");
    arg.Clear();

    // 2. Execute method

   hr = wbemServices->ExecMethod(ipmiInstancePath, L"RequestResponse", 0, NULL,inParams, &outParams, NULL);
   CHECK_HRESULT(hr, "IWbemServices::ExecMethod");

   // 3. Translate out-params

   hr = outParams->Get(L"CompletionCode", 0, &arg, NULL, NULL);
   CHECK_HRESULT(hr, "Getting CompletionCode");
   unsigned char completionCode = arg.bVal;
   arg.Clear();

   hr = outParams->Get(L"ResponseData", 0, &arg, NULL, NULL);
   CHECK_HRESULT(hr, "Getting ResponseData");
   argArray.Attach(static_cast<SAFEARRAY*>(arg.byref));

   IpmiResponse rsp(completionCode, argArray.GetCount(),
                             reinterpret_cast<BYTE*>(argArray.m_psa->pvData));

   argArray.Detach();

   return rsp;
}
