/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/

#include <string>
#include <algorithm>

#include "TransportPluginImpl.h"
#include "IpmiRequest.h"
#include "IpmiResponse.h"

#include "IpmiProxy.h"
#include "RemoteLogger.h"
#include "IpmiDCMIImpl.h"

#ifdef WIN32
#include "IpmiWMIImpl.h"
#else
#include "IpmiDevIpmi0Impl.h"
#endif

using namespace TransportPlugin;

namespace InBandTransportPlugin
{

TransportPluginImpl* TransportPluginImpl::instance = NULL;

TransportPluginImpl::TransportPluginImpl()
                            :ipmiProxy(NULL)
{
#ifdef WIN32
	m_IpmiDriverName = "Microsoft_IPMI"; // Windowd default
#else
	m_IpmiDriverName = "ipmi0"; // Linux default
#endif

}

TransportPluginImpl::~TransportPluginImpl()
{
}

TransportPluginImpl*
TransportPluginImpl::getInstance()
{
    if (NULL == instance)
    {
        instance = new TransportPluginImpl;
    }
    return instance;
}

void
TransportPluginImpl::initialize()
{
    getInstance();
}

void
TransportPluginImpl::destroy()
    throw()
{
    try
    {
        if (NULL != instance)
        {
            delete instance;
            instance = NULL;
        }
    }
    catch (...)
    {
    }
}

int
TransportPluginImpl::start(const dcmi_init_info_t* initInfo)
{
    if (initInfo == NULL)
    {
        return E_OTHER;
    }

    RemoteLogger::setLogPrefix("InBand transport");
    RemoteLogger::setLoggerFunc(initInfo->loggerFun);

    LOG_DEBUG << "Successful plugin initialization.";

    return E_OK;
}

int
TransportPluginImpl::shutdown()
{
    return E_OK;
}

int
TransportPluginImpl::configure(const char* paramName, const char* value)
   throw()
{
    LOG_DEBUG << "Configure: " << paramName << " = " << value;

	std::string param (paramName);
	std::transform(param.begin(), param.end(), param.begin(), ::toupper);

    if(param.compare("DRIVERNAME")==0)
    {
    	m_IpmiDriverName = value;
    	return E_OK;
    }
    else
    {
    	LOG_WARN << "Configure: Unknown configuration parameter: " << paramName;
    	return E_NAME_UNKNOWN;
    }
}

int
TransportPluginImpl::connect()
    throw()
{
    try
    {
        if (NULL == ipmiProxy.get())
        {
			 ipmiProxy.reset(createIpmiProxyDriver(m_IpmiDriverName.c_str()));
        }

    }
    catch (std::exception& e)
    {
       LOG_DEBUG << "\nERROR: connect(): " << e.what() << std::endl;
        return E_OTHER;
    }
    catch (...)
    {
        return E_OTHER;
    }
    return E_OK;
}

int
TransportPluginImpl::sendIpmiCmd(const struct dcmi_req_t* req, struct dcmi_rsp_t* rsp)
    throw()
{
    try
    {
    	IpmiRequest ipmiReq(req->rsAddr, req->netFun, req->rsLun, req->cmd, static_cast<unsigned int>(req->len), req->data);
        IpmiResponse ipmiRsp(ipmiProxy->requestResponse(ipmiReq));

        rsp->compCode = ipmiRsp.getCompletionCode();
        rsp->len = (int)ipmiRsp.getData().size();

#ifdef _MSC_VER
        std::copy(ipmiRsp.getData().begin(), ipmiRsp.getData().end(), stdext::checked_array_iterator<byte_t*>(rsp->data, sizeof(rsp->data)));
#else
        std::copy(ipmiRsp.getData().begin(), ipmiRsp.getData().end(), rsp->data);
#endif
    }
    catch (std::exception& e)
    {
       LOG_ERR << "ERROR: sendIpmiCmd(): " << e.what() << std::endl;
        return E_OTHER;
    }
    catch (...)
    {
        return E_OTHER;
    }

    return E_OK;
}

int
TransportPluginImpl::disconnect()
    throw()
{
	try {
		ipmiProxy.reset(NULL);
	}
	catch (std::exception& e) {
		LOG_DEBUG << "\nERROR: disconnect(): " << e.what() << std::endl;
		return E_OTHER;
	}
	catch (...) {
		return E_OTHER;
	}
	return E_OK;
}


IpmiProxy *
TransportPluginImpl::createIpmiProxyDriver(const char* ipmiDevice)
{
	std::string ipmidev (ipmiDevice);
	std::transform(ipmidev.begin(), ipmidev.end(), ipmidev.begin(), ::toupper);

#ifdef WIN32
	if(ipmidev.compare("MICROSOFT_IPMI")==0)  {
		LOG_DEBUG << "Create ipmi transport device: \"Microsoft_IPMI\"";
		return new IpmiWMIImpl();
	}
#else
	if(ipmidev.compare("IPMI0")==0)  {
		LOG_DEBUG << "Create ipmi transport device: \"/dev/ipmi0\"";
		return new IpmiDevIpmi0Impl();
	}
#endif

	if(ipmidev.compare("DCMI")==0)  {
		LOG_DEBUG << "Create ipmi transport device: \"DCMI\"";
		return new IpmiDCMIImpl();
	}

	throw std::invalid_argument("Unknown IPMI transport device" );
}

}//namespace

