/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/

#ifndef _TRANSPORT_PLUGIN_IMPL_H
#define _TRANSPORT_PLUGIN_IMPL_H

#include <string>
#include <limits.h>
#include <memory>

#include "OsSpecificSockets.h"

// forward declarations

struct dcmi_req_t;
struct dcmi_rsp_t;
struct dcmi_init_info_t;

class IpmiProxy;

namespace InBandTransportPlugin
{

/**
 * Singleton implementation for TCP Inband Transport Plug-in.
 */
class TransportPluginImpl
{
public:
    /**
     * Dtor.
     */
    ~TransportPluginImpl();

    /**
     * Retrieves the only instance of the plug-in object.
     *
     * @return Pointer to the plug-in object.
     */
    static TransportPluginImpl* getInstance();

    /**
     * Initializes the plugin. Internally the method calls <code>getInstance()</code>.
     * This method may be called in client code to control creation time
     * of the plug-in object without calling <code>getInstance()</code> explicitely
     * just to create the object.
     */
    static void initialize();

    /**
     * Destroys the instance of the plug-in. Internally the method calls destructor.
     * This method may be called in client code to control destruction time
     * of the plug-in object without calling dtor explicitely.
     */
    static void destroy() throw();

    // public Transport Plug-in API implementation

    /**
     *
     * Initializes the transport library and sets initial mandatory
	 * configuration parameters, such as logger functionality.
     *
     * @param initInfo Pointer to the initialization information data structure.
     *
     * @retval E_OK Successful initialization.
     * @retval E_OTHER Other error.
     */
    int start(const dcmi_init_info_t* initInfo);

    /**
     *
     * Performs final cleanup such as freeing allocated memory.
     *
     * @retval E_OK Successful shutdown.
     * @retval E_OTHER Other error.
     */
    int shutdown();

    /**
     * Sets a configuration option of the plug-in.
     *
     * The following options are supported:
     *
     * Hostname : string
     *    Name of the UUT.
     *
     * Verbose : boolean
     *    Toggles verbose logging of the plug-in activities.
     *
     * @param paramName Name of the option.
     * @param value Value of the option.
     *
     * @return See Transport Plug-in API specification for the
     *    description of return values.
     */
    int configure(const char* paramName, const char* value) throw();

    /**
     * Connects to the UUT.
     * @return See Transport Plug-in API specification for the
     *    description of return values.
     */
    int connect() throw();

    /**
     * Sends an IPMI request to the UUT and receives a response.
     *
     * @param req IPMI request.
     * @param rsp IPMI response.
     *
     * @return See Transport Plug-in API specification for the
     *    description of return values.
     */
    int sendIpmiCmd(const struct dcmi_req_t* req, struct dcmi_rsp_t* rsp) throw();

    /**
     * Disconnects from UUT.
     *
     * @return See Transport Plug-in API specificiation for the
     *    description of return values.
     */
    int disconnect() throw();

protected:
    /**
     *Factory method, create specific Ipmi object handling different transport device
     * throws exception in case of error - unknown transport device name
     *
     * @param ipmiDevice - ipmi device name, supported:
     * "DCMI" over  HECI
     * "Microsoft_IPMI" windows wmi
     * "IPMI" linux
     *
     *
     * @return IpmiProxy object @see IpmiWMIImpl
     */
    IpmiProxy * createIpmiProxyDriver(const char* ipmiDevice);

private:
    /// Hidden ctor for singleton.
    TransportPluginImpl();

    /// Hidden copy ctor for singleton.
    TransportPluginImpl(TransportPluginImpl const&);

    /// Hidden assignment operator for singleton.
    TransportPluginImpl& operator=(TransportPluginImpl const&);

      /// Pointer to the only instance of the class.
    static TransportPluginImpl* instance;

    // Configuration
	 std::string m_IpmiDriverName;

    // End-of-Configuration
	std::auto_ptr<IpmiProxy> ipmiProxy;
    //IpmiProxy * ipmiProxy;
};

}

#endif //#ifndef _TCP_INBAND_TRANSPORT_PLUGIN_IMPL_H
