/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008 INTEL CORPORATION                       **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intel�s	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/

#include "transport.h"
#include "RmcppTransportPluginImpl.h"
#include "OsSpecificSockets.h"
#include "IpmiSession.h"
#include "RemoteLogger.h"

#include <stdarg.h>

using namespace std;
using namespace TransportPlugin;

namespace RmcppTransportPlugin
{

RmcppTransportPluginImpl* RmcppTransportPluginImpl::instance = NULL;

RmcppTransportPluginImpl::RmcppTransportPluginImpl()
                        :ipmiSession(NULL), ip("127.000.000.001"),
                         userName(""),password(""),
                         authType(AUTH_TYPE_RMCPP),
                         authAlgo(AUTH_ALGO_RAKP_NONE),
                         integAlgo(INTEG_ALGO_NONE),
                         confidAlgo(CONFID_ALGO_NONE),
                         doPing(false)

{
#ifdef	WIN32
    // Init Windows Sockets
    WSADATA wsaData;
    int rc = WSAStartup(MAKEWORD(REQ_WINSOCK_VER_MAJOR, REQ_WINSOCK_VER_MINOR), &wsaData);
    if (NO_ERROR != rc)
    {
        const char* errorMsg = "Uknown WinSock intialization error";
        if (WSASYSNOTREADY == rc)
        {
            errorMsg = "Network subsystem is not ready for communication";
        }
        else if (WSAVERNOTSUPPORTED == rc)
        {
            errorMsg = "Requested version of Windows Sockets is not supported";
        }
        throw runtime_error(errorMsg);
    }
    //prevent debug builds from filling 0xFD
    _CrtSetDebugFillThreshold( 0);
#endif // WIN32
}

RmcppTransportPluginImpl::~RmcppTransportPluginImpl()
{
    delete ipmiSession;

#ifdef	WIN32
    // Clean up Windows Sockets
    WSACleanup();

    //Replace deprecated function
    #define strncpy(x,y,z)	strncpy_s(x,sizeof(x),y,z)
#endif // WIN32
}

RmcppTransportPluginImpl*
RmcppTransportPluginImpl::getInstance()
{
    if (NULL == instance)
    {
        instance = new RmcppTransportPluginImpl;
    }
    return instance;
}

void
RmcppTransportPluginImpl::initialize()
{
    getInstance();
}

void
RmcppTransportPluginImpl::destroy()
    throw()
{
    try
    {
        if (instance)
        {
            delete instance;
            instance = NULL;
        }
    }
    catch (...)
    {
    }
}

int
RmcppTransportPluginImpl::start(const dcmi_init_info_t* initInfo)
{
    if (initInfo == NULL)
    {
        return E_OTHER;
    }

    RemoteLogger::setLogPrefix("RMCPP  transport");
    RemoteLogger::setLoggerFunc(initInfo->loggerFun);

    LOG_DEBUG << "Successful plugin initialization.";

    return E_OK;
}

int
RmcppTransportPluginImpl::shutdown()
{
	/*1. Memories have been freed as and when they get created.
	     So it is not necessary to free the memory in Shutdown().
	  2. RMCPP does not use libraries, and unloading itself does not make sense,
	     so there are none to unload.*/
		return E_OK;
}

int
RmcppTransportPluginImpl::configure(const char* paramName, const char* value)
    throw()
{
    int optionNumber = match(paramName, 10,
                             "targetip", "username", "password",
                             "confid", "integ", "format",
                             "bmcKey", "ping","ciphersuite","send_sol",
                             NULL);
	int RetVal = E_OK;

	if (value == NULL)
	{
		LOG_DEBUG<<"configure called with NULL value";
		return E_INVALID_VALUE;
	}

    switch (optionNumber)
    {
    case 0:
        //If TargetIP is valid the assign the IP to make a connection. Else return an error.
        RetVal = CheckTargetIP(value);//const_cast<char *>(value));
		if(RetVal == E_OK)
			ip = value;
		else
			return RetVal;

        break;

    case 1:
        // If username is valid then assign the username.
		RetVal = CheckUserName(value);
		if(RetVal == E_OK)
		{
			userName = value;
		}
		else
		{
			LOG_DEBUG<<"UserName is empty, using NULL user account";
			return RetVal;
		}

        break;

    case 2:
        //If Password is valid then assign the password.
		RetVal = CheckPassword(value);
		if(RetVal == E_OK)
			password = value;
		else
			return RetVal;

        break;

    case 3:
		//Confidentiality algorithm to use for RMCP+ session encryption.
        optionNumber = match(value, 2, "NONE", "AES_CBC_128", NULL);
        switch (optionNumber)
        {
        case 0:
            confidAlgo = CONFID_ALGO_NONE;
            break;

        case 1:
            confidAlgo = CONFID_ALGO_AES_CBC_128;
            break;

        default:
			LOG_DEBUG<<"ERROR: Invalid Confidentiality algorithm to "
				"use for RMCP+ session encryption.";
			LOG_DEBUG<<"Expected Confidentiality algorithm is 'NONE' and 'AES_CBC_128'";
			return E_INVALID_VALUE;
        }
        break;

    case 4:
		//Integrity algorithm to use for RMCP+ session integrity.
        optionNumber = match(value, 5, "NONE", "HMAC_SHA1_96", "HMAC_MD5_128", "MD5_128","HMAC_SHA256_128", NULL);
        switch (optionNumber)
        {
        case 0:
            integAlgo = INTEG_ALGO_NONE;
            break;

        case 1:
            integAlgo = INTEG_ALGO_HMAC_SHA1_96;
            break;

        case 2:
            integAlgo = INTEG_ALGO_HMAC_MD5_128;
            break;

        case 3:
            integAlgo = INTEG_ALGO_MD5_128;
            break;

        case 4:
            integAlgo = INTEG_ALGO_HMAC_SHA256_128;
            break;

		default:
			LOG_DEBUG<<"ERROR: Invalid Integrity algorithm to use for RMCP+ session integrity.";
			LOG_DEBUG<<"Expected Integrity algorithm: 'NONE', 'HMAC_SHA1_96', 'HMAC_MD5_128', 'MD5_128'";
			return E_INVALID_VALUE;
        }
        break;

    case 5:
        //format
        optionNumber = match(value, 4, "RMCPP_MD5",
                             "RMCPP_SHA", "RMCPP_NONE", "SOL_NONE", NULL);
        switch (optionNumber)
        {
        case 0:
            //RMCPP_MD5 - IPMI 2.0
            authType = AUTH_TYPE_RMCPP;
            authAlgo = AUTH_ALGO_RAKP_HMAC_MD5;
            break;

        case 1:
            //RMCPP_SHA
            authType = AUTH_TYPE_RMCPP;
            authAlgo = AUTH_ALGO_RAKP_HMAC_SHA1;
            break;

        case 2:
            //RMCPP_NONE
            authType = AUTH_TYPE_RMCPP;
            authAlgo = AUTH_ALGO_RAKP_NONE;
            break;

        case 3:
			return E_INVALID_VALUE;
            break;

        default:
			LOG_DEBUG<<"ERROR: Invalid Format Option to use for RMCP+ session.";
			LOG_DEBUG<<"Expected Format Options: 'RMCPP_MD5', 'RMCPP_SHA', 'RMCPP_NONE'";

			return E_INVALID_VALUE;

        }
        break;

    case 6:
        {
            if (strlen(value) != 40)
                return E_INVALID_VALUE; //expecting hex string representing 20 bytes
            const char* pVal = value;
            for (int i = 0; i < 20; ++i)
            {
                char cv[3];
                cv[0] = *pVal++;
                cv[1] = *pVal++;
                cv[2] = '\0';
                char* e;
                byte_t nextByte = static_cast<byte_t>(strtoul(cv, &e, 16));
                kG += nextByte;
            }
        }
        break;

    case 7:
        doPing = strcmp(value, "true") == 0;
        break;

	case 8:
		//CipherSuite
		//DCMI Table 4-1 Supported Cipher Suites
		//this could be done better,  but its clear

		optionNumber = match(value, 14, "0","1","2","3","4","5","6",
											"7","8","9","10","11","12","17", NULL);


	switch (optionNumber)
		{
			case 0:
				authType =    AUTH_TYPE_RMCPP;
				authAlgo = AUTH_ALGO_RAKP_NONE;
				integAlgo = INTEG_ALGO_NONE;
				confidAlgo = CONFID_ALGO_NONE;
				break;
			case 1:
				authType =    AUTH_TYPE_RMCPP;
				authAlgo = AUTH_ALGO_RAKP_HMAC_SHA1;
				integAlgo = INTEG_ALGO_NONE;
				confidAlgo = CONFID_ALGO_NONE;
				break;
			case 2:
				authType =    AUTH_TYPE_RMCPP;
				authAlgo = AUTH_ALGO_RAKP_HMAC_SHA1;
				integAlgo = INTEG_ALGO_HMAC_SHA1_96;
				confidAlgo = CONFID_ALGO_NONE;
				break;
			case 3:
				authType =    AUTH_TYPE_RMCPP;
				authAlgo = AUTH_ALGO_RAKP_HMAC_SHA1;
				integAlgo = INTEG_ALGO_HMAC_SHA1_96;
				confidAlgo = CONFID_ALGO_AES_CBC_128;
				break;
			case 4:
				LOG_DEBUG<<"CipherSuite 4 is not a supported option.";
				return E_INVALID_VALUE;
				break;
			case 5:
				LOG_DEBUG<<"CipherSuite 5 is not a supported option.";
				return E_INVALID_VALUE;
				break;
			case 6:
				authType =    AUTH_TYPE_RMCPP;
				authAlgo = AUTH_ALGO_RAKP_HMAC_MD5;
				integAlgo = INTEG_ALGO_NONE;
				confidAlgo = CONFID_ALGO_NONE;
				break;
			case 7:
				authType =    AUTH_TYPE_RMCPP;
				authAlgo = AUTH_ALGO_RAKP_HMAC_MD5;
				integAlgo = INTEG_ALGO_HMAC_MD5_128;
				confidAlgo = CONFID_ALGO_NONE;
				break;
			case 8:
				authType =    AUTH_TYPE_RMCPP;
				authAlgo = AUTH_ALGO_RAKP_HMAC_MD5;
				integAlgo = INTEG_ALGO_HMAC_MD5_128;
				confidAlgo = CONFID_ALGO_AES_CBC_128;
				break;
			case 9:
				LOG_DEBUG<<"CipherSuite 9 is not a supported option.";
				return E_INVALID_VALUE;
				break;
			case 10:
				LOG_DEBUG<<"CipherSuite 10 is not a supported option.";
				return E_INVALID_VALUE;
				break;
			case 11:
				authType =    AUTH_TYPE_RMCPP;
				authAlgo = AUTH_ALGO_RAKP_HMAC_MD5;
				integAlgo = INTEG_ALGO_MD5_128;
				confidAlgo = CONFID_ALGO_NONE;
				break;
			case 12:
				authType =    AUTH_TYPE_RMCPP;
				authAlgo = AUTH_ALGO_RAKP_HMAC_MD5;
				integAlgo = INTEG_ALGO_MD5_128;
				confidAlgo = CONFID_ALGO_AES_CBC_128;
				break;
			case 13:
				authType =    AUTH_TYPE_RMCPP;
				authAlgo = AUTH_ALGO_RAKP_HMAC_SHA256;
				integAlgo = INTEG_ALGO_HMAC_SHA256_128;
				confidAlgo = CONFID_ALGO_AES_CBC_128;
				break;
			default:
				LOG_DEBUG<<"CipherSuite option is Out-Of-Range. "
					"Supported options are '0, 1, 2, 3, 6, 7, 8, 11, 12, 17'.";
				return E_INVALID_VALUE;
		}
	break;
	case 9:
		//SEND_SOL
		if (ipmiSession != NULL)
		{
			ipmiSession->session_send_sol = strcmp(value, "true") == 0;
		}
	break;

	default:
			LOG_DEBUG<<"Invalid ParamName. Please enter valid setting for ParamName.";
        	return E_NAME_UNKNOWN;

    }
    return E_OK;
}

int
RmcppTransportPluginImpl::connect()
    throw()
{
	//Checking for "Empty" in CheckTargetIP() Added here also in case if user calls connect with configure.
	if (ip.empty())
	{
		LOG_ERR << "ERROR: connect(): IP not configured";
		return E_NOT_CONFIGURED;
	}

//#ifdef CIPHER_AUTOCHECK
	if (ipmiSession != NULL)
    {
		//throw logic_error("Connection is already established. ");
		LOG_DEBUG << "Connection is already established. Another session "
			         "can be established only after the current session has been disconnected.";
		return E_OTHER;
    }
//#endif
    try
    {
        LOG_DEBUG << "Connecting to " << ip << " as " << userName;

		if (authType == AUTH_TYPE_RMCPP)//RMCPP session
        {
            RmcppIpmiSession* rmcppSes = new RmcppIpmiSession(ip.c_str(), userName.c_str(), password.c_str(), authAlgo, integAlgo, confidAlgo);
            if (!kG.empty())
            {
                LOG_DEBUG << "Using KG: " << kG;
                rmcppSes->setKg(kG);
            }
            ipmiSession = rmcppSes;
        }
        else
        {
			throw runtime_error("RMCPP: incorrect auth type in response header");
            //ipmiSession = new IolIpmiSession(ip.c_str(), userName.c_str(), password.c_str(), authType);
        }

        ipmiSession->open();
    }
    catch (exception& e)
    {
        LOG_DEBUG << "ERROR: connect(): " << e.what() << endl;
        return E_NETWORK_PROBLEM;
    }
    catch (...)
    {
        return E_OTHER;
    }
    return E_OK;
}


int
RmcppTransportPluginImpl::open_session()
throw()
{
	//Checking for "Empty" in CheckTargetIP() Added here also in case if user calls connect with configure.
	if (ip.empty())
	{
		LOG_ERR << "ERROR: connect(): IP not configured";
		return E_NOT_CONFIGURED;
	}

	if (ipmiSession == NULL)
	{
		//throw logic_error("Connection is already established. ");
		LOG_ERR << "Connection is not already established.";
		return E_NOT_CONNECTED;
	}

    try
    {
       LOG_DEBUG << "Opening  session on: " << ip << " as " << userName;
       ipmiSession->openSession();
    }
    catch (exception& e)
    {
        LOG_DEBUG << "ERROR: opening session: " << e.what() << endl;

        return E_NETWORK_PROBLEM;
    }
    catch (...)
    {
        return E_OTHER;
    }
	return E_OK;
}

int
RmcppTransportPluginImpl::close_session()
throw()
{
	if (ipmiSession == NULL)
	{
		LOG_ERR << "Connection is not already established.";
		return E_NOT_CONNECTED;
	}

	try
	{
		LOG_DEBUG << "Closing session for: " << ip << " as " << userName;
		ipmiSession->closeSession();
	}
	catch (exception& e)
	{
		LOG_DEBUG << "ERROR: closing session: " << e.what() << endl;
		return E_NETWORK_PROBLEM;
	}
	catch (...)
	{
		return E_OTHER;
	}

	return E_OK;
}


int
RmcppTransportPluginImpl::sendIpmiCmd(const dcmi_req_t* req, dcmi_rsp_t* rsp)
    throw()
{
    if (req == NULL)
    {
        LOG_ERR << "ERROR: cannot handle NULL request" << endl;
        return E_OTHER;
    }
    if (rsp == NULL)
    {
        LOG_ERR << "ERROR: cannot copy to NULL response" << endl;
        return E_OTHER;
    }
    try
    {
        ipmiSession->sendRecv(*req, *rsp);
    }
    catch (exception& e)
    {
        LOG_DEBUG << "ERROR: SendIPMICmd: " << e.what() << endl;
        return E_NETWORK_PROBLEM;
    }
    catch (...)
    {
        return E_OTHER;
    }
    return E_OK;
}

int
RmcppTransportPluginImpl::disconnect()
    throw()
{
	//Check for the connection exists. If not return Error.
	if (ipmiSession == NULL)
	{
		LOG_DEBUG<<"Connection has not been established. Unable to Disconnect.";
		return E_NOT_CONNECTED;
	}

    try
    {
        ipmiSession->close();

        delete ipmiSession;
//#ifdef CIPHER_AUTOCHECK
//dtor exception if not nulled
		ipmiSession = NULL;
//#endif
    }
    catch (exception& e)
    {
        LOG_DEBUG << "ERROR: disconnect(): " << e.what() << endl;
	 	  LOG_DEBUG << "Unable to close session, exiting test suite"<<endl;
        delete ipmiSession;
        ipmiSession = NULL;
        return E_NETWORK_PROBLEM;
    }
    catch (...)
    {
    	ipmiSession = NULL;
        return E_OTHER;
    }
    return E_OK;
}

int
RmcppTransportPluginImpl::match(const char* base, int MAX_STRINGS, const char* strings, ...)
{
    const char* compare;
    int stringNumber;
    va_list list;

    //The base string to compare with other strings (quit if NULL)
    if (base == NULL)
        return -1;

    //compare all the other strings (quit when NULL)
    va_start(list, strings);
    compare = (char *)strings;
    stringNumber = 0;
    while ((compare != NULL) && stringNumber < MAX_STRINGS)
    {
        if (strcmp(base, compare) == 0)
        {
            return stringNumber;
        }
        //no luck, try the next one
        compare = va_arg(list, char*);
        stringNumber++;
    }
    return -1;
}

int RmcppTransportPluginImpl::CheckTargetIP(const char* value)
{
	int Int_Value = 0, Ele_Cnt = 0;
	size_t IP_Length = 0;
	char value_Local[IP_MAX_LEN] = {0}; //Maximum IP Address element allowed for IPv4 is 15 i.e xxx.xxx.xxx.xxx
	char IP_Delim[] = "."; //Delimiter "." for splitting the IP address
	char *valuePtr = NULL;


	//Initialize array variables to 0
	//memset ( value_Local, NULL, IP_MAX_LEN*sizeof (char) );

	if(value==NULL)
	{
		LOG_DEBUG<<"ERROR: IP Address cannot be initialised with 'NULL'."
			"Expected format is [0-255].[0-255].[0-255].[0-255]";
		return E_INVALID_VALUE;
	}
	else
	{
		IP_Length = strlen(value); //Warning for conversion from 'size_t' to 'int', possible loss of data.

		//IP Address string is "Empty"
		if(IP_Length == 0)
		{
			LOG_DEBUG<<"ERROR: IP Address cannot be Empty. Expected format "
				"is [0-255].[0-255].[0-255].[0-255]";
			return E_INVALID_VALUE;
		}
		if(IP_Length <= IP_MAX_LEN)//"xxx.xxx.xxx.xxx\0"
		{
			//Copy the parameter value to local array to verify the IP address.
			strncpy(value_Local, value, IP_Length);

			//Find the substring seperated by delimiter "."
  			valuePtr = strtok(value_Local, IP_Delim);

			if(valuePtr == NULL)
			{
				LOG_DEBUG<<"IP Address cannot be parsed. Please enter valid IP Address.";
				return E_INVALID_VALUE;
			}
			else
			{

			while(valuePtr != NULL && Ele_Cnt < 4)
			{
				//Check the IP address if it has any character.
				for(size_t i = 0;i < strlen(valuePtr);i++)
				{
					if (isdigit(valuePtr[i]) == 0)
					{
						LOG_DEBUG<<"ERROR: IP Address cannot contain characters.";
						return E_INVALID_VALUE;
					}
				}

				Int_Value =  atoi(valuePtr);

				Ele_Cnt++; //Counts the number of Octets in IP address
				if(Int_Value < 0  || Int_Value > 255)
				{
					LOG_DEBUG<<"ERROR: IP Address is Out-Of-Range. Expected between "
						"0 - 255 for each octet.";
					return E_INVALID_VALUE;
				}

				valuePtr = strtok(NULL, IP_Delim);
			}

			// Four octets for IP i.e x.x.x.x) and if all four octets are zeroes
			if (valuePtr != NULL )
			{
				LOG_DEBUG<<"ERROR: IP ADdress has more than 4 octets. "
					"Expected IP Address format: [0-255].[0-255].[0-255].[0-255]";
				return E_INVALID_VALUE;
			}
			}

		}
		else
		{
			LOG_DEBUG<<"ERROR: IP Address length is bigger. Expected IP Address - xxx.xxx.xxx.xxx";
			return E_INVALID_VALUE;
		}
        return E_OK;
	}
}

int RmcppTransportPluginImpl::CheckUserName(const char* value)
{
	//Check the username for maximum length of 16 bytes.
	//username for string should be ASCII.

	//Check for the "NULL" username
	if(value == NULL)
	{
		LOG_DEBUG << "ERROR: Username has been initialised with 'NULL'. Please enter valid username.";
		return E_INVALID_VALUE;
	}
	else
	{
		//Check for the length of the username entered for maximum of 16 bytes.
		if ((int)strlen(value) > USERNAME_MAX_LEN)
		{
			//Username is not in valid format
			LOG_DEBUG<<"ERROR: Invalid Username Length. Maximum allowed is 16 characters. ";
		    return E_INVALID_VALUE;
		}
		else
		{
			while(*value != '\0')
			{
				if(*value < 48 || *value > 122) // 'a' and 'Z'
				{
					LOG_DEBUG<<"ERROR: Username has invalid ASCII character.";
					return E_INVALID_VALUE;
				}
				value++;
			}
			//debug - need to remove
			//LOG_DEBUG<<"Valid UserName";
		}

	}
	return E_OK;

}

int RmcppTransportPluginImpl::CheckPassword(const char* value)
{
	//Password should be max of 20 bytes.

	//Password is NULL
	if(value == NULL)
	{
		LOG_DEBUG << "ERROR: Password has been initialised with 'NULL'. Please enter valid password.";
		return E_INVALID_VALUE;
	}
	else
	{
		//Password length greater than 20 bytes.
		if ((int)strlen(value) > PASSWORD_MAX_LEN)//#define
		{
			LOG_DEBUG<<"ERROR: Invalid Password Length. Maximum allowed is 20 characters.";
		    return E_INVALID_VALUE;
		}
	}

	return E_OK;
}

#ifdef SOL

int RmcppTransportPluginImpl::sendSessionPayload(const dcmi_req_t* req)
    throw()
{
    if (req == NULL)
    {
        LOG_ERR << "ERROR: cannot handle NULL request" ;
        return E_OTHER;
    }
    try
    {
		dcmi_rsp_t dummy;
		bool timeoutDummy;
		timeval tvDummy;
		tvDummy.tv_sec = 0;
		tvDummy.tv_usec = 0;
        ipmiSession->sendRecv(tvDummy, true,
					 false, timeoutDummy,
					 *req, dummy);
    }
    catch (exception& e)
    {
        LOG_ERR << "ERROR: Sending: " << e.what() ;
        return E_NETWORK_PROBLEM;
    }
    catch (...)
    {
        return E_OTHER;
    }
    return E_OK;
}

int
RmcppTransportPluginImpl::recvSessionPayload(const timeval timeoutVal, bool *timeout, dcmi_rsp_t* rsp)
    throw()
{
    if (rsp == NULL || timeout == NULL)
    {
        LOG_ERR << "ERROR: cannot copy to NULL response" ;
        return E_OTHER;
    }
    try
    {
		dcmi_req_t dummy;
        ipmiSession->sendRecv(timeoutVal, false,
					 true, *timeout,
					 dummy, *rsp);
    }
    catch (exception& e)
    {
        LOG_ERR << "ERROR: Receiving: " << e.what();
        return E_NETWORK_PROBLEM;
    }
    catch (...)
    {
        return E_OTHER;
    }
    return E_OK;
}
#endif

}
