/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008 INTEL CORPORATION                       **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*******************************************************************************
***		 File		:	TransportPluginLoader.h								  **
***																			  **
********************************************************************************/
#ifndef _TRANSPORT_PLUGIN_LOADER_H
#define _TRANSPORT_PLUGIN_LOADER_H

#include <string>
#include <vector>
#include <map>

#include "TransportPluginDll.h"

/**
 * Transport Plugin Loader responsible for loading transport 
 * plug-in libraries and providing access to their implementations of the 
 * <code>transport_plugin_interface</code>.
 */
class TransportPluginLoader
{
public:
    /**
     * Dtor.
     */
    ~TransportPluginLoader();

    /**
     * Attempts to load specified transport plug-ins (dynamic libraries)
     * and retrieves entrypoints to their implementation of the
     * <code>transport_plugin_interface</code>.
     * 
     * @param pluginIds Array of plug-ins to load.
     *
     * @throws <code>std::runtime_error()</code> If either failed to load
     *      one of the dynamic libraries or failed to obtain entrypoints
     *      to the functions from the plug-in interface.
     */
    void load(const std::vector<PluginId>& pluginIds);

    /**
     * Retrieves a pointer to the specified plug-in interface.
     * 
     * @param id Id of the plug-in. Must be one of the Ids passed to
     *      the <code>load()</code>.
     *
     * @return Pointer to the plug-in interface.
     *
     * @throws <code>std::runtime_error()</code> If no plug-in
     *      with the specified <code>id</code> is loaded.
     */
    transport_plugin_interface* getPluginById(unsigned int id) const;

    /**
     * Retrieves a pointer to the specified plug-in interface.
     * 
     * @param name Name of the plug-in. Must be one of the names passed
     *      to the <code>load()</code>.
     *
     * @return Pointer to the plug-in interface.
     *
     * @throws <code>std::runtime_error()</code> If no plug-in
     *      with the specified <code>name</code> is loaded.
     */
    transport_plugin_interface* getPluginByName(const char* name) const;

protected:

    /**
     * Internal structure that holds all the information about
     * single plug-in loaded by the loader.
     */
    struct TransportPlugin
    {
        /// Id of the plug-in passed to <code>load()</code>.
        unsigned int id;

        /// Name of the plug-in passed to <code>load()</code>.
        std::string name;

        /// Pointer to the plug-in library.
        TransportPluginDll* pDll;

        /// Interace of the plug-in.
        transport_plugin_interface tpi;

        /// Contructs TransportPlugin: loads DLL, gets interace of the plug-in
        TransportPlugin(unsigned int pluginId, const char* pluginName);

        /// Destructor: unloads DLL
        ~TransportPlugin();

    };

    /// Array of loaded plugins.
    std::vector<TransportPlugin*> plugins;

    /// Maps <code>TransportPlugin::id</code> to the pointer
    /// for fast search by <code>getPluginById()</code>.
    std::map<unsigned int, TransportPlugin*> id2plugin;

    /// Maps <code>TransportPlugin::name</code> to the pointer
    /// for fast search by <code>getPluginByName()</code>.
    std::map<std::string, TransportPlugin*> name2plugin;

};

#endif //#ifndef _TRANSPORT_PLUGIN_LOADER_H
